@@ -1,5 +1,5 @@
 /* luafont.c
-   
+
    Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
@@ -17,13 +17,12 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
-#include <ptexlib.h>
-
-#include "nodes.h"
-
 static const char _svn_version[] =
-    "$Id: luafont.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/font/luafont.c $";
+    "$Id: luafont.c 3274 2009-12-20 19:25:34Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/font/luafont.c $";
+
+#include <ptexlib.h>
+#include "lua/luatex-api.h"
 
 #define noVERBOSE
 
@@ -31,6 +30,7 @@ char *font_type_strings[] = { "unknown", "virtual", "real", NULL };
 char *font_format_strings[] =
     { "unknown", "type1", "type3", "truetype", "opentype", NULL };
 char *font_embedding_strings[] = { "unknown", "no", "subset", "full", NULL };
+
 char *ligature_type_strings[] =
     { "=:", "=:|", "|=:", "|=:|", "", "=:|>", "|=:>", "|=:|>", "", "", "",
     "|=:|>>", NULL
@@ -96,6 +96,8 @@ const char *MATH_param_names[] = {
     "RadicalDegreeBottomRaisePercent",
     "MinConnectorOverlap",
     "SubscriptShiftDownWithSuperscript",
+    "FractionDelimiterSize",
+    "FractionDelimiterDisplayStyleSize",
     NULL,
 };
 
@@ -382,7 +384,7 @@ int font_to_lua(lua_State * L, int f)
     int k;
     charinfo *co;
     if (font_cache_id(f) > 0) {
-        /* fetch the table from the registry if  it was 
+        /* fetch the table from the registry if it was
            saved there by font_from_lua() */
         lua_rawgeti(L, LUA_REGISTRYINDEX, font_cache_id(f));
         /* fontdimens can be changed from tex code */
@@ -405,6 +407,10 @@ int font_to_lua(lua_State * L, int f)
         lua_pushstring(L, font_fullname(f));
         lua_setfield(L, -2, "fullname");
     }
+    if (font_psname(f) != NULL) {
+        lua_pushstring(L, font_psname(f));
+        lua_setfield(L, -2, "psname");
+    }
     if (font_encodingname(f) != NULL) {
         lua_pushstring(L, font_encodingname(f));
         lua_setfield(L, -2, "encodingname");
@@ -480,7 +486,9 @@ int font_to_lua(lua_State * L, int f)
         lua_setfield(L, -2, "auto_expand");
     }
     if (pdf_font_attr(f) != 0) {
-        lua_pushstring(L, makecstring(pdf_font_attr(f)));
+        char *s = makecstring(pdf_font_attr(f));
+        lua_pushstring(L, s);
+        free(s);
         lua_setfield(L, -2, "attributes");
     }
 
@@ -503,7 +511,7 @@ int font_to_lua(lua_State * L, int f)
     }
 
     for (k = font_bc(f); k <= font_ec(f); k++) {
-        if (char_exists(f, k)) {
+        if (quick_char_exists(f, k)) {
             lua_pushnumber(L, k);
             co = get_charinfo(f, k);
             font_char_to_lua(L, f, co);
@@ -513,7 +521,7 @@ int font_to_lua(lua_State * L, int f)
     lua_setfield(L, -2, "characters");
 
     if (font_cache_id(f) == 0) {        /* renew */
-        integer r;
+        int r;
         lua_pushvalue(L, -1);
         r = luaL_ref(Luas, LUA_REGISTRYINDEX);  /* pops the table */
         set_font_cache_id(f, r);
@@ -856,13 +864,13 @@ scaled sp_to_dvi(halfword sp, halfword atsize)
 
 
 static void
-read_char_packets(lua_State * L, integer * l_fonts, charinfo * co, int atsize)
+read_char_packets(lua_State * L, int *l_fonts, charinfo * co, int atsize)
 {
     int i, n, m;
     size_t l;
     int cmd;
     char *s;
-    real_eight_bits *cpackets;
+    eight_bits *cpackets;
     int ff = 0;
     int np = 0;
     int max_f = 0;
@@ -1158,8 +1166,8 @@ static void store_math_kerns(lua_State * L, charinfo * co, int id)
 }
 
 void
-font_char_from_lua(lua_State * L, internal_font_number f, integer i,
-                   integer * l_fonts)
+font_char_from_lua(lua_State * L, internal_font_number f, int i,
+                   int *l_fonts, boolean has_math)
 {
     int k, r, t;
     charinfo *co;
@@ -1182,10 +1190,6 @@ font_char_from_lua(lua_State * L, internal_font_number f, integer i,
         set_charinfo_depth(co, j);
         j = n_numeric_field(L, luaS_italic_index, 0);
         set_charinfo_italic(co, j);
-        j = n_numeric_field(L, luaS_top_accent_index, 0);
-        set_charinfo_top_accent(co, j);
-        j = n_numeric_field(L, luaS_bot_accent_index, 0);
-        set_charinfo_bot_accent(co, j);
         j = n_numeric_field(L, luaS_index_index, 0);
         set_charinfo_index(co, j);
         j = n_numeric_field(L, luaS_expansion_factor_index, 0);
@@ -1200,111 +1204,118 @@ font_char_from_lua(lua_State * L, internal_font_number f, integer i,
         set_charinfo_name(co, s);
         s = n_string_field(L, luaS_tounicode_index, NULL);
         set_charinfo_tounicode(co, s);
-        k = n_numeric_field(L, luaS_next_index, -1);
-        if (k >= 0) {
-            set_charinfo_tag(co, list_tag);
-            set_charinfo_remainder(co, k);
-        }
 
-        lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_extensible_index);
-        lua_rawget(L, -2);
-        if (lua_istable(L, -1)) {
-            int top, bot, mid, rep;
-            top = n_numeric_field(L, luaS_top_index, 0);
-            bot = n_numeric_field(L, luaS_bot_index, 0);
-            mid = n_numeric_field(L, luaS_mid_index, 0);
-            rep = n_numeric_field(L, luaS_rep_index, 0);
-            if (top != 0 || bot != 0 || mid != 0 || rep != 0) {
+        if (has_math) {
+            j = n_numeric_field(L, luaS_top_accent_index, 0);
+            set_charinfo_top_accent(co, j);
+            j = n_numeric_field(L, luaS_bot_accent_index, 0);
+            set_charinfo_bot_accent(co, j);
+            k = n_numeric_field(L, luaS_next_index, -1);
+            if (k >= 0) {
+                set_charinfo_tag(co, list_tag);
+                set_charinfo_remainder(co, k);
+            }
+
+            lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_extensible_index);
+            lua_rawget(L, -2);
+            if (lua_istable(L, -1)) {
+                int top, bot, mid, rep;
+                top = n_numeric_field(L, luaS_top_index, 0);
+                bot = n_numeric_field(L, luaS_bot_index, 0);
+                mid = n_numeric_field(L, luaS_mid_index, 0);
+                rep = n_numeric_field(L, luaS_rep_index, 0);
+                if (top != 0 || bot != 0 || mid != 0 || rep != 0) {
+                    set_charinfo_tag(co, ext_tag);
+                    set_charinfo_extensible(co, top, bot, mid, rep);
+                } else {
+                    pdftex_warn
+                        ("lua-loaded font %s char [U+%X] has an invalid extensible field!",
+                         font_name(f), (int) i);
+                }
+            }
+            lua_pop(L, 1);
+
+            lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_horiz_variants_index);
+            lua_rawget(L, -2);
+            if (lua_istable(L, -1)) {
+                int glyph, startconnect, endconnect, advance, extender;
+                extinfo *h;
                 set_charinfo_tag(co, ext_tag);
-                set_charinfo_extensible(co, top, bot, mid, rep);
-            } else {
-                pdftex_warn
-                    ("lua-loaded font %s char [U+%X] has an invalid extensible field!",
-                     font_name(f), (int) i);
-            }
-        }
-        lua_pop(L, 1);
-
-        lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_horiz_variants_index);
-        lua_rawget(L, -2);
-        if (lua_istable(L, -1)) {
-            int glyph, startconnect, endconnect, advance, extender;
-            extinfo *h;
-            set_charinfo_tag(co, ext_tag);
-            set_charinfo_hor_variants(co, NULL);
-            for (k = 1;; k++) {
-                lua_rawgeti(L, -1, k);
-                if (lua_istable(L, -1)) {
-                    glyph = n_numeric_field(L, luaS_glyph_index, 0);
-                    extender = n_numeric_field(L, luaS_extender_index, 0);
-                    startconnect = n_numeric_field(L, luaS_start_index, 0);
-                    endconnect = n_numeric_field(L, luaS_end_index, 0);
-                    advance = n_numeric_field(L, luaS_advance_index, 0);
-                    h = new_variant(glyph, startconnect, endconnect, advance,
-                                    extender);
-                    add_charinfo_hor_variant(co, h);
-                    lua_pop(L, 1);
-                } else {
-                    lua_pop(L, 1);
-                    break;
+                set_charinfo_hor_variants(co, NULL);
+                for (k = 1;; k++) {
+                    lua_rawgeti(L, -1, k);
+                    if (lua_istable(L, -1)) {
+                        glyph = n_numeric_field(L, luaS_glyph_index, 0);
+                        extender = n_numeric_field(L, luaS_extender_index, 0);
+                        startconnect = n_numeric_field(L, luaS_start_index, 0);
+                        endconnect = n_numeric_field(L, luaS_end_index, 0);
+                        advance = n_numeric_field(L, luaS_advance_index, 0);
+                        h = new_variant(glyph, startconnect, endconnect,
+                                        advance, extender);
+                        add_charinfo_hor_variant(co, h);
+                        lua_pop(L, 1);
+                    } else {
+                        lua_pop(L, 1);
+                        break;
+                    }
                 }
             }
-        }
-        lua_pop(L, 1);
+            lua_pop(L, 1);
 
-        lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_vert_variants_index);
-        lua_rawget(L, -2);
-        if (lua_istable(L, -1)) {
-            int glyph, startconnect, endconnect, advance, extender;
-            extinfo *h;
-            set_charinfo_tag(co, ext_tag);
-            set_charinfo_vert_variants(co, NULL);
-            for (k = 1;; k++) {
-                lua_rawgeti(L, -1, k);
-                if (lua_istable(L, -1)) {
-                    glyph = n_numeric_field(L, luaS_glyph_index, 0);
-                    extender = n_numeric_field(L, luaS_extender_index, 0);
-                    startconnect = n_numeric_field(L, luaS_start_index, 0);
-                    endconnect = n_numeric_field(L, luaS_end_index, 0);
-                    advance = n_numeric_field(L, luaS_advance_index, 0);
-                    h = new_variant(glyph, startconnect, endconnect, advance,
-                                    extender);
-                    add_charinfo_vert_variant(co, h);
-                    lua_pop(L, 1);
-                } else {
-                    lua_pop(L, 1);
-                    break;
+            lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_vert_variants_index);
+            lua_rawget(L, -2);
+            if (lua_istable(L, -1)) {
+                int glyph, startconnect, endconnect, advance, extender;
+                extinfo *h;
+                set_charinfo_tag(co, ext_tag);
+                set_charinfo_vert_variants(co, NULL);
+                for (k = 1;; k++) {
+                    lua_rawgeti(L, -1, k);
+                    if (lua_istable(L, -1)) {
+                        glyph = n_numeric_field(L, luaS_glyph_index, 0);
+                        extender = n_numeric_field(L, luaS_extender_index, 0);
+                        startconnect = n_numeric_field(L, luaS_start_index, 0);
+                        endconnect = n_numeric_field(L, luaS_end_index, 0);
+                        advance = n_numeric_field(L, luaS_advance_index, 0);
+                        h = new_variant(glyph, startconnect, endconnect,
+                                        advance, extender);
+                        add_charinfo_vert_variant(co, h);
+                        lua_pop(L, 1);
+                    } else {
+                        lua_pop(L, 1);
+                        break;
+                    }
                 }
             }
-        }
-        lua_pop(L, 1);
+            lua_pop(L, 1);
 
-        /* Here is a complete example:
-           ["mathkern"]={
-           ["bottom_left"] ={ { ["height"]=420, ["kern"]=80  }, { ["height"]=520,  ["kern"]=4   } },
-           ["bottom_right"]={ { ["height"]=0,   ["kern"]=48  } },
-           ["top_left"]    ={ { ["height"]=620, ["kern"]=0   }, { ["height"]=720,  ["kern"]=-80 } },
-           ["top_right"]   ={ { ["height"]=676, ["kern"]=115 }, { ["height"]=776,  ["kern"]=45  } },
-           } 
-         */
-        lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_mathkern_index);
-        lua_rawget(L, -2);
-        if (lua_istable(L, -1)) {
-            lua_getfield(L, -1, "top_left");
-            store_math_kerns(L, co, top_left_kern);
-            lua_pop(L, 1);
-            lua_getfield(L, -1, "top_right");
-            store_math_kerns(L, co, top_right_kern);
-            lua_pop(L, 1);
-            lua_getfield(L, -1, "bottom_right");
-            store_math_kerns(L, co, bottom_right_kern);
-            lua_pop(L, 1);
-            lua_getfield(L, -1, "bottom_left");
-            store_math_kerns(L, co, bottom_left_kern);
+            /* Here is a complete example:
+               ["mathkern"]={
+               ["bottom_left"] ={ { ["height"]=420, ["kern"]=80  }, { ["height"]=520,  ["kern"]=4   } },
+               ["bottom_right"]={ { ["height"]=0,   ["kern"]=48  } },
+               ["top_left"]    ={ { ["height"]=620, ["kern"]=0   }, { ["height"]=720,  ["kern"]=-80 } },
+               ["top_right"]   ={ { ["height"]=676, ["kern"]=115 }, { ["height"]=776,  ["kern"]=45  } },
+               }
+             */
+            lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_mathkern_index);
+            lua_rawget(L, -2);
+            if (lua_istable(L, -1)) {
+                lua_getfield(L, -1, "top_left");
+                store_math_kerns(L, co, top_left_kern);
+                lua_pop(L, 1);
+                lua_getfield(L, -1, "top_right");
+                store_math_kerns(L, co, top_right_kern);
+                lua_pop(L, 1);
+                lua_getfield(L, -1, "bottom_right");
+                store_math_kerns(L, co, bottom_right_kern);
+                lua_pop(L, 1);
+                lua_getfield(L, -1, "bottom_left");
+                store_math_kerns(L, co, bottom_left_kern);
+                lua_pop(L, 1);
+            }
             lua_pop(L, 1);
         }
-        lua_pop(L, 1);
-
+        /* end of |has_math| */
         nk = count_hash_items(L, luaS_index(kerns));
         if (nk > 0) {
             ckerns = xcalloc((nk + 1), sizeof(kerninfo));
@@ -1360,7 +1371,7 @@ font_char_from_lua(lua_State * L, internal_font_number f, integer i,
             lua_pushnil(L);     /* first key */
             if (lua_next(L, -2) != 0) {
                 lua_pop(L, 2);
-                read_char_packets(L, (integer *) l_fonts, co, atsize);
+                read_char_packets(L, (int *) l_fonts, co, atsize);
             }
         }
         lua_pop(L, 1);
@@ -1434,8 +1445,9 @@ int font_from_lua(lua_State * L, int f)
     int bc;                     /* first char index */
     int ec;                     /* last char index */
     char *s;
-    integer *l_fonts = NULL;
-    integer save_ref = 1;       /* unneeded, really */
+    int *l_fonts = NULL;
+    int save_ref = 1;           /* unneeded, really */
+    boolean no_math = false;
 
     /* will we save a cache of the luat table? */
     s = string_field(L, "cache", "yes");
@@ -1468,6 +1480,8 @@ int font_from_lua(lua_State * L, int f)
         pdftex_fail("lua-loaded font [%d] has no name!", f);
         return false;
     }
+    s = string_field(L, "psname", NULL);
+    set_font_psname(f, s);
 
     i = numeric_field(L, "designsize", 655360);
     set_font_dsize(f, i);
@@ -1482,24 +1496,22 @@ int font_from_lua(lua_State * L, int f)
     i = numeric_field(L, "tounicode", 0);
     set_font_tounicode(f, i);
 
-    i = numeric_field(L, "extend", 0);
-    if (i < -2000)
-        i = -2000;
-    if (i > 2000)
-        i = 2000;
-    if (i == 1000)
-        i = 0;
+    i = numeric_field(L, "extend", 1000);
+    if (i < FONT_EXTEND_MIN)
+        i = FONT_EXTEND_MIN;
+    if (i > FONT_EXTEND_MAX)
+        i = FONT_EXTEND_MAX;
     set_font_extend(f, i);
     i = numeric_field(L, "slant", 0);
-    if (i < -1000)
-        i = -1000;
-    if (i > 1000)
-        i = 1000;
+    if (i < FONT_SLANT_MIN)
+        i = FONT_SLANT_MIN;
+    if (i > FONT_SLANT_MAX)
+        i = FONT_SLANT_MAX;
     set_font_slant(f, i);
 
-    i = numeric_field(L, "hyphenchar", get_default_hyphen_char());
+    i = numeric_field(L, "hyphenchar", int_par(default_hyphen_char_code));
     set_hyphen_char(f, i);
-    i = numeric_field(L, "skewchar", get_default_skew_char());
+    i = numeric_field(L, "skewchar", int_par(default_skew_char_code));
     set_skew_char(f, i);
     i = boolean_field(L, "used", 0);
     set_font_used(f, i);
@@ -1525,8 +1537,8 @@ int font_from_lua(lua_State * L, int f)
     /* now fetch the base fonts, if needed */
     n = count_hash_items(L, luaS_index(fonts));
     if (n > 0) {
-        l_fonts = xmalloc((n + 2) * sizeof(integer));
-        memset(l_fonts, 0, (n + 2) * sizeof(integer));
+        l_fonts = xmalloc((n + 2) * sizeof(int));
+        memset(l_fonts, 0, (n + 2) * sizeof(int));
         lua_rawgeti(L, LUA_REGISTRYINDEX, luaS_index(fonts));
         lua_rawget(L, -2);
         for (i = 1; i <= n; i++) {
@@ -1553,8 +1565,8 @@ int font_from_lua(lua_State * L, int f)
                 t = (lua_isnumber(L, -1) ? lua_roundnumber(L, -1) : -1000);
                 lua_pop(L, 1);
 
-                /* TODO: the stack is messed up, otherwise this 
-                 * explicit resizing would not be needed 
+                /* TODO: the stack is messed up, otherwise this
+                 * explicit resizing would not be needed
                  */
                 s_top = lua_gettop(L);
                 if (strcmp(font_name(f), s) == 0)
@@ -1572,7 +1584,7 @@ int font_from_lua(lua_State * L, int f)
         if (font_type(f) == virtual_font_type) {
             pdftex_fail("Invalid local fonts in font %s!\n", font_name(f));
         } else {
-            l_fonts = xmalloc(3 * sizeof(integer));
+            l_fonts = xmalloc(3 * sizeof(int));
             l_fonts[0] = 0;
             l_fonts[1] = f;
             l_fonts[2] = 0;
@@ -1580,14 +1592,19 @@ int font_from_lua(lua_State * L, int f)
     }
 
     /* parameters */
+    no_math = boolean_field(L, "nomath", 0);
     read_lua_parameters(L, f);
-    read_lua_math_parameters(L, f);
+    if (!no_math) {
+        read_lua_math_parameters(L, f);
+    }
     read_lua_cidinfo(L, f);
 
     /* characters */
+
     lua_getfield(L, -1, "characters");
     if (lua_istable(L, -1)) {
         /* find the array size values */
+        int num = 0;            /* number of charinfo's to add */
         ec = 0;
         bc = -1;
         lua_pushnil(L);         /* first key */
@@ -1596,6 +1613,7 @@ int font_from_lua(lua_State * L, int f)
                 i = lua_tointeger(L, -2);
                 if (i >= 0) {
                     if (lua_istable(L, -1)) {
+                        num++;
                         if (i > ec)
                             ec = i;
                         if (bc < 0)
@@ -1607,9 +1625,9 @@ int font_from_lua(lua_State * L, int f)
             }
             lua_pop(L, 1);
         }
-
         if (bc != -1) {
             /* fprintf(stdout,"defined a font at %d with %d-%d\n",f,bc,ec); */
+            font_malloc_charinfo(f, num);
             set_font_bc(f, bc);
             set_font_ec(f, ec);
             lua_pushnil(L);     /* first key */
@@ -1617,21 +1635,23 @@ int font_from_lua(lua_State * L, int f)
                 if (lua_isnumber(L, -2)) {
                     i = lua_tonumber(L, -2);
                     if (i >= 0) {
-                        font_char_from_lua(L, f, i, l_fonts);
+                        font_char_from_lua(L, f, i, l_fonts, !no_math);
                     }
                 } else if (lua_isstring(L, -2)) {
                     s = (char *) lua_tostring(L, -2);
                     if (luaS_ptr_eq(s, left_boundary)) {
-                        font_char_from_lua(L, f, left_boundarychar, l_fonts);
+                        font_char_from_lua(L, f, left_boundarychar, l_fonts,
+                                           !no_math);
                     } else if (luaS_ptr_eq(s, right_boundary)) {
-                        font_char_from_lua(L, f, right_boundarychar, l_fonts);
+                        font_char_from_lua(L, f, right_boundarychar, l_fonts,
+                                           !no_math);
                     }
                 }
                 lua_pop(L, 1);
             }
             lua_pop(L, 1);
 
-            /* handle font expansion last: the |copy_font| routine is called eventually, 
+            /* handle font expansion last: the |copy_font| routine is called eventually,
                and that needs to know |bc| and |ec|. */
             if (font_type(f) != virtual_font_type) {
                 int fstep = numeric_field(L, "step", 0);
