@@ -1,7 +1,7 @@
 /* mapfile.c
-   
+
    Copyright 1996-2006 Han The Thanh <thanh@pdftex.org>
-   Copyright 2006-2008 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -18,27 +18,26 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+static const char _svn_version[] =
+    "$Id: mapfile.c 3282 2009-12-21 21:55:47Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/font/mapfile.c $";
+
 #include <math.h>
 #include "ptexlib.h"
 #include <kpathsea/c-auto.h>
 #include <kpathsea/c-memstr.h>
 #include <string.h>
-#include "luatex-api.h"
-
-static const char _svn_version[] =
-    "$Id: mapfile.c 2331 2009-04-18 16:39:50Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/font/mapfile.c $";
 
 #define FM_BUF_SIZE     1024
 
 static FILE *fm_file;
 
 static unsigned char *fm_buffer = NULL;
-static integer fm_size = 0;
-static integer fm_curbyte = 0;
+static int fm_size = 0;
+static int fm_curbyte = 0;
 
-#define fm_open()       \
-    (fm_file = fopen((char *) nameoffile + 1, FOPEN_RBIN_MODE))
+#define fm_open(a)       \
+    (fm_file = fopen((char *)(a), FOPEN_RBIN_MODE))
 #define fm_read_file()  \
     readbinfile(fm_file,&fm_buffer,&fm_size)
 #define fm_close()      xfclose(fm_file, cur_file_name)
@@ -48,7 +47,6 @@ static integer fm_curbyte = 0;
     (c == 10 || c == '*' || c == '#' || c == ';' || c == '%')
 
 typedef enum { FM_DUPIGNORE, FM_REPLACE, FM_DELETE } updatemode;
-typedef enum { MAPFILE, MAPLINE } maptype;
 
 typedef struct mitem {
     updatemode mode;            /* FM_DUPIGNORE or FM_REPLACE or FM_DELETE */
@@ -87,12 +85,14 @@ fm_entry *new_fm_entry(void)
     fm->encname = NULL;
     fm->type = 0;
     fm->slant = 0;
-    fm->extend = 0;
+    fm->extend = 1000;
     fm->links = 0;
     fm->pid = -1;
     fm->eid = -1;
     fm->subfont = NULL;
-    fm->in_use = false;
+    unset_slantset(fm);
+    unset_extendset(fm);
+    unset_inuse(fm);
     return fm;
 }
 
@@ -121,7 +121,7 @@ static void delete_ff_entry(ff_entry * ff)
     xfree(ff);
 }
 
-static fm_entry *dummy_fm_entry()
+static fm_entry *dummy_fm_entry(void)
 {
     static const fm_entry const_fm_entry;
     return (fm_entry *) & const_fm_entry;
@@ -130,7 +130,6 @@ static fm_entry *dummy_fm_entry()
 /**********************************************************************/
 
 struct avl_table *tfm_tree = NULL;
-struct avl_table *ps_tree = NULL;
 struct avl_table *ff_tree = NULL;
 struct avl_table *encname_tree = NULL;
 
@@ -143,21 +142,6 @@ static int comp_fm_entry_tfm(const void *pa, const void *pb, void *p)
                   ((const fm_entry *) pb)->tfm_name);
 }
 
-/* AVL sort fm_entry into ps_tree by ps_name, slant, and extend */
-
-static int comp_fm_entry_ps(const void *pa, const void *pb, void *p)
-{
-    int i;
-    const fm_entry *p1 = (const fm_entry *) pa, *p2 = (const fm_entry *) pb;
-    (void) p;
-    assert(p1->ps_name != NULL && p2->ps_name != NULL);
-    if ((i = strcmp(p1->ps_name, p2->ps_name)))
-        return i;
-    cmp_return(p1->slant, p2->slant);
-    cmp_return(p1->extend, p2->extend);
-    return 0;
-}
-
 /* AVL sort ff_entry into ff_tree by ff_name */
 
 static int comp_ff_entry(const void *pa, const void *pb, void *p)
@@ -167,14 +151,11 @@ static int comp_ff_entry(const void *pa, const void *pb, void *p)
                   ((const ff_entry *) pb)->ff_name);
 }
 
-static void create_avl_trees()
+static void create_avl_trees(void)
 {
     assert(tfm_tree == NULL);
     tfm_tree = avl_create(comp_fm_entry_tfm, NULL, &avl_xallocator);
     assert(tfm_tree != NULL);
-    assert(ps_tree == NULL);
-    ps_tree = avl_create(comp_fm_entry_ps, NULL, &avl_xallocator);
-    assert(ps_tree != NULL);
     assert(ff_tree == NULL);
     ff_tree = avl_create(comp_ff_entry, NULL, &avl_xallocator);
     assert(ff_tree != NULL);
@@ -183,21 +164,11 @@ static void create_avl_trees()
     assert(encname_tree != NULL);
 }
 
-/*
-The function avl_do_entry() is not completely symmetrical with regards
-to tfm_name and ps_name handling, e. g. a duplicate tfm_name gives a
-"goto exit", and no ps_name link is tried. This is to keep it compatible
-with the original version.
-*/
-
 int avl_do_entry(fm_entry * fm, int mode)
 {
     fm_entry *p;
     void *a;
     void **aa;
-
-    /* handle tfm_name link */
-
     if (strcmp(fm->tfm_name, nontfm)) {
         p = (fm_entry *) avl_find(tfm_tree, fm);
         if (p != NULL) {
@@ -210,7 +181,7 @@ int avl_do_entry(fm_entry * fm, int mode)
                 break;
             case FM_REPLACE:
             case FM_DELETE:
-                if (p->in_use) {
+                if (is_inuse(p)) {
                     pdftex_warn
                         ("fontmap entry for `%s' has been used, replace/delete not allowed",
                          fm->tfm_name);
@@ -219,8 +190,7 @@ int avl_do_entry(fm_entry * fm, int mode)
                 a = avl_delete(tfm_tree, p);
                 assert(a != NULL);
                 unset_tfmlink(p);
-                if (!has_pslink(p))
-                    delete_fm_entry(p);
+                delete_fm_entry(p);
                 break;
             default:
                 assert(0);
@@ -232,38 +202,8 @@ int avl_do_entry(fm_entry * fm, int mode)
             set_tfmlink(fm);
         }
     }
-
-    /* handle ps_name link */
-
-    if (fm->ps_name != NULL) {
-        p = (fm_entry *) avl_find(ps_tree, fm);
-        if (p != NULL) {
-            switch (mode) {
-            case FM_DUPIGNORE:
-                goto exit;
-                break;
-            case FM_REPLACE:
-            case FM_DELETE:
-                if (p->in_use)
-                    goto exit;
-                a = avl_delete(ps_tree, p);
-                assert(a != NULL);
-                unset_pslink(p);
-                if (!has_tfmlink(p))
-                    delete_fm_entry(p);
-                break;
-            default:
-                assert(0);
-            }
-        }
-        if (mode != FM_DELETE && is_t1fontfile(fm) && is_included(fm)) {
-            aa = avl_probe(ps_tree, fm);
-            assert(aa != NULL);
-            set_pslink(fm);
-        }
-    }
   exit:
-    if (!has_tfmlink(fm) && !has_pslink(fm))    /* e. g. after FM_DELETE */
+    if (!has_tfmlink(fm))       /* e. g. after FM_DELETE */
         return 1;               /* deallocation of fm_entry structure required */
     else
         return 0;
@@ -320,25 +260,15 @@ int check_fm_entry(fm_entry * fm, boolean warn)
         a += 2;
     }
 
-    /* SlantFont and ExtendFont can be used only with Type1 fonts */
-    if ((fm->slant != 0 || fm->extend != 0)
-        && !(is_t1fontfile(fm) && is_included(fm))) {
-        if (warn)
-            pdftex_warn
-                ("invalid entry for `%s': SlantFont/ExtendFont can be used only with embedded Type1 fonts",
-                 fm->tfm_name);
-        a += 4;
-    }
-
     /* the value of SlantFont and ExtendFont must be reasonable */
-    if (abs(fm->slant) > 1000) {
+    if (fm->slant < FONT_SLANT_MIN || fm->slant > FONT_SLANT_MAX) {
         if (warn)
             pdftex_warn
                 ("invalid entry for `%s': too big value of SlantFont (%g)",
                  fm->tfm_name, fm->slant / 1000.0);
         a += 8;
     }
-    if (abs(fm->extend) > 2000) {
+    if (fm->extend < FONT_EXTEND_MIN || fm->extend > FONT_EXTEND_MAX) {
         if (warn)
             pdftex_warn
                 ("invalid entry for `%s': too big value of ExtendFont (%g)",
@@ -414,7 +344,7 @@ int check_std_t1font(char *s)
 
 /**********************************************************************/
 
-static void fm_scan_line()
+static void fm_scan_line(void)
 {
     int a, b, c, j, u = 0, v = 0;
     float d;
@@ -477,13 +407,13 @@ static void fm_scan_line()
                         s--;    /* e. g. 0.5ExtendFont: %f = 0.5E */
                     if (str_prefix(s, "SlantFont")) {
                         d *= 1000.0;    /* correct rounding also for neg. numbers */
-                        fm->slant = (integer) (d > 0 ? d + 0.5 : d - 0.5);
+                        fm->slant = (int) (d > 0 ? d + 0.5 : d - 0.5);
+                        set_slantset(fm);
                         r = s + strlen("SlantFont");
                     } else if (str_prefix(s, "ExtendFont")) {
                         d *= 1000.0;
-                        fm->extend = (integer) (d > 0 ? d + 0.5 : d - 0.5);
-                        if (fm->extend == 1000)
-                            fm->extend = 0;
+                        fm->extend = (int) (d > 0 ? d + 0.5 : d - 0.5);
+                        set_extendset(fm);
                         r = s + strlen("ExtendFont");
                     } else {    /* unknown name */
                         for (r = s; *r != ' ' && *r != '"' && *r != '\0'; r++); /* jump over name */
@@ -552,6 +482,8 @@ static void fm_scan_line()
     if (is_fontfile(fm) && strlen(fm_fontfile(fm)) > 3) {
         if (strcasecmp(strend(fm_fontfile(fm)) - 4, ".ttf") == 0)
             set_truetype(fm);
+        else if (strcasecmp(strend(fm_fontfile(fm)) - 4, ".ttc") == 0)
+            set_truetype(fm);
         else if (strcasecmp(strend(fm_fontfile(fm)) - 4, ".otf") == 0)
             set_opentype(fm);
         else
@@ -575,11 +507,11 @@ static void fm_scan_line()
 
 /**********************************************************************/
 
-void fm_read_info()
+void fm_read_info(void)
 {
     int callback_id;
     int file_opened = 0;
-    char *ftemp = NULL;
+
     if (tfm_tree == NULL)
         create_avl_trees();
     if (mitem->line == NULL)    /* nothing to do */
@@ -587,75 +519,52 @@ void fm_read_info()
     mitem->lineno = 1;
     switch (mitem->type) {
     case MAPFILE:
-        set_cur_file_name(mitem->line);
         if (fm_buffer != NULL) {
             xfree(fm_buffer);
             fm_buffer = NULL;
         }
         fm_curbyte = 0;
         fm_size = 0;
-        callback_id = callback_defined(find_map_file_callback);
-        if (callback_id > 0) {
-            if (run_callback
-                (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-                if (ftemp != NULL && strlen(ftemp)) {
-                    free(nameoffile);
-                    namelength = strlen(ftemp);
-                    nameoffile = xmalloc(namelength + 2);
-                    strcpy((char *) (nameoffile + 1), ftemp);
-                    free(ftemp);
-                }
-            }
-        } else {
-            ftemp =
-                kpse_find_file((char *) (nameoffile + 1), kpse_fontmap_format,
-                               0);
-            if (ftemp != NULL) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            }
-        }
-        callback_id = callback_defined(read_map_file_callback);
-        if (callback_id > 0) {
-            if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
-                             &file_opened, &fm_buffer, &fm_size)) {
-                if (file_opened) {
-                    if (fm_size > 0) {
-                        cur_file_name = (char *) nameoffile + 1;
-                        if (tracefilenames)
-                            tex_printf("{%s", cur_file_name);
-                        while (!fm_eof()) {
-                            fm_scan_line();
-                            mitem->lineno++;
+        cur_file_name = luatex_find_file(mitem->line, find_map_file_callback);
+        if (cur_file_name) {
+            callback_id = callback_defined(read_map_file_callback);
+            if (callback_id > 0) {
+                if (run_callback(callback_id, "S->bSd", cur_file_name,
+                                 &file_opened, &fm_buffer, &fm_size)) {
+                    if (file_opened) {
+                        if (fm_size > 0) {
+                            if (tracefilenames)
+                                tex_printf("{%s", cur_file_name);
+                            while (!fm_eof()) {
+                                fm_scan_line();
+                                mitem->lineno++;
+                            }
+                            if (tracefilenames)
+                                tex_printf("}");
+                            fm_file = NULL;
                         }
-                        if (tracefilenames)
-                            tex_printf("}");
-                        fm_file = NULL;
+                    } else {
+                        pdftex_warn("cannot open font map file");
                     }
                 } else {
                     pdftex_warn("cannot open font map file");
                 }
             } else {
-                pdftex_warn("cannot open font map file");
-            }
-        } else {
-            if (!fm_open()) {
-                pdftex_warn("cannot open font map file");
-            } else {
-                fm_read_file();
-                cur_file_name = (char *) nameoffile + 1;
-                tex_printf("{%s", cur_file_name);
-                while (!fm_eof()) {
-                    fm_scan_line();
-                    mitem->lineno++;
+                if (!fm_open(cur_file_name)) {
+                    pdftex_warn("cannot open font map file");
+                } else {
+                    fm_read_file();
+                    tex_printf("{%s", cur_file_name);
+                    while (!fm_eof()) {
+                        fm_scan_line();
+                        mitem->lineno++;
+                    }
+                    fm_close();
+                    tex_printf("}");
+                    fm_file = NULL;
                 }
-                fm_close();
-                tex_printf("}");
-                fm_file = NULL;
             }
+            cur_file_name = NULL;
         }
         break;
     case MAPLINE:
@@ -680,13 +589,15 @@ static fm_entry_ptr fmlookup(internalfontnumber f)
     if (tfm_tree == NULL)
         fm_read_info();         /* only to read default map file */
     tfm = font_name(f);
+    if (tfm == NULL)            /* wide, lua loaded fonts may not have a name */
+        return (fm_entry_ptr) dummy_fm_entry();
     assert(strcmp(tfm, nontfm));
 
-    /* Look up for full <tfmname>[+-]<expand> */
+    /* Look up for tfmname */
     tmp.tfm_name = tfm;
     fm = (fm_entry *) avl_find(tfm_tree, &tmp);
     if (fm != NULL) {
-        fm->in_use = true;
+        set_inuse(fm);
         return (fm_entry_ptr) fm;
     }
     return (fm_entry_ptr) dummy_fm_entry();
@@ -697,122 +608,14 @@ boolean hasfmentry(internalfontnumber f)
     if (font_map(f) == NULL)
         set_font_map(f, (fm_entry_ptr) fmlookup(f));
     assert(font_map(f) != NULL);
+    /* TODO: this still overrides already set font_slant(f) */
+    if (is_slantset((fm_entry *) font_map(f)))
+        font_slant(f) = ((fm_entry *) font_map(f))->slant;
+    if (is_extendset((fm_entry *) font_map(f)))
+        font_extend(f) = ((fm_entry *) font_map(f))->extend;
     return font_map(f) != (fm_entry_ptr) dummy_fm_entry();
 }
 
-/* check whether a map entry is valid for font replacement */
-
-static boolean fm_valid_for_font_replacement(fm_entry * fm)
-{
-    ff_entry *ff;
-
-    assert(fm != NULL);
-    assert(is_fontfile(fm));    /* ps_tree should contain only entries with font file */
-    assert(is_type1(fm));       /* ps_tree should contain only Type1 entries */
-
-    ff = check_ff_exist(fm->ff_name, false);
-    assert(ff != NULL);
-    if (ff->ff_path == NULL)    /* ...there is no font file available */
-        return false;
-    return true;                /* all tests passed */
-}
-
-/**********************************************************************/
-/*
- * lookup fontmap by ps_name;
- * used for Type1 font replacement when embedding of PDF files
- */
-
-fm_entry *lookup_fontmap(char *ps_name)
-{
-    fm_entry *fm, *fm2, tmp;
-    char *a, *b, *c, *d, *e, *s;
-    int i, sl, ex;
-    struct avl_traverser t, t2;
-    if (tfm_tree == NULL)
-        fm_read_info();         /* only to read default map file */
-    assert(ps_name != NULL);
-    s = ps_name;
-    if (strlen(ps_name) > 7) {  /* check for subsetted name tag */
-        for (i = 0; i < 6; i++, s++)
-            if (*s < 'A' || *s > 'Z')
-                break;
-        if (i == 6 && *s == '+')
-            s++;                /* if name tag found, skip behind it */
-        else
-            s = ps_name;
-    }
-
-    /*
-     * Scan -Slant_<slant> and -Extend_<extend> font name extensions;
-     * three valid formats:
-     * <fontname>-Slant_<slant>
-     * <fontname>-Slant_<slant>-Extend_<extend>
-     * <fontname>-Extend_<extend>
-     * Slant entry must come _before_ Extend entry
-     */
-
-    tmp.slant = 0;
-    tmp.extend = 0;
-    if ((a = strstr(s, "-Slant_")) != NULL) {
-        b = a + strlen("-Slant_");
-        sl = (int) strtol(b, &e, 10);
-        if ((e != b) && (e == strend(b))) {
-            tmp.slant = sl;
-            *a = '\0';          /* ps_name string ends before "-Slant_" */
-        } else {
-            if (e != b) {       /* only if <slant> is valid number */
-                if ((c = strstr(e, "-Extend_")) != NULL) {
-                    d = c + strlen("-Extend_");
-                    ex = (int) strtol(d, &e, 10);
-                    if ((e != d) && (e == strend(d))) {
-                        tmp.slant = sl;
-                        tmp.extend = ex;
-                        *a = '\0';      /* ps_name string ends before "-Slant_" */
-                    }
-                }
-            }
-        }
-    } else {
-        if ((a = strstr(s, "-Extend_")) != NULL) {
-            b = a + strlen("-Extend_");
-            ex = (int) strtol(b, &e, 10);
-            if ((e != b) && (e == strend(b))) {
-                tmp.extend = ex;
-                *a = '\0';      /* ps_name string ends before "-Extend_" */
-            }
-        }
-    }
-    tmp.ps_name = s;
-
-    fm = (fm_entry *) avl_t_find(&t, ps_tree, &tmp);
-    if (fm == NULL)
-        return NULL;            /* no entry found */
-
-    /* at this point we know there is at least one fm_entry with given ps_name;
-     * we test all such entries and return the first one that is valid for font
-     * replacement */
-
-    t2 = t;
-    fm2 = (fm_entry *) avl_t_prev(&t2);
-
-    /* search forward */
-    do {
-        if (fm_valid_for_font_replacement(fm))
-            return fm;
-        fm = (fm_entry *) avl_t_next(&t);
-    } while (fm != NULL && comp_fm_entry_ps(fm, &tmp, NULL) == 0);
-
-    /* search backward */
-    while (fm2 != NULL && comp_fm_entry_ps(fm2, &tmp, NULL) == 0) {
-        if (fm_valid_for_font_replacement(fm2))
-            return fm2;
-        fm2 = (fm_entry *) avl_t_prev(&t2);
-    }
-
-    return NULL;
-}
-
 /**********************************************************************/
 /*
  * Process map file given by its name or map line contents. Items not
@@ -868,16 +671,18 @@ void process_map_item(char *s, int type)
     }
 }
 
-void pdfmapfile(integer t)
+void pdfmapfile(int t)
 {
-    process_map_item(makecstring(tokens_to_string(t)), MAPFILE);
-    flush_str(last_tokens_string);
+    char *s = tokenlist_to_cstring(t, true, NULL);
+    process_map_item(s, MAPFILE);
+    free(s);
 }
 
-void pdfmapline(integer t)
+void pdfmapline(int t)
 {
-    process_map_item(makecstring(tokens_to_string(t)), MAPLINE);
-    flush_str(last_tokens_string);
+    char *s = tokenlist_to_cstring(t, true, NULL);
+    process_map_item(s, MAPLINE);
+    free(s);
 }
 
 void pdf_init_map_file(string map_name)
@@ -898,7 +703,7 @@ void pdf_init_map_file(string map_name)
  * same fonts are to be embedded.
  *
  * The ff_tree contains only font files, which are actually needed,
- * so this tree typically is much smaller than the tfm_tree or ps_tree.
+ * so this tree typically is much smaller than the tfm_tree.
  */
 
 ff_entry *check_ff_exist(char *ff_name, boolean is_tt)
@@ -958,21 +763,7 @@ static void destroy_fm_entry_tfm(void *pa, void *pb)
     fm_entry *fm;
     (void) pb;
     fm = (fm_entry *) pa;
-    if (!has_pslink(fm))
-        delete_fm_entry(fm);
-    else
-        unset_tfmlink(fm);
-}
-
-static void destroy_fm_entry_ps(void *pa, void *pb)
-{
-    fm_entry *fm;
-    (void) pb;
-    fm = (fm_entry *) pa;
-    if (!has_tfmlink(fm))
-        delete_fm_entry(fm);
-    else
-        unset_pslink(fm);
+    delete_fm_entry(fm);
 }
 
 static void destroy_ff_entry(void *pa, void *pb)
@@ -989,10 +780,6 @@ void fm_free(void)
         avl_destroy(tfm_tree, destroy_fm_entry_tfm);
         tfm_tree = NULL;
     }
-    if (ps_tree != NULL) {
-        avl_destroy(ps_tree, destroy_fm_entry_ps);
-        ps_tree = NULL;
-    }
     if (ff_tree != NULL) {
         avl_destroy(ff_tree, destroy_ff_entry);
         ff_tree = NULL;
