@@ -17,6 +17,10 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+static const char _svn_version[] =
+    "$Id: texfont.c 3272 2009-12-20 18:58:08Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/font/texfont.c $";
+
 /* Main font API implementation for the pascal parts */
 
 /* stuff to watch out for: 
@@ -32,21 +36,18 @@
  */
 
 #include "ptexlib.h"
-#include "luatex-api.h"
+#include "lua/luatex-api.h"
 
 #define noDEBUG
 
-static const char _svn_version[] =
-    "$Id: texfont.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/font/texfont.c $";
-
 #define proper_char_index(c) (c<=font_ec(f) && c>=font_bc(f))
 #define dxfree(a,b) { xfree(a); a = b ; }
 #define do_realloc(a,b,d)    a = xrealloc(a,(b)*sizeof(d))
 
 texfont **font_tables = NULL;
 
-static integer font_arr_max = 0;
-static integer font_id_maxval = 0;
+static int font_arr_max = 0;
+static int font_id_maxval = 0;
 
 extern extinfo *get_charinfo_vert_variants(charinfo * ci);
 extern extinfo *get_charinfo_hor_variants(charinfo * ci);
@@ -56,7 +57,7 @@ extern void set_charinfo_vert_variants(charinfo * ci, extinfo * ext);
 extern extinfo *copy_variants(extinfo * o);
 
 
-static void grow_font_table(integer id)
+static void grow_font_table(int id)
 {
     int j;
     if (id >= font_arr_max) {
@@ -70,7 +71,7 @@ static void grow_font_table(integer id)
     }
 }
 
-integer new_font_id(void)
+int new_font_id(void)
 {
     int i;
     for (i = 0; i < font_arr_max; i++) {
@@ -85,17 +86,17 @@ integer new_font_id(void)
     return i;
 }
 
-integer max_font_id(void)
+int max_font_id(void)
 {
     return font_id_maxval;
 }
 
-void set_max_font_id(integer i)
+void set_max_font_id(int i)
 {
     font_id_maxval = i;
 }
 
-integer new_font(void)
+int new_font(void)
 {
     int k;
     int id;
@@ -108,6 +109,7 @@ integer new_font(void)
     font_tables[id]->_font_area = NULL;
     font_tables[id]->_font_filename = NULL;
     font_tables[id]->_font_fullname = NULL;
+    font_tables[id]->_font_psname = NULL;
     font_tables[id]->_font_encodingname = NULL;
     font_tables[id]->_font_cidregistry = NULL;
     font_tables[id]->_font_cidordering = NULL;
@@ -119,6 +121,8 @@ integer new_font(void)
     set_font_bc(id, 1);         /* ec = 0 */
     set_hyphen_char(id, '-');
     set_skew_char(id, -1);
+    font_slant(id) = 0;         /* vertical */
+    font_extend(id) = 1000;     /* normal width */
 
     /* allocate eight values including 0 */
     set_font_params(id, 7);
@@ -137,9 +141,18 @@ integer new_font(void)
     return id;
 }
 
+void font_malloc_charinfo(internal_font_number f, int num)
+{
+    int glyph = font_tables[f]->charinfo_size;
+    font_bytes += (num * sizeof(charinfo));
+    do_realloc(font_tables[f]->charinfo, (glyph + num), charinfo);
+    memset(&(font_tables[f]->charinfo[glyph]), 0, (num * sizeof(charinfo)));
+    font_tables[f]->charinfo_size += num;
+}
+
 #define find_charinfo_id(f,c) get_sa_item(font_tables[f]->characters,c)
 
-charinfo *get_charinfo(internal_font_number f, integer c)
+charinfo *get_charinfo(internal_font_number f, int c)
 {
     sa_tree_item glyph;
     charinfo *ci;
@@ -149,11 +162,7 @@ charinfo *get_charinfo(internal_font_number f, integer c)
 
             glyph = ++font_tables[f]->charinfo_count;
             if (glyph >= (unsigned) font_tables[f]->charinfo_size) {
-                font_bytes += (16 * sizeof(charinfo));
-                do_realloc(font_tables[f]->charinfo, (glyph + 16), charinfo);
-                memset(&(font_tables[f]->charinfo[glyph]), 0,
-                       (16 * sizeof(charinfo)));
-                font_tables[f]->charinfo_size += 16;
+                font_malloc_charinfo(f, 256);
             }
             font_tables[f]->charinfo[glyph].ef = 1000;  /* init */
             set_sa_item(font_tables[f]->characters, c, glyph, 1);       /* 1= global */
@@ -177,7 +186,7 @@ charinfo *get_charinfo(internal_font_number f, integer c)
     return &(font_tables[f]->charinfo[0]);
 }
 
-void set_charinfo(internal_font_number f, integer c, charinfo * ci)
+void set_charinfo(internal_font_number f, int c, charinfo * ci)
 {
     sa_tree_item glyph;
     if (proper_char_index(c)) {
@@ -198,10 +207,10 @@ void set_charinfo(internal_font_number f, integer c, charinfo * ci)
 
 charinfo *copy_charinfo(charinfo * ci)
 {
-    int x;
+    int x, k;
     kerninfo *kern;
     liginfo *lig;
-    real_eight_bits *packet;
+    eight_bits *packet;
     charinfo *co = NULL;
     if (ci == NULL)
         return NULL;
@@ -258,7 +267,6 @@ charinfo *copy_charinfo(charinfo * ci)
         set_charinfo_hor_variants(co,
                                   copy_variants(get_charinfo_hor_variants(ci)));
     }
-    /* TODO: copy mathkerns */
     x = ci->top_left_math_kerns;
     co->top_left_math_kerns = x;
     if (x > 0) {
@@ -308,7 +316,7 @@ charinfo *copy_charinfo(charinfo * ci)
     return co;
 }
 
-charinfo *char_info(internal_font_number f, integer c)
+charinfo *char_info(internal_font_number f, int c)
 {
     if (f > font_id_maxval)
         return 0;
@@ -323,18 +331,18 @@ charinfo *char_info(internal_font_number f, integer c)
     return &(font_tables[f]->charinfo[0]);
 }
 
-charinfo_short char_info_short(internal_font_number f, integer c)
+scaled_whd get_charinfo_whd(internal_font_number f, int c)
 {
-    charinfo_short s;
+    scaled_whd s;
     charinfo *i;
     i = char_info(f, c);
-    s.ci_wd = i->width;
-    s.ci_dp = i->depth;
-    s.ci_ht = i->height;
+    s.wd = i->width;
+    s.dp = i->depth;
+    s.ht = i->height;
     return s;
 }
 
-integer char_exists(internal_font_number f, integer c)
+int char_exists(internal_font_number f, int c)
 {
     if (f > font_id_maxval)
         return 0;
@@ -348,9 +356,9 @@ integer char_exists(internal_font_number f, integer c)
     return 0;
 }
 
-int lua_char_exists_callback(internal_font_number f, integer c)
+int lua_char_exists_callback(internal_font_number f, int c)
 {
-    integer callback_id;
+    int callback_id;
     lua_State *L = Luas;
     int ret = 0;
     callback_id = callback_defined(char_exists_callback);
@@ -414,7 +422,7 @@ void dump_variant(extinfo * ext)
 
 extinfo *undump_variant(void)
 {
-    integer x;
+    int x;
     extinfo *ext;
     undump_int(x);
     if (x == 0)
@@ -551,10 +559,12 @@ void set_charinfo_index(charinfo * ci, scaled val)
 {
     ci->index = val;
 }
+
 void set_charinfo_name(charinfo * ci, char *val)
 {
     dxfree(ci->name, val);
 }
+
 void set_charinfo_tounicode(charinfo * ci, char *val)
 {
     dxfree(ci->tounicode, val);
@@ -570,7 +580,7 @@ void set_charinfo_kerns(charinfo * ci, kerninfo * val)
     dxfree(ci->kerns, val);
 }
 
-void set_charinfo_packets(charinfo * ci, real_eight_bits * val)
+void set_charinfo_packets(charinfo * ci, eight_bits * val)
 {
     dxfree(ci->packets, val);
 }
@@ -632,7 +642,7 @@ int get_charinfo_math_kerns(charinfo * ci, int id)
     } else if (id == top_right_kern) {
         k = ci->top_right_math_kerns;
     } else {
-        tconfusion("get_charinfo_math_kerns");
+        confusion("get_charinfo_math_kerns");
     }
     return k;
 }
@@ -668,14 +678,14 @@ void add_charinfo_math_kern(charinfo * ci, int id, scaled ht, scaled krn)
         ci->top_right_math_kern_array[(2 * (k)) + 1] = krn;
         ci->top_right_math_kerns++;
     } else {
-        tconfusion("add_charinfo_math_kern");
+        confusion("add_charinfo_math_kern");
     }
 }
 
 
 static void dump_math_kerns(charinfo * ci)
 {
-    integer k, l;
+    int k, l;
     l = ci->top_left_math_kerns;
     dump_int(l);
     for (k = 0; k < l; k++) {
@@ -705,7 +715,7 @@ static void dump_math_kerns(charinfo * ci)
 static void undump_math_kerns(charinfo * ci)
 {
     int k;
-    integer x;
+    int x;
     undump_int(x);
     ci->top_left_math_kerns = (int) x;
     if (x > 0)
@@ -826,7 +836,7 @@ char get_charinfo_tag(charinfo * ci)
     return ci->tag;
 }
 
-integer get_charinfo_remainder(charinfo * ci)
+int get_charinfo_remainder(charinfo * ci)
 {
     return ci->remainder;
 }
@@ -836,7 +846,7 @@ char get_charinfo_used(charinfo * ci)
     return ci->used;
 }
 
-integer get_charinfo_index(charinfo * ci)
+int get_charinfo_index(charinfo * ci)
 {
     return ci->index;
 }
@@ -861,22 +871,22 @@ kerninfo *get_charinfo_kerns(charinfo * ci)
     return ci->kerns;
 }
 
-real_eight_bits *get_charinfo_packets(charinfo * ci)
+eight_bits *get_charinfo_packets(charinfo * ci)
 {
     return ci->packets;
 }
 
-integer get_charinfo_ef(charinfo * ci)
+int get_charinfo_ef(charinfo * ci)
 {
     return ci->ef;
 }
 
-integer get_charinfo_rp(charinfo * ci)
+int get_charinfo_rp(charinfo * ci)
 {
     return ci->rp;
 }
 
-integer get_charinfo_lp(charinfo * ci)
+int get_charinfo_lp(charinfo * ci)
 {
     return ci->lp;
 }
@@ -898,7 +908,7 @@ extinfo *get_charinfo_hor_variants(charinfo * ci)
 }
 
 
-scaled char_width(internal_font_number f, integer c)
+scaled char_width(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     scaled w = get_charinfo_width(ci);
@@ -906,7 +916,7 @@ scaled char_width(internal_font_number f, integer c)
     return w;
 }
 
-scaled char_depth(internal_font_number f, integer c)
+scaled char_depth(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     scaled w = get_charinfo_depth(ci);
@@ -914,7 +924,7 @@ scaled char_depth(internal_font_number f, integer c)
     return w;
 }
 
-scaled char_height(internal_font_number f, integer c)
+scaled char_height(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     scaled w = get_charinfo_height(ci);
@@ -922,68 +932,68 @@ scaled char_height(internal_font_number f, integer c)
     return w;
 }
 
-scaled char_italic(internal_font_number f, integer c)
+scaled char_italic(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_italic(ci);
 }
 
-scaled char_top_accent(internal_font_number f, integer c)
+scaled char_top_accent(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_top_accent(ci);
 }
 
-scaled char_bot_accent(internal_font_number f, integer c)
+scaled char_bot_accent(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_bot_accent(ci);
 }
 
 
-integer char_remainder(internal_font_number f, integer c)
+int char_remainder(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_remainder(ci);
 }
 
-char char_tag(internal_font_number f, integer c)
+char char_tag(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_tag(ci);
 }
 
-char char_used(internal_font_number f, integer c)
+char char_used(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_used(ci);
 }
 
-char *char_name(internal_font_number f, integer c)
+char *char_name(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_name(ci);
 }
 
-integer char_index(internal_font_number f, integer c)
+int char_index(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_index(ci);
 }
 
-liginfo *char_ligatures(internal_font_number f, integer c)
+liginfo *char_ligatures(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_ligatures(ci);
 }
 
-kerninfo *char_kerns(internal_font_number f, integer c)
+kerninfo *char_kerns(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_kerns(ci);
 }
 
-real_eight_bits *char_packets(internal_font_number f, integer c)
+eight_bits *char_packets(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_packets(ci);
@@ -996,7 +1006,7 @@ void set_font_params(internal_font_number f, int b)
     i = font_params(f);
     if (i != b) {
         font_bytes += (b - font_params(f) + 1) * sizeof(scaled);
-        do_realloc(param_base(f), (b + 2), integer);
+        do_realloc(param_base(f), (b + 2), int);
         font_params(f) = b;
         if (b > i) {
             while (i < b) {
@@ -1013,7 +1023,7 @@ void set_font_math_params(internal_font_number f, int b)
     i = font_math_params(f);
     if (i != b) {
         font_bytes += (b - font_math_params(f) + 1) * sizeof(scaled);
-        do_realloc(math_param_base(f), (b + 2), integer);
+        do_realloc(math_param_base(f), (b + 2), int);
         font_math_params(f) = b;
         if (b > i) {
             while (i < b) {
@@ -1026,13 +1036,23 @@ void set_font_math_params(internal_font_number f, int b)
 
 
 
-integer copy_font(integer f)
+int copy_font(int f)
 {
-    int i;
+    int i, ci_cnt, ci_size;
     charinfo *ci;
-    integer k = new_font();
-    memcpy(font_tables[k], font_tables[f], sizeof(texfont));
+    int k = new_font();
 
+    {
+        ci = font_tables[k]->charinfo;
+        ci_cnt = font_tables[k]->charinfo_count;
+        ci_size = font_tables[k]->charinfo_size;
+        memcpy(font_tables[k], font_tables[f], sizeof(texfont));
+        font_tables[k]->charinfo = ci;
+        font_tables[k]->charinfo_count = ci_cnt;
+        font_tables[k]->charinfo_size = ci_size;
+    }
+
+    font_malloc_charinfo(k, font_tables[f]->charinfo_count);
     set_font_cache_id(k, 0);
     set_font_used(k, 0);
     set_font_touched(k, 0);
@@ -1040,6 +1060,7 @@ integer copy_font(integer f)
     font_tables[k]->_font_name = NULL;
     font_tables[k]->_font_filename = NULL;
     font_tables[k]->_font_fullname = NULL;
+    font_tables[k]->_font_psname = NULL;
     font_tables[k]->_font_encodingname = NULL;
     font_tables[k]->_font_area = NULL;
     font_tables[k]->_font_cidregistry = NULL;
@@ -1052,6 +1073,8 @@ integer copy_font(integer f)
         set_font_filename(k, xstrdup(font_filename(f)));
     if (font_fullname(f) != NULL)
         set_font_fullname(k, xstrdup(font_fullname(f)));
+    if (font_psname(f) != NULL)
+        set_font_psname(k, xstrdup(font_psname(f)));
     if (font_encodingname(f) != NULL)
         set_font_encodingname(k, xstrdup(font_encodingname(f)));
     if (font_area(f) != NULL)
@@ -1073,11 +1096,7 @@ integer copy_font(integer f)
         memcpy(math_param_base(k), math_param_base(f), i);
     }
 
-    i = sizeof(charinfo) * (font_tables[f]->charinfo_size + 1);
-    font_bytes += i;
-    font_tables[k]->charinfo = xmalloc(i);
-    memset(font_tables[k]->charinfo, 0, i);
-    for (i = 0; i < font_tables[k]->charinfo_size; i++) {
+    for (i = 0; i <= font_tables[f]->charinfo_count; i++) {
         ci = copy_charinfo(&font_tables[f]->charinfo[i]);
         font_tables[k]->charinfo[i] = *ci;
     }
@@ -1094,7 +1113,7 @@ integer copy_font(integer f)
     return k;
 }
 
-void delete_font(integer f)
+void delete_font(int f)
 {
     int i;
     charinfo *co;
@@ -1103,6 +1122,7 @@ void delete_font(integer f)
         set_font_name(f, NULL);
         set_font_filename(f, NULL);
         set_font_fullname(f, NULL);
+        set_font_psname(f, NULL);
         set_font_encodingname(f, NULL);
         set_font_area(f, NULL);
         set_font_cidregistry(f, NULL);
@@ -1111,7 +1131,7 @@ void delete_font(integer f)
         set_right_boundary(f, NULL);
 
         for (i = font_bc(f); i <= font_ec(f); i++) {
-            if (char_exists(f, i)) {
+            if (quick_char_exists(f, i)) {
                 co = char_info(f, i);
                 set_charinfo_name(co, NULL);
                 set_charinfo_tounicode(co, NULL);
@@ -1148,7 +1168,7 @@ void create_null_font(void)
     set_font_touched(i, 1);
 }
 
-boolean is_valid_font(integer id)
+boolean is_valid_font(int id)
 {
     int ret = 0;
     if (id >= 0 && id <= font_id_maxval && font_tables[id] != NULL)
@@ -1157,7 +1177,7 @@ boolean is_valid_font(integer id)
 }
 
 /* return 1 == identical */
-boolean cmp_font_name(integer id, str_number t)
+boolean cmp_font_name(int id, str_number t)
 {
     char *tid, *tt;
     if (!is_valid_font(id))
@@ -1168,10 +1188,11 @@ boolean cmp_font_name(integer id, str_number t)
         return 1;
     if (tt == NULL || tid == NULL || strcmp(tid, tt) != 0)
         return 0;
+    free(tt);
     return 1;
 }
 
-boolean cmp_font_area(integer id, str_number t)
+boolean cmp_font_area(int id, str_number t)
 {
     char *tt = NULL;
     char *tid = font_area(id);
@@ -1186,11 +1207,12 @@ boolean cmp_font_area(integer id, str_number t)
         return 1;
     if (tt == NULL || strcmp(tid, tt) != 0)
         return 0;
+    free(tt);
     return 1;
 }
 
 
-static boolean same_font_name(integer id, integer t)
+static boolean same_font_name(int id, int t)
 {
     int ret = 0;
     if (font_name(t) == NULL ||
@@ -1237,9 +1259,9 @@ boolean font_shareable(internal_font_number f, internal_font_number k)
     return ret;
 }
 
-integer test_no_ligatures(internal_font_number f)
+int test_no_ligatures(internal_font_number f)
 {
-    integer c;
+    int c;
     for (c = font_bc(f); c <= font_ec(f); c++) {
         if (has_lig(f, c))      /* char_exists(f,c) */
             return 0;
@@ -1247,7 +1269,7 @@ integer test_no_ligatures(internal_font_number f)
     return 1;
 }
 
-integer get_tag_code(internal_font_number f, integer c)
+int get_tag_code(internal_font_number f, int c)
 {
     small_number i;
     if (char_exists(f, c)) {
@@ -1264,27 +1286,27 @@ integer get_tag_code(internal_font_number f, integer c)
     return -1;
 }
 
-integer get_lp_code(internal_font_number f, integer c)
+int get_lp_code(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_lp(ci);
 }
 
-integer get_rp_code(internal_font_number f, integer c)
+int get_rp_code(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_rp(ci);
 }
 
-integer get_ef_code(internal_font_number f, integer c)
+int get_ef_code(internal_font_number f, int c)
 {
     charinfo *ci = char_info(f, c);
     return get_charinfo_ef(ci);
 }
 
-void set_tag_code(internal_font_number f, integer c, integer i)
+void set_tag_code(internal_font_number f, int c, int i)
 {
-    integer fixedi;
+    int fixedi;
     charinfo *co;
     if (char_exists(f, c)) {
         /* abs(fix_int(i-7,0)) */
@@ -1310,7 +1332,7 @@ void set_tag_code(internal_font_number f, integer c, integer i)
 }
 
 
-void set_lp_code(internal_font_number f, integer c, integer i)
+void set_lp_code(internal_font_number f, int c, int i)
 {
     charinfo *co;
     if (char_exists(f, c)) {
@@ -1319,7 +1341,7 @@ void set_lp_code(internal_font_number f, integer c, integer i)
     }
 }
 
-void set_rp_code(internal_font_number f, integer c, integer i)
+void set_rp_code(internal_font_number f, int c, int i)
 {
     charinfo *co;
     if (char_exists(f, c)) {
@@ -1328,7 +1350,7 @@ void set_rp_code(internal_font_number f, integer c, integer i)
     }
 }
 
-void set_ef_code(internal_font_number f, integer c, integer i)
+void set_ef_code(internal_font_number f, int c, int i)
 {
     charinfo *co;
     if (char_exists(f, c)) {
@@ -1339,7 +1361,7 @@ void set_ef_code(internal_font_number f, integer c, integer i)
 
 void set_no_ligatures(internal_font_number f)
 {
-    integer c;
+    int c;
     charinfo *co;
 
     if (font_tables[f]->ligatures_disabled)
@@ -1356,8 +1378,9 @@ void set_no_ligatures(internal_font_number f)
     font_tables[f]->ligatures_disabled = 1;
 }
 
-liginfo get_ligature(internal_font_number f, integer lc, integer rc)
+liginfo get_ligature(internal_font_number f, int lc, int rc)
 {
+    int k;
     liginfo t, u;
     charinfo *co;
     t.lig = 0;
@@ -1384,9 +1407,9 @@ liginfo get_ligature(internal_font_number f, integer lc, integer rc)
 }
 
 
-scaled raw_get_kern(internal_font_number f, integer lc, integer rc)
+scaled raw_get_kern(internal_font_number f, int lc, int rc)
 {
-    integer k;
+    int k;
     kerninfo u;
     charinfo *co;
     if (lc == non_boundarychar || rc == non_boundarychar)
@@ -1409,7 +1432,7 @@ scaled raw_get_kern(internal_font_number f, integer lc, integer rc)
 }
 
 
-scaled get_kern(internal_font_number f, integer lc, integer rc)
+scaled get_kern(internal_font_number f, int lc, int rc)
 {
     if (lc == non_boundarychar || rc == non_boundarychar || (!has_kern(f, lc)))
         return 0;
@@ -1430,7 +1453,7 @@ scaled get_kern(internal_font_number f, integer lc, integer rc)
 void dump_charinfo(int f, int c)
 {
     charinfo *co;
-    integer x;
+    int x;
     liginfo *lig;
     kerninfo *kern;
     dump_int(c);
@@ -1529,7 +1552,7 @@ void dump_font_entry(texfont * f)
 
 void dump_font(int f)
 {
-    integer i, x;
+    int i, x;
 
     set_font_used(f, 0);
     font_tables[f]->charinfo_cache = NULL;
@@ -1538,6 +1561,7 @@ void dump_font(int f)
     dump_string(font_area(f));
     dump_string(font_filename(f));
     dump_string(font_fullname(f));
+    dump_string(font_psname(f));
     dump_string(font_encodingname(f));
     dump_string(font_cidregistry(f));
     dump_string(font_cidordering(f));
@@ -1561,7 +1585,7 @@ void dump_font(int f)
     }
 
     for (i = font_bc(f); i <= font_ec(f); i++) {
-        if (char_exists(f, i)) {
+        if (quick_char_exists(f, i)) {
             dump_charinfo(f, i);
         }
     }
@@ -1570,11 +1594,11 @@ void dump_font(int f)
 int undump_charinfo(int f)
 {
     charinfo *co;
-    integer x, i;
+    int x, i;
     char *s = NULL;
     liginfo *lig = NULL;
     kerninfo *kern = NULL;
-    real_eight_bits *packet = NULL;
+    eight_bits *packet = NULL;
 
     undump_int(i);
     co = get_charinfo(f, i);
@@ -1664,7 +1688,7 @@ int undump_charinfo(int f)
 
 void undump_font_entry(texfont * f)
 {
-    integer x = 0;
+    int x = 0;
     /* *INDENT-OFF* */  
     undump_int(x); f->_font_size = x;
     undump_int(x); f->_font_dsize = x;
@@ -1706,7 +1730,7 @@ void undump_font_entry(texfont * f)
 
 void undump_font(int f)
 {
-    integer x, i;
+    int x, i;
     texfont *tt;
     charinfo *ci;
     char *s;
@@ -1721,6 +1745,7 @@ void undump_font(int f)
     undump_font_string(set_font_area);
     undump_font_string(set_font_filename);
     undump_font_string(set_font_fullname);
+    undump_font_string(set_font_psname);
     undump_font_string(set_font_encodingname);
     undump_font_string(set_font_cidregistry);
     undump_font_string(set_font_cidordering);
