@@ -20,7 +20,7 @@
 #include "ptexlib.h"
 
 static const char _svn_version[] =
-    "$Id: tounicode.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/font/tounicode.c $";
+    "$Id: tounicode.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/font/tounicode.c $";
 
 #define isXdigit(c) (isdigit(c) || ('A' <= (c) && (c) <= 'F'))
 #define UNI_UNDEF          -1
@@ -65,7 +65,7 @@ void glyph_unicode_free(void)
 
 void def_tounicode(str_number glyph, str_number unistr)
 {
-    char buf[SMALL_BUF_SIZE], *p;
+    char buf[SMALL_BUF_SIZE], *p, *ph;
     char buf2[SMALL_BUF_SIZE], *q;
     int valid_unistr;           /* 0: invalid; 1: unicode value; 2: string */
     int i, l;
@@ -74,8 +74,10 @@ void def_tounicode(str_number glyph, str_number unistr)
 
     p = makecstring(glyph);
     assert(strlen(p) < SMALL_BUF_SIZE);
-    strcpy(buf, p);             /* copy the result to buf before next call of makecstring() */
+    strcpy(buf, p);
+    free(p);
     p = makecstring(unistr);
+    ph = p;
     while (*p == ' ')
         p++;                    /* ignore leading spaces */
     l = strlen(p);
@@ -125,6 +127,7 @@ void def_tounicode(str_number glyph, str_number unistr)
     }
     aa = avl_probe(glyph_unicode_tree, gu);
     assert(aa != NULL);
+    free(ph);
 }
 
 
@@ -307,19 +310,19 @@ static void set_cid_glyph_unicode(long index, glyph_unicode_entry * gp,
 }
 
 
-integer write_tounicode(char **glyph_names, char *name)
+int write_tounicode(PDF pdf, char **glyph_names, char *name)
 {
     char buf[SMALL_BUF_SIZE], *p;
     static char builtin_suffix[] = "-builtin";
     short range_size[257];
     glyph_unicode_entry gtab[257];
-    integer objnum;
+    int objnum;
     int i, j;
     int bfchar_count, bfrange_count, subrange_count;
     assert(strlen(name) + strlen(builtin_suffix) < SMALL_BUF_SIZE);
     if (glyph_unicode_tree == NULL) {
         /*pdftex_warn("no GlyphToUnicode entry has been inserted yet!"); */
-        fixed_gen_tounicode = 0;
+        pdf->gen_tounicode = 0;
         return 0;
     }
     strcpy(buf, name);
@@ -328,10 +331,10 @@ integer write_tounicode(char **glyph_names, char *name)
     else
         strcat(buf, builtin_suffix);    /* ".enc" not present, this is a builtin
                                            encoding so the name is eg "cmr10-builtin" */
-    objnum = pdf_new_objnum();
-    pdf_begin_dict(objnum, 0);
-    pdf_begin_stream();
-    pdf_printf("%%!PS-Adobe-3.0 Resource-CMap\n"
+    objnum = pdf_new_objnum(pdf);
+    pdf_begin_dict(pdf, objnum, 0);
+    pdf_begin_stream(pdf);
+    pdf_printf(pdf, "%%!PS-Adobe-3.0 Resource-CMap\n"
                "%%%%DocumentNeededResources: ProcSet (CIDInit)\n"
                "%%%%IncludeResource: ProcSet (CIDInit)\n"
                "%%%%BeginResource: CMap (TeX-%s-0)\n"
@@ -399,16 +402,16 @@ integer write_tounicode(char **glyph_names, char *name)
     else
         subrange_count = bfrange_count;
     bfrange_count -= subrange_count;
-    pdf_printf("%i beginbfrange\n", subrange_count);
+    pdf_printf(pdf, "%i beginbfrange\n", subrange_count);
     for (j = 0; j < subrange_count; j++) {
         while (range_size[i] <= 1 && i < 256)
             i++;
         assert(i < 256);
-        pdf_printf("<%02X> <%02X> <%s>\n", i, i + range_size[i] - 1,
+        pdf_printf(pdf, "<%02X> <%02X> <%s>\n", i, i + range_size[i] - 1,
                    utf16be_str(gtab[i].code));
         i += range_size[i];
     }
-    pdf_printf("endbfrange\n");
+    pdf_printf(pdf, "endbfrange\n");
     if (bfrange_count > 0)
         goto write_bfrange;
 
@@ -420,7 +423,7 @@ integer write_tounicode(char **glyph_names, char *name)
     else
         subrange_count = bfchar_count;
     bfchar_count -= subrange_count;
-    pdf_printf("%i beginbfchar\n", subrange_count);
+    pdf_printf(pdf, "%i beginbfchar\n", subrange_count);
     for (j = 0; j < subrange_count; j++) {
         while (i < 256) {
             if (range_size[i] > 1)
@@ -433,12 +436,12 @@ integer write_tounicode(char **glyph_names, char *name)
         assert(i < 256 && gtab[i].code != UNI_UNDEF);
         if (gtab[i].code == UNI_STRING || gtab[i].code == UNI_EXTRA_STRING) {
             assert(gtab[i].unicode_seq != NULL);
-            pdf_printf("<%02X> <%s>\n", i, gtab[i].unicode_seq);
+            pdf_printf(pdf, "<%02X> <%s>\n", i, gtab[i].unicode_seq);
         } else
-            pdf_printf("<%02X> <%s>\n", i, utf16be_str(gtab[i].code));
+            pdf_printf(pdf, "<%02X> <%s>\n", i, utf16be_str(gtab[i].code));
         i++;
     }
-    pdf_printf("endbfchar\n");
+    pdf_printf(pdf, "endbfchar\n");
     if (bfchar_count > 0)
         goto write_bfchar;
 
@@ -448,19 +451,19 @@ integer write_tounicode(char **glyph_names, char *name)
             xfree(gtab[i].unicode_seq);
     }
 
-    pdf_printf("endcmap\n"
+    pdf_printf(pdf, "endcmap\n"
                "CMapName currentdict /CMap defineresource pop\n"
                "end\n" "end\n" "%%%%EndResource\n" "%%%%EOF\n");
-    pdf_end_stream();
+    pdf_end_stream(pdf);
     return objnum;
 }
 
-integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
+int write_cid_tounicode(PDF pdf, fo_entry * fo, internalfontnumber f)
 {
 
     int range_size[65537];
     glyph_unicode_entry gtab[65537];
-    integer objnum;
+    int objnum;
     int i, j, k;
     int bfchar_count, bfrange_count, subrange_count;
     char *buf;
@@ -471,10 +474,10 @@ integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
             (fo->fd->subset_tag != NULL ? fo->fd->subset_tag : "UCS"),
             fo->fd->fontname);
 
-    objnum = pdf_new_objnum();
-    pdf_begin_dict(objnum, 0);
-    pdf_begin_stream();
-    pdf_printf("%%!PS-Adobe-3.0 Resource-CMap\n"
+    objnum = pdf_new_objnum(pdf);
+    pdf_begin_dict(pdf, objnum, 0);
+    pdf_begin_stream(pdf);
+    pdf_printf(pdf, "%%!PS-Adobe-3.0 Resource-CMap\n"
                "%%%%DocumentNeededResources: ProcSet (CIDInit)\n"
                "%%%%IncludeResource: ProcSet (CIDInit)\n"
                "%%%%BeginResource: CMap (TeX-%s-0)\n"
@@ -502,7 +505,7 @@ integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
     for (k = 1; k <= max_font_id(); k++) {
         if (k == f || -f == pdf_font_num(k)) {
             for (i = font_bc(k); i <= font_ec(k); i++) {
-                if (char_exists(k, i) && char_used(k, i)) {
+                if (quick_char_exists(k, i) && char_used(k, i)) {
                     j = char_index(k, i);
                     if (gtab[j].code == UNI_UNDEF) {
                         set_cid_glyph_unicode(i, &gtab[j], f);
@@ -553,16 +556,16 @@ integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
     else
         subrange_count = bfrange_count;
     bfrange_count -= subrange_count;
-    pdf_printf("%i beginbfrange\n", subrange_count);
+    pdf_printf(pdf, "%i beginbfrange\n", subrange_count);
     for (j = 0; j < subrange_count; j++) {
         while (range_size[i] <= 1 && i < 65536)
             i++;
         assert(i < 65536);
-        pdf_printf("<%04X> <%04X> <%s>\n", i, i + range_size[i] - 1,
+        pdf_printf(pdf, "<%04X> <%04X> <%s>\n", i, i + range_size[i] - 1,
                    utf16be_str(gtab[i].code));
         i += range_size[i];
     }
-    pdf_printf("endbfrange\n");
+    pdf_printf(pdf, "endbfrange\n");
     if (bfrange_count > 0)
         goto write_bfrange;
 
@@ -574,7 +577,7 @@ integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
     else
         subrange_count = bfchar_count;
     bfchar_count -= subrange_count;
-    pdf_printf("%i beginbfchar\n", subrange_count);
+    pdf_printf(pdf, "%i beginbfchar\n", subrange_count);
     for (j = 0; j < subrange_count; j++) {
         while (i < 65536) {
             if (range_size[i] > 1)
@@ -587,12 +590,12 @@ integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
         assert(i < 65536 && gtab[i].code != UNI_UNDEF);
         if (gtab[i].code == UNI_STRING || gtab[i].code == UNI_EXTRA_STRING) {
             assert(gtab[i].unicode_seq != NULL);
-            pdf_printf("<%04X> <%s>\n", i, gtab[i].unicode_seq);
+            pdf_printf(pdf, "<%04X> <%s>\n", i, gtab[i].unicode_seq);
         } else
-            pdf_printf("<%04X> <%s>\n", i, utf16be_str(gtab[i].code));
+            pdf_printf(pdf, "<%04X> <%s>\n", i, utf16be_str(gtab[i].code));
         i++;
     }
-    pdf_printf("endbfchar\n");
+    pdf_printf(pdf, "endbfchar\n");
     if (bfchar_count > 0)
         goto write_bfchar;
 
@@ -602,9 +605,9 @@ integer write_cid_tounicode(fo_entry * fo, internalfontnumber f)
             xfree(gtab[i].unicode_seq);
     }
 
-    pdf_printf("endcmap\n"
+    pdf_printf(pdf, "endcmap\n"
                "CMapName currentdict /CMap defineresource pop\n"
                "end\n" "end\n" "%%%%EndResource\n" "%%%%EOF\n");
-    pdf_end_stream();
+    pdf_end_stream(pdf);
     return objnum;
 }
