@@ -19,12 +19,12 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 #include "ptexlib.h"
-#include "luatex-api.h"
+
 #include "luatexfont.h"
 
 static const char _svn_version[] =
-    "$Id: vfovf.c 2448 2009-06-08 07:43:50Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/font/vfovf.c $";
+    "$Id: vfovf.c 3261 2009-12-18 11:38:21Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/font/vfovf.c $";
 
 /* this is a hack! */
 #define font_max 5000
@@ -98,11 +98,11 @@ static const char _svn_version[] =
 
 /* go out \.{VF} processing with an error message */
 #define bad_vf(a) { xfree(vf_buffer); print_nlp();  \
-    print_string("Error in processing VF font (");  \
-    print_string(font_name(f));       \
-    print_string(".vf): ");       \
-    print_string(a);          \
-    print_string(", virtual font will be ignored"); \
+    tprint("Error in processing VF font (");  \
+    tprint(font_name(f));       \
+    tprint(".vf): ");       \
+    tprint(a);          \
+    tprint(", virtual font will be ignored"); \
     print_ln();  return; }
 
 #define lua_bad_vf(a) { xfree(vf_buffer);   \
@@ -131,7 +131,7 @@ boolean auto_expand_vf(internal_font_number f); /* forward */
 
 #define vf_byte(a)                                     \
 {                                                      \
-    real_eight_bits vf_tmp_b;                          \
+  eight_bits vf_tmp_b;				       \
     if (vf_cur >= vf_size) {                           \
         pdftex_fail("unexpected eof on virtual font"); \
     }                                                  \
@@ -155,10 +155,10 @@ boolean auto_expand_vf(internal_font_number f); /* forward */
 
 #define vf_read(k, l)                            \
 {                                                \
-    integer itmp = 0, dtmp = k, jtmp = 0;        \
+    int itmp = 0, dtmp = k, jtmp = 0;            \
     while (dtmp > 0) {                           \
         vf_byte(jtmp);                           \
-        if ((dtmp == (integer) k) && jtmp > 127) \
+        if ((dtmp == (int) k) && jtmp > 127)     \
             jtmp = jtmp - 256;                   \
         itmp = itmp * 256 + jtmp;                \
         decr(dtmp);                              \
@@ -176,44 +176,41 @@ boolean auto_expand_vf(internal_font_number f); /* forward */
     l = itmp;                                    \
 }
 
-void pdf_check_vf_cur_val(void)
+void pdf_check_vf(internal_font_number f)
 {
-    internal_font_number f;
-    f = cur_val;
     if (font_type(f) == virtual_font_type)
-        pdf_error(maketexstring("font"),
-                  maketexstring("command cannot be used with virtual font"));
+        pdf_error("font", "command cannot be used with virtual font");
 }
 
 static void
 vf_local_font_warning(internal_font_number f, internal_font_number k, char *s,
-                      integer a, integer b)
+                      int a, int b)
 {
     print_nlp();
-    print_string(s);
-    print_string(" in local font ");
-    print_string(font_name(k));
-    print_string(" (");
+    tprint(s);
+    tprint(" in local font ");
+    tprint(font_name(k));
+    tprint(" (");
     print_int(b);
-    print_string(" != ");
+    tprint(" != ");
     print_int(a);
-    print_string(") in virtual font ");
-    print_string(font_name(f));
-    print_string(".vf ignored.");
+    tprint(") in virtual font ");
+    tprint(font_name(f));
+    tprint(".vf ignored.");
 }
 
 
 /* process a local font in \.{VF} file */
 
 internal_font_number
-vf_def_font(internal_font_number f, unsigned char *vf_buffer, integer * vf_cr)
+vf_def_font(internal_font_number f, unsigned char *vf_buffer, int *vf_cr)
 {
     internal_font_number k;
     str_number s;
     scaled ds, fs;
     four_quarters cs;
     memory_word tmp_w;          /* accumulator */
-    integer junk;
+    int junk;
     unsigned long checksum;
     cs.b0 = vf_buffer[(*vf_cr)];
     cs.b1 = vf_buffer[(*vf_cr) + 1];
@@ -257,17 +254,17 @@ vf_def_font(internal_font_number f, unsigned char *vf_buffer, integer * vf_cr)
         tmp_b0--;
         (*vf_cr)++;             /* skip the font path */
     }
-    string_room(tmp_b1);
+    str_room((unsigned) tmp_b1);
     while (tmp_b1 > 0) {
         tmp_b1--;
         junk = vf_buffer[(*vf_cr)];
         (*vf_cr)++;
-        append_pool_char(junk);
+        append_char(junk);
     }
     s = make_string();
     k = tfm_lookup(s, fs);
     if (k == null_font)
-        k = read_font_info(get_nullcs(), s, fs, -1);
+        k = read_font_info(null_cs, s, fs, -1);
     if (k != null_font) {
         if (checksum != 0 && font_checksum(k) != 0
             && checksum != font_checksum(k))
@@ -286,38 +283,19 @@ vf_def_font(internal_font_number f, unsigned char *vf_buffer, integer * vf_cr)
 }
 
 
-int open_vf_file(char *fn, unsigned char **vbuffer, integer * vsize)
+int open_vf_file(char *fn, unsigned char **vbuffer, int *vsize)
 {
     boolean res;                /* was the callback successful? */
-    integer callback_id;
-    boolean file_read;          /* was |vf_file| successfully read? */
+    int callback_id;
+    boolean file_read = false;  /* was |vf_file| successfully read? */
     FILE *vf_file;
-    char *fnam = NULL;
+    char *fname = luatex_find_file(fn, find_vf_file_callback);
+    if (!fname)
+        fname = fn;
 
-
-
-
-    namelength = strlen(fn);
-    nameoffile = xmalloc(namelength + 2);
-    strcpy((char *) (nameoffile + 1), fn);
-
-    callback_id = callback_defined(find_vf_file_callback);
-    if (callback_id > 0) {
-        res = run_callback(callback_id, "S->S", fn, &fnam);
-        if (res && (fnam != NULL) && (strlen(fnam) > 0)) {
-            /* @<Fixup |nameoffile| after callback@>; */
-            free(nameoffile);
-            namelength = strlen(fnam);
-            nameoffile = xmalloc(namelength + 2);
-            strcpy((char *) (nameoffile + 1), fnam);
-        } else {
-            return 0;
-        }
-    }
     callback_id = callback_defined(read_vf_file_callback);
     if (callback_id > 0) {
-        file_read = false;
-        res = run_callback(callback_id, "S->bSd", stringcast(nameoffile + 1),
+        res = run_callback(callback_id, "S->bSd", fname,
                            &file_read, vbuffer, vsize);
         if (res && file_read && (*vsize > 0)) {
             return 1;
@@ -325,9 +303,12 @@ int open_vf_file(char *fn, unsigned char **vbuffer, integer * vsize)
         if (!file_read)
             return 0;           /* -1 */
     } else {
-        if (ovf_b_open_in(vf_file) || vf_b_open_in(vf_file)) {
+        if (luatex_open_input
+            (&(vf_file), fname, kpse_ovf_format, FOPEN_RBIN_MODE, false)
+            || luatex_open_input(&(vf_file), fname, kpse_vf_format,
+                                 FOPEN_RBIN_MODE, false)) {
             res = read_vf_file(vf_file, vbuffer, vsize);
-            b_close(vf_file);
+            close_file(vf_file);
             if (res) {
                 return 1;
             }
@@ -377,9 +358,9 @@ int open_vf_file(char *fn, unsigned char **vbuffer, integer * vsize)
   ((tmp_b0 != font_check_0(f)) || (tmp_b1 != font_check_1(f)) ||  \
    (tmp_b2 != font_check_2(f)) || (tmp_b3 != font_check_3(f)))) { \
       print_nlp();              \
-      print_string("checksum mismatch in font ");     \
-      print_string(font_name(f));         \
-      print_string(".vf ignored "); } }
+      tprint("checksum mismatch in font ");     \
+      tprint(font_name(f));         \
+      tprint(".vf ignored "); } }
 
 #define test_dsize()                                   \
 {                                                      \
@@ -387,13 +368,13 @@ int open_vf_file(char *fn, unsigned char **vbuffer, integer * vsize)
     vf_read(4, read_tmp);                              \
     if ((read_tmp / 16) != font_dsize(f)) {            \
         print_nlp();                                   \
-        print_string("design size mismatch in font "); \
-        print_string(font_name(f));                    \
-        print_string(".vf ignored");                   \
+        tprint("design size mismatch in font ");       \
+        tprint(font_name(f));			       \
+        tprint(".vf ignored");			       \
     }                                                  \
 }
 
-int count_packet_bytes(real_eight_bits * vf_buf, int cur_bute, int count)
+int count_packet_bytes(eight_bits * vf_buf, int cur_bute, int count)
 {
     int k = 0;
     int ff = 0;
@@ -666,27 +647,27 @@ int count_packet_bytes(real_eight_bits * vf_buf, int cur_bute, int count)
 
 void do_vf(internal_font_number f)
 {
-    integer k, i;
+    int k, i;
     unsigned cmd, n;
     scaled x, y, w, z, h, v;
-    integer cc, cmd_length, packet_length;
+    int cc, cmd_length, packet_length;
     charinfo *co;
     scaled tfm_width;
-    integer save_cur_byte;
+    int save_cur_byte;
     vf_stack_index stack_level;
-    integer vf_z;               /* multiplier */
-    integer vf_alpha;           /* correction for negative values */
+    int vf_z;                   /* multiplier */
+    int vf_alpha;               /* correction for negative values */
     char vf_beta;               /* divisor */
-    integer vf_np;
-    real_eight_bits *vpackets;
+    int vf_np;
+    eight_bits *vpackets;
     memory_word tmp_w;          /* accumulator */
     vf_stack_record vf_stack[256];
-    integer junk;
+    int junk;
     unsigned char *vf_buffer;
-    integer vf_size;
-    integer vf_cur;
-    integer *vf_local_fnts = NULL;      /* external font ids */
-    integer *vf_real_fnts = NULL;       /* internal font ids */
+    int vf_size;
+    int vf_cur;
+    int *vf_local_fnts = NULL;  /* external font ids */
+    int *vf_real_fnts = NULL;   /* internal font ids */
     unsigned vf_nf = 0;         /* local font counter */
 
     if (font_type(f) != unknown_font_type)
@@ -740,7 +721,7 @@ void do_vf(internal_font_number f)
     vf_byte(cmd);
     /* malloc and fill the local font arrays */
     if (vf_nf > 0) {
-        i = vf_nf * sizeof(integer);
+        i = vf_nf * sizeof(int);
         vf_local_fnts = xmalloc(i);
         memset(vf_local_fnts, 0, i);
         vf_real_fnts = xmalloc(i);
@@ -780,9 +761,9 @@ void do_vf(internal_font_number f)
         if (tfm_width != char_width(f, cc)) {
             if (tfm_width != char_width(f, cc)) {
                 print_nlp();
-                print_string("character width mismatch in font ");
-                print_string(font_name(f));
-                print_string(".vf ignored");
+                tprint("character width mismatch in font ");
+                tprint(font_name(f));
+                tprint(".vf ignored");
             }
         }
         k = count_packet_bytes(vf_buffer, vf_cur, packet_length);
@@ -967,8 +948,7 @@ void do_vf(internal_font_number f)
                     break;
                 case push:
                     if (stack_level == vf_stack_size) {
-                        overflow_string("virtual font stack size",
-                                        vf_stack_size);
+                        overflow("virtual font stack size", vf_stack_size);
                     } else {
                         vf_stack[stack_level].stack_w = w;
                         vf_stack[stack_level].stack_x = x;
@@ -1051,25 +1031,25 @@ void do_vf(internal_font_number f)
 
 int make_vf_table(lua_State * L, char *cnom, scaled atsize)
 {
-    integer cmd, k, i;
-    integer cc, cmd_length, packet_length;
+    int cmd, k, i;
+    int cc, cmd_length, packet_length;
     scaled tfm_width;
     vf_stack_index stack_level;
-    integer vf_z;               /* multiplier */
-    integer vf_alpha;           /* correction for negative values */
+    int vf_z;                   /* multiplier */
+    int vf_alpha;               /* correction for negative values */
     char vf_beta;               /* divisor */
     char *s;
     scaled h, v;
     scaled w, x, y, z;
-    integer s_top;              /* lua stack */
-    integer vf_nf;              /* local font counter */
+    int s_top;                  /* lua stack */
+    int vf_nf;                  /* local font counter */
     scaled ds, fs;
     four_quarters cs;
     memory_word tmp_w;          /* accumulator */
     vf_stack_record vf_stack[256];
     unsigned char *vf_buffer;
-    integer vf_size;
-    integer vf_cur;
+    int vf_size;
+    int vf_cur;
 
 
     stack_level = 0;
@@ -1359,8 +1339,7 @@ int make_vf_table(lua_State * L, char *cnom, scaled atsize)
                     break;
                 case push:
                     if (stack_level == vf_stack_size) {
-                        overflow_string("virtual font stack size",
-                                        vf_stack_size);
+                        overflow("virtual font stack size", vf_stack_size);
                     } else {
                         vf_stack[stack_level].stack_w = w;
                         vf_stack[stack_level].stack_x = x;
@@ -1415,9 +1394,9 @@ boolean auto_expand_vf(internal_font_number f)
 {
 
     internal_font_number bf;
-    integer e, k;
-    integer *vf_old_fonts, *vf_new_fonts;
-    integer num = 0;
+    int e, k;
+    int *vf_old_fonts, *vf_new_fonts;
+    int num = 0;
 
     if ((!pdf_font_auto_expand(f)) || (pdf_font_blink(f) == null_font))
         return false;           /* not an auto-expanded font */
@@ -1430,7 +1409,7 @@ boolean auto_expand_vf(internal_font_number f)
 
     vf_old_fonts = packet_local_fonts(bf, &num);
     if (num > 0) {
-        vf_new_fonts = xmalloc(num * sizeof(integer));
+        vf_new_fonts = xmalloc(num * sizeof(int));
         for (k = 0; k < num; k++) {
             vf_new_fonts[k] = auto_expand_font(vf_old_fonts[k], e);
             copy_expand_params(vf_new_fonts[k], vf_old_fonts[k], e);
@@ -1443,13 +1422,12 @@ boolean auto_expand_vf(internal_font_number f)
     return true;
 }
 
-/* this function is not always safe, because it needs makecstring() */
-str_number expand_font_name(internal_font_number f, integer e)
+str_number expand_font_name(internal_font_number f, int e)
 {
     int old_setting;
     old_setting = selector;
     selector = new_string;
-    print_string(font_name(f));
+    tprint(font_name(f));
     if (e > 0) {
         print_char('+');
     }
@@ -1459,13 +1437,13 @@ str_number expand_font_name(internal_font_number f, integer e)
 }
 
 
-internal_font_number auto_expand_font(internal_font_number f, integer e)
+internal_font_number auto_expand_font(internal_font_number f, int e)
 {
     internal_font_number k;
     kerninfo *krn;
     charinfo *co;
     char *fn;
-    integer i;
+    int i;
     scaled w;
     k = copy_font(f);
     i = strlen(font_name(f)) + 12;
@@ -1473,7 +1451,7 @@ internal_font_number auto_expand_font(internal_font_number f, integer e)
     snprintf(fn, i, "%s%s%d", font_name(f), (e > 0 ? "+" : ""), (int) e);
     set_font_name(k, fn);
     for (i = font_bc(k); i <= font_ec(k); i++) {
-        if (char_exists(k, i)) {
+        if (quick_char_exists(k, i)) {
             co = get_charinfo(k, i);
             w = round_xn_over_d(char_width(f, i), 1000 + e, 1000);
             set_charinfo_width(co, w);
@@ -1495,8 +1473,8 @@ internal_font_number auto_expand_font(internal_font_number f, integer e)
 void vf_expand_local_fonts(internal_font_number f)
 {
     internal_font_number lf;
-    integer k, num;
-    integer *vf_old_fonts;
+    int k, num;
+    int *vf_old_fonts;
     pdfassert(font_type(f) == virtual_font_type);
     num = 0;
     vf_old_fonts = packet_local_fonts(f, &num);
@@ -1515,15 +1493,15 @@ void vf_expand_local_fonts(internal_font_number f)
 }
 
 internal_font_number
-letter_space_font(halfword u, internal_font_number f, integer e)
+letter_space_font(halfword u, internal_font_number f, int e)
 {
     internal_font_number k;
     scaled w, r;
 
     char *new_font_name;
-    integer vf_z;
-    integer vf_alpha;
-    integer vf_beta;
+    int vf_z;
+    int vf_alpha;
+    int vf_beta;
     memory_word tmp_w;          /* accumulator */
 
 
