@@ -18,10 +18,15 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+static const char _svn_version[] =
+    "$Id: writeimg.c 3261 2009-12-18 11:38:21Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/image/writeimg.c $";
+
 #include <assert.h>
 #include "ptexlib.h"
 #include <kpathsea/c-auto.h>
 #include <kpathsea/c-memstr.h>
+         /* for tokenlist_to_cstring */
 
 #include "image.h"
 #include "writejpg.h"
@@ -29,12 +34,11 @@
 #include "writejbig2.h"
 #include "pdftoepdf.h"
 
-#include <../lua51/lua.h>
+#include <../lua51/lua.h>       /* for LUA_NOREF */
 #include <../lua51/lauxlib.h>
 
-static const char _svn_version[] =
-    "$Id: writeimg.c 2448 2009-06-08 07:43:50Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/image/writeimg.c $";
+#define pdf_image_resolution int_par(pdf_image_resolution_code)
+#define pdf_pagebox int_par(pdf_pagebox_code)
 
 /**********************************************************************/
 /*
@@ -118,7 +122,7 @@ static const char _svn_version[] =
 #define HEADER_PDF "%PDF-1."
 #define MAX_HEADER (sizeof(HEADER_PNG)-1)
 
-integer epdf_lastGroupObjectNum;
+int epdf_lastGroupObjectNum;
 
 static void check_type_by_header(image_dict * idict)
 {
@@ -182,82 +186,65 @@ void new_img_pdfstream_struct(image_dict * p)
 
 /**********************************************************************/
 
-void init_image(image * p)
+static void init_image(image * p)
 {
     assert(p != NULL);
     set_wd_running(p);
     set_ht_running(p);
     set_dp_running(p);
     img_transform(p) = 0;
-    img_flags(p) = 0;
-    img_unset_scaled(p);
     img_dict(p) = NULL;
-    img_arrayidx(p) = -1;       /* -1 = unused, used count from 0 */
     img_dictref(p) = LUA_NOREF;
 }
 
-image *new_image()
+image *new_image(void)
 {
     image *p = xtalloc(1, image);
     init_image(p);
     return p;
 }
 
-void init_image_dict(image_dict * p)
+static void init_image_dict(image_dict * p)
 {
     assert(p != NULL);
-    img_objnum(p) = 0;
-    img_index(p) = 0;
-    img_xsize(p) = 0;
-    img_ysize(p) = 0;
-    img_xorig(p) = 0;
-    img_yorig(p) = 0;
-    img_xres(p) = 0;
-    img_yres(p) = 0;
-    img_rotation(p) = 0;
-    img_colorspace(p) = 0;
-    img_group_ref(p) = 0;
-    img_totalpages(p) = 0;
+    memset(p, 0, sizeof(image_dict));
+    set_wd_running(p);
+    set_ht_running(p);
+    set_dp_running(p);
+    img_transform(p) = 0;
     img_pagenum(p) = 1;
-    img_pagename(p) = NULL;
-    img_filename(p) = NULL;
-    img_filepath(p) = NULL;
-    img_attr(p) = NULL;
-    img_file(p) = NULL;
     img_type(p) = IMG_TYPE_NONE;
-    img_color(p) = 0;
-    img_colordepth(p) = 0;
     img_pagebox(p) = PDF_BOX_SPEC_MEDIA;
-    img_bbox(p)[0] = 0;
-    img_bbox(p)[1] = 0;
-    img_bbox(p)[2] = 0;
-    img_bbox(p)[3] = 0;
     img_unset_bbox(p);
     img_state(p) = DICT_NEW;
-    img_png_ptr(p) = NULL;      /* union */
+    img_index(p) = -1;          /* -1 = unused, used count from 0 */
 }
 
-image_dict *new_image_dict()
+image_dict *new_image_dict(void)
 {
     image_dict *p = xtalloc(1, image_dict);
     init_image_dict(p);
     return p;
 }
 
-void free_dict_strings(image_dict * p)
+static void free_dict_strings(image_dict * p)
 {
-    if (img_filename(p) != NULL)
+    if (img_filename(p) != NULL) {
         xfree(img_filename(p));
-    img_filename(p) = NULL;
-    if (img_filepath(p) != NULL)
+        img_filename(p) = NULL;
+    }
+    if (img_filepath(p) != NULL) {
         xfree(img_filepath(p));
-    img_filepath(p) = NULL;
-    if (img_attr(p) != NULL)
+        img_filepath(p) = NULL;
+    }
+    if (img_attr(p) != NULL) {
         xfree(img_attr(p));
-    img_attr(p) = NULL;
-    if (img_pagename(p) != NULL)
+        img_attr(p) = NULL;
+    }
+    if (img_pagename(p) != NULL) {
         xfree(img_pagename(p));
-    img_pagename(p) = NULL;
+        img_pagename(p) = NULL;
+    }
 }
 
 void free_image_dict(image_dict * p)
@@ -297,14 +284,8 @@ void free_image_dict(image_dict * p)
 
 /**********************************************************************/
 
-void pdf_print_resname_prefix()
-{
-    if (pdf_resname_prefix != 0)
-        pdf_printf(makecstring(pdf_resname_prefix));
-}
-
-void read_img(image_dict * idict, integer pdf_minor_version,
-              integer pdf_inclusion_errorlevel)
+void read_img(PDF pdf,
+              image_dict * idict, int minor_version, int inclusion_errorlevel)
 {
     char *filepath;
     int callback_id;
@@ -321,9 +302,9 @@ void read_img(image_dict * idict, integer pdf_minor_version,
         } else
             img_filepath(idict) =
                 kpse_find_file(img_filename(idict), kpse_tex_format, true);
+        if (img_filepath(idict) == NULL)
+            pdftex_fail("cannot find image file");
     }
-    if (img_filepath(idict) == NULL)
-        pdftex_fail("cannot find image file");
     recorder_record_input(img_filename(idict));
     /* type checks */
     check_type_by_header(idict);
@@ -331,20 +312,22 @@ void read_img(image_dict * idict, integer pdf_minor_version,
     /* read image */
     switch (img_type(idict)) {
     case IMG_TYPE_PDF:
-        read_pdf_info(idict, pdf_minor_version, pdf_inclusion_errorlevel);
+        assert(pdf != NULL);    /* TODO! */
+        read_pdf_info(pdf, idict, minor_version, inclusion_errorlevel,
+                      IMG_CLOSEINBETWEEN);
         img_group_ref(idict) = epdf_lastGroupObjectNum;
         break;
     case IMG_TYPE_PNG:
-        read_png_info(idict, IMG_CLOSEINBETWEEN);
+        read_png_info(pdf, idict, IMG_CLOSEINBETWEEN);
         break;
     case IMG_TYPE_JPG:
-        read_jpg_info(idict, IMG_CLOSEINBETWEEN);
+        read_jpg_info(pdf, idict, IMG_CLOSEINBETWEEN);
         break;
     case IMG_TYPE_JBIG2:
-        if (pdf_minor_version < 4) {
+        if (minor_version < 4) {
             pdftex_fail
                 ("JBIG2 images only possible with at least PDF 1.4; you are generating PDF 1.%i",
-                 (int) pdf_minor_version);
+                 (int) minor_version);
         }
         read_jbig2_info(idict);
         break;
@@ -356,19 +339,181 @@ void read_img(image_dict * idict, integer pdf_minor_version,
         img_state(idict) = DICT_FILESCANNED;
 }
 
+static image_dict *read_image(PDF pdf, char *file_name, int page_num,
+                              char *page_name, int colorspace,
+                              int page_box, int minor_version,
+                              int inclusion_errorlevel)
+{
+    image *a = new_image();
+    image_dict *idict = img_dict(a) = new_image_dict();
+    incr(pdf->ximage_count);
+    pdf_create_obj(pdf, obj_type_ximage, pdf->ximage_count);
+    img_objnum(idict) = pdf->obj_ptr;
+    img_index(idict) = pdf->ximage_count;
+    set_obj_data_ptr(pdf, img_objnum(idict), img_index(idict));
+    idict_to_array(idict);
+    img_colorspace(idict) = colorspace;
+    img_pagenum(idict) = page_num;
+    img_pagename(idict) = page_name;
+    assert(file_name != NULL);
+    cur_file_name = file_name;
+    img_filename(idict) = file_name;
+    img_pagebox(idict) = page_box;
+    read_img(pdf, idict, minor_version, inclusion_errorlevel);
+    return idict;
+}
+
+/* scans PDF pagebox specification */
+static pdfboxspec_e scan_pdf_box_spec(void)
+{
+    if (scan_keyword("mediabox"))
+        return PDF_BOX_SPEC_MEDIA;
+    else if (scan_keyword("cropbox"))
+        return PDF_BOX_SPEC_CROP;
+    else if (scan_keyword("bleedbox"))
+        return PDF_BOX_SPEC_BLEED;
+    else if (scan_keyword("trimbox"))
+        return PDF_BOX_SPEC_TRIM;
+    else if (scan_keyword("artbox"))
+        return PDF_BOX_SPEC_ART;
+    return PDF_BOX_SPEC_NONE;
+}
+
+void scan_pdfximage(PDF pdf)
+{
+    scaled_whd alt_rule;
+    image_dict *idict;
+    int transform = 0, page = 1, pagebox, colorspace = 0;
+    char *named = NULL, *attr = NULL, *file_name = NULL;
+    alt_rule = scan_alt_rule(); /* scans |<rule spec>| to |alt_rule| */
+    if (scan_keyword("attr")) {
+        scan_pdf_ext_toks();
+        attr = tokenlist_to_cstring(def_ref, true, NULL);
+        delete_token_ref(def_ref);
+    }
+    if (scan_keyword("named")) {
+        scan_pdf_ext_toks();
+        named = tokenlist_to_cstring(def_ref, true, NULL);
+        delete_token_ref(def_ref);
+        page = 0;
+    } else if (scan_keyword("page")) {
+        scan_int();
+        page = cur_val;
+    }
+    if (scan_keyword("colorspace")) {
+        scan_int();
+        colorspace = cur_val;
+    }
+    pagebox = scan_pdf_box_spec();
+    if (pagebox == PDF_BOX_SPEC_NONE) {
+        pagebox = pdf_pagebox;
+        if (pagebox == PDF_BOX_SPEC_NONE)
+            pagebox = PDF_BOX_SPEC_CROP;
+    }
+    scan_pdf_ext_toks();
+    file_name = tokenlist_to_cstring(def_ref, true, NULL);
+    assert(file_name != NULL);
+    delete_token_ref(def_ref);
+    idict =
+        read_image(pdf, file_name, page, named, colorspace, pagebox,
+                   pdf_minor_version, pdf_inclusion_errorlevel);
+    img_attr(idict) = attr;
+    img_dimen(idict) = alt_rule;
+    img_transform(idict) = transform;
+    pdf_last_ximage = img_objnum(idict);
+    pdf_last_ximage_pages = img_totalpages(idict);
+    pdf_last_ximage_colordepth = img_colordepth(idict);
+}
+
+#define tail          cur_list.tail_field
+
+void scan_pdfrefximage(PDF pdf)
+{
+    int transform = 0;          /* one could scan transform as well */
+    image_dict *idict;
+    scaled_whd alt_rule, dim;
+    alt_rule = scan_alt_rule(); /* scans |<rule spec>| to |alt_rule| */
+    scan_int();
+    check_obj_exists(pdf, obj_type_ximage, cur_val);
+    new_whatsit(pdf_refximage_node);
+    idict = idict_array[obj_data_ptr(pdf, cur_val)];
+    if (alt_rule.wd != null_flag || alt_rule.ht != null_flag
+        || alt_rule.dp != null_flag)
+        dim = scale_img(idict, alt_rule, transform);
+    else
+        dim = scale_img(idict, img_dimen(idict), img_transform(idict));
+    width(tail) = dim.wd;
+    height(tail) = dim.ht;
+    depth(tail) = dim.dp;
+    pdf_ximage_transform(tail) = transform;
+    pdf_ximage_index(tail) = img_index(idict);
+}
+
+/* tex_scale() sequence of decisions:
+wd ht dp : res = tex;
+wd ht --
+wd -- dp
+wd -- --
+-- ht dp
+-- ht --
+-- -- dp
+-- -- -- : res = nat;
+*/
+
+scaled_whd tex_scale(scaled_whd nat, scaled_whd tex)
+{
+    scaled_whd res;
+    if (!is_running(tex.wd) && !is_running(tex.ht) && !is_running(tex.dp)) {
+        /* width, height, and depth specified */
+        res = tex;
+    } else /* max. 2 dimensions are specified */ if (!is_running(tex.wd)) {
+        res.wd = tex.wd;
+        if (!is_running(tex.ht)) {
+            res.ht = tex.ht;
+            /* width and height specified */
+            res.dp = ext_xn_over_d(tex.ht, nat.dp, nat.ht);
+        } else if (!is_running(tex.dp)) {
+            res.dp = tex.dp;
+            /* width and depth specified */
+            res.ht = ext_xn_over_d(tex.wd, nat.ht + nat.dp, nat.wd) - tex.dp;
+        } else {
+            /* only width specified */
+            res.ht = ext_xn_over_d(tex.wd, nat.ht, nat.wd);
+            res.dp = ext_xn_over_d(tex.wd, nat.dp, nat.wd);
+        }
+    } else if (!is_running(tex.ht)) {
+        res.ht = tex.ht;
+        if (!is_running(tex.dp)) {
+            res.dp = tex.dp;
+            /* height and depth specified */
+            res.wd = ext_xn_over_d(tex.ht + tex.dp, nat.wd, nat.ht + nat.dp);
+        } else {
+            /* only height specified */
+            res.wd = ext_xn_over_d(tex.ht, nat.wd, nat.ht);
+            res.dp = ext_xn_over_d(tex.ht, nat.dp, nat.ht);
+        }
+    } else if (!is_running(tex.dp)) {
+        res.dp = tex.dp;
+        /* only depth specified */
+        res.ht = nat.ht - (tex.dp - nat.dp);
+        res.wd = nat.wd;
+    } else {
+        /* nothing specified */
+        res = nat;
+    }
+    return res;
+}
+
 // Within scale_img() only image width and height matter;
 // the offsets and positioning are not interesting here.
 // But one needs rotation info to swap width and height.
 // img_rotation() comes from the optional /Rotate key in the PDF file.
 
-void scale_img(image * img)
+scaled_whd scale_img(image_dict * idict, scaled_whd alt_rule, int transform)
 {
-    integer x, y, xr, yr, tmp;  /* size and resolution of image */
-    scaled w = 0, h = 0;        /* indeed size corresponds to image resolution */
-    integer default_res;
-    image_dict *idict;
-    assert(img != NULL);
-    idict = img_dict(img);
+    int x, y, xr, yr, tmp;      /* size and resolution of image */
+    scaled_whd nat;             /* natural size corresponding to image resolution */
+    int default_res;
     assert(idict != NULL);
     if ((img_type(idict) == IMG_TYPE_PDF
          || img_type(idict) == IMG_TYPE_PDFSTREAM) && img_is_bbox(idict)) {
@@ -389,7 +534,7 @@ void scale_img(image * img)
         yr = 0;
         pdftex_warn("ext1: too large image resolution ignored");
     }
-    if (((img_transform(img) - img_rotation(idict)) & 1) == 1) {
+    if (((transform - img_rotation(idict)) & 1) == 1) {
         tmp = x;
         x = y;
         y = tmp;
@@ -397,186 +542,29 @@ void scale_img(image * img)
         xr = yr;
         yr = tmp;
     }
+    nat.dp = 0;                 /* always for images */
     if (img_type(idict) == IMG_TYPE_PDF
         || img_type(idict) == IMG_TYPE_PDFSTREAM) {
-        w = x;
-        h = y;
+        nat.wd = x;
+        nat.ht = y;
     } else {
-        default_res = fix_int(get_pdf_image_resolution(), 0, 65535);
+        default_res = fix_int(pdf_image_resolution, 0, 65535);
         if (default_res > 0 && (xr == 0 || yr == 0)) {
             xr = default_res;
             yr = default_res;
         }
-        if (is_wd_running(img) && is_ht_running(img)) {
-            if (xr > 0 && yr > 0) {
-                w = ext_xn_over_d(one_hundred_inch, x, 100 * xr);
-                h = ext_xn_over_d(one_hundred_inch, y, 100 * yr);
-            } else {
-                w = ext_xn_over_d(one_hundred_inch, x, 7200);
-                h = ext_xn_over_d(one_hundred_inch, y, 7200);
-            }
-        }
-    }
-    if (is_wd_running(img) && is_ht_running(img) && is_dp_running(img)) {
-        img_width(img) = w;
-        img_height(img) = h;
-        img_depth(img) = 0;
-    } else if (is_wd_running(img)) {
-        /* image depth or height is explicitly specified */
-        if (is_ht_running(img)) {
-            /* image depth is explicitly specified */
-            img_width(img) = ext_xn_over_d(h, x, y);
-            img_height(img) = h - img_depth(img);
-        } else if (is_dp_running(img)) {
-            /* image height is explicitly specified */
-            img_width(img) = ext_xn_over_d(img_height(img), x, y);
-            img_depth(img) = 0;
+        if (xr > 0 && yr > 0) {
+            nat.wd = ext_xn_over_d(one_hundred_inch, x, 100 * xr);
+            nat.ht = ext_xn_over_d(one_hundred_inch, y, 100 * yr);
         } else {
-            /* both image depth and height are explicitly specified */
-            img_width(img) =
-                ext_xn_over_d(img_height(img) + img_depth(img), x, y);
+            nat.wd = ext_xn_over_d(one_hundred_inch, x, 7200);
+            nat.ht = ext_xn_over_d(one_hundred_inch, y, 7200);
         }
-    } else {
-        /* image width is explicitly specified */
-        if (is_ht_running(img) && is_dp_running(img)) {
-            /* both image depth and height are not specified */
-            img_height(img) = ext_xn_over_d(img_width(img), y, x);
-            img_depth(img) = 0;
-        }
-        /* image depth is explicitly specified */
-        else if (is_ht_running(img)) {
-            img_height(img) =
-                ext_xn_over_d(img_width(img), y, x) - img_depth(img);
-        }
-        /* image height is explicitly specified */
-        else if (is_dp_running(img)) {
-            img_depth(img) = 0;
-        }
-        /* else both image depth and height are explicitly specified */
     }
-    img_set_scaled(img);
+    return tex_scale(nat, alt_rule);
 }
 
-void out_img(image * img, pdfstructure * p, scaledpos * pos)
-{
-    float a[6];                 /* transformation matrix */
-    float xoff, yoff, tmp;
-    int r;                      /* number of digits after the decimal point */
-    int k;
-    scaled wd, ht, dp;
-    scaledpos tmppos;
-    pdffloat cm[6];
-    image_dict *idict;
-    integer groupref;           /* added from web for 1.40.8 */
-    assert(img != 0);
-    idict = img_dict(img);
-    assert(idict != 0);
-    wd = img_width(img);
-    ht = img_height(img);
-    dp = img_depth(img);
-    a[0] = a[3] = 1.0e6;
-    a[1] = a[2] = 0;
-    if (img_type(idict) == IMG_TYPE_PDF
-        || img_type(idict) == IMG_TYPE_PDFSTREAM) {
-        a[0] /= img_xsize(idict);
-        a[3] /= img_ysize(idict);
-        xoff = (float) img_xorig(idict) / img_xsize(idict);
-        yoff = (float) img_yorig(idict) / img_ysize(idict);
-        r = 6;
-    } else {
-        /* added from web for 1.40.8 */
-        if (img_type(idict) == IMG_TYPE_PNG) {
-            groupref = img_group_ref(idict);
-            if ((groupref > 0) && (pdf_page_group_val < 1))
-                pdf_page_group_val = groupref;
-        }
-        /* /added from web */
-        a[0] /= one_hundred_bp;
-        a[3] = a[0];
-        xoff = yoff = 0;
-        r = 4;
-    }
-    if ((img_transform(img) & 7) > 3) { // mirror cases
-        a[0] *= -1;
-        xoff *= -1;
-    }
-    switch ((img_transform(img) + img_rotation(idict)) & 3) {
-    case 0:                    /* no transform */
-        break;
-    case 1:                    /* rot. 90 deg. (counterclockwise) */
-        a[1] = a[0];
-        a[2] = -a[3];
-        a[3] = a[0] = 0;
-        tmp = yoff;
-        yoff = xoff;
-        xoff = -tmp;
-        break;
-    case 2:                    /* rot. 180 deg. (counterclockwise) */
-        a[0] *= -1;
-        a[3] *= -1;
-        xoff *= -1;
-        yoff *= -1;
-        break;
-    case 3:                    /* rot. 270 deg. (counterclockwise) */
-        a[1] = -a[0];
-        a[2] = a[3];
-        a[3] = a[0] = 0;
-        tmp = yoff;
-        yoff = -xoff;
-        xoff = tmp;
-        break;
-    default:;
-    }
-    xoff *= wd;
-    yoff *= ht + dp;
-    a[0] *= wd;
-    a[1] *= ht + dp;
-    a[2] *= wd;
-    a[3] *= ht + dp;
-    a[4] = pos->h - xoff;
-    a[5] = pos->v - yoff;
-    k = img_transform(img) + img_rotation(idict);
-    if ((img_transform(img) & 7) > 3)
-        k++;
-    switch (k & 3) {
-    case 0:                    /* no transform */
-        break;
-    case 1:                    /* rot. 90 deg. (counterclockwise) */
-        a[4] += wd;
-        break;
-    case 2:                    /* rot. 180 deg. */
-        a[4] += wd;
-        a[5] += ht + dp;
-        break;
-    case 3:                    /* rot. 270 deg. */
-        a[5] += ht + dp;
-        break;
-    default:;
-    }
-    /* the following is a kludge, TODO: use pdfpage.c functions */
-    setpdffloat(cm[0], round(a[0]), r);
-    setpdffloat(cm[1], round(a[1]), r);
-    setpdffloat(cm[2], round(a[2]), r);
-    setpdffloat(cm[3], round(a[3]), r);
-    tmppos.h = round(a[4]);
-    tmppos.v = round(a[5]);
-    (void) calc_pdfpos(p, &tmppos);
-    cm[4] = p->cm[4];
-    cm[5] = p->cm[5];
-    pdf_goto_pagemode();
-    if (pdf_page_group_val < 1)
-        pdf_page_group_val = img_group_ref(idict);      /* added from web for 1.40.8 */
-    pdf_printf("q\n");
-    pdf_print_cm(cm);
-    pdf_printf("/Im");
-    pdf_print_int(img_index(idict));
-    pdf_print_resname_prefix();
-    pdf_printf(" Do\nQ\n");
-    if (img_state(idict) < DICT_OUTIMG)
-        img_state(idict) = DICT_OUTIMG;
-}
-
-void write_img(image_dict * idict)
+void write_img(PDF pdf, image_dict * idict)
 {
     assert(idict != NULL);
     if (img_state(idict) < DICT_WRITTEN) {
@@ -584,20 +572,20 @@ void write_img(image_dict * idict)
             tex_printf(" <%s", img_filepath(idict));
         switch (img_type(idict)) {
         case IMG_TYPE_PNG:
-            write_png(idict);
+            write_png(pdf, idict);
             break;
         case IMG_TYPE_JPG:
-            write_jpg(idict);
+            write_jpg(pdf, idict);
             break;
         case IMG_TYPE_JBIG2:
-            write_jbig2(idict);
+            write_jbig2(pdf, idict);
             break;
         case IMG_TYPE_PDF:
-            write_epdf(idict);
+            write_epdf(pdf, idict);
             epdf_lastGroupObjectNum = img_group_ref(idict);
             break;
         case IMG_TYPE_PDFSTREAM:
-            write_pdfstream(idict);
+            write_pdfstream(pdf, idict);
             break;
         default:
             pdftex_fail("internal error: unknown image type (1)");
@@ -605,10 +593,10 @@ void write_img(image_dict * idict)
         if (tracefilenames)
             tex_printf(">");
         if (img_type(idict) == IMG_TYPE_PDF) {
-            write_additional_epdf_objects();
+            write_additional_epdf_objects(pdf, img_filepath(idict));
         } else {
             if (img_type(idict) == IMG_TYPE_PNG) {
-                write_additional_png_objects();
+                write_additional_png_objects(pdf);
             }
         }
     }
@@ -616,6 +604,14 @@ void write_img(image_dict * idict)
         img_state(idict) = DICT_WRITTEN;
 }
 
+/* write an image */
+void pdf_write_image(PDF pdf, int n)
+{
+    pdf_begin_dict(pdf, n, 0);
+    if (pdf->draftmode == 0)
+        write_img(pdf, idict_array[obj_data_ptr(pdf, n)]);
+}
+
 /**********************************************************************/
 
 void check_pdfstream_dict(image_dict * idict)
@@ -626,39 +622,43 @@ void check_pdfstream_dict(image_dict * idict)
         img_state(idict) = DICT_FILESCANNED;
 }
 
-void write_pdfstream(image_dict * idict)
+void write_pdfstream(PDF pdf, image_dict * idict)
 {
     char s[256];
     assert(img_pdfstream_ptr(idict) != NULL);
     assert(img_is_bbox(idict));
-    pdf_puts("/Type /XObject\n/Subtype /Form\n");
+    pdf_puts(pdf, "/Type /XObject\n/Subtype /Form\n");
     if (img_attr(idict) != NULL && strlen(img_attr(idict)) > 0)
-        pdf_printf("%s\n", img_attr(idict));
-    pdf_puts("/FormType 1\n");
+        pdf_printf(pdf, "%s\n", img_attr(idict));
+    pdf_puts(pdf, "/FormType 1\n");
     sprintf(s, "/BBox [%.8f %.8f %.8f %.8f]\n", int2bp(img_bbox(idict)[0]),
             int2bp(img_bbox(idict)[1]), int2bp(img_bbox(idict)[2]),
             int2bp(img_bbox(idict)[3]));
-    pdf_printf(stripzeros(s));
-    pdf_begin_stream();
+    pdf_printf(pdf, stripzeros(s));
+    pdf_begin_stream(pdf);
     if (img_pdfstream_stream(idict) != NULL)
-        pdf_puts(img_pdfstream_stream(idict));
-    pdf_end_stream();
+        pdf_puts(pdf, img_pdfstream_stream(idict));
+    pdf_end_stream(pdf);
 }
 
 /**********************************************************************/
 
-typedef image *img_entry;
-/* define img_ptr, img_array, & img_limit */
+/* define idict_ptr, idict_array, & idict_limit */
 /* avoid use of size_t */
-img_entry *img_ptr, *img_array = NULL;
-size_t img_limit;
+idict_entry *idict_ptr, *idict_array = NULL;
+size_t idict_limit;
 
-integer img_to_array(image * img)
+void idict_to_array(image_dict * idict)
 {
-    assert(img != NULL);
-    alloc_array(img, 1, SMALL_BUF_SIZE);
-    *img_ptr = img;
-    return img_ptr++ - img_array;       /* now img is read-only */
+    assert(idict != NULL);
+    if (idict_ptr - idict_array == 0) { /* align to count from 1 */
+        alloc_array(idict, 1, SMALL_BUF_SIZE);  /* /Im0 unused */
+        idict_ptr++;
+    }
+    alloc_array(idict, 1, SMALL_BUF_SIZE);
+    *idict_ptr = idict;
+    assert(img_index(idict) == idict_ptr - idict_array);
+    idict_ptr++;
 }
 
 /**********************************************************************/
@@ -690,48 +690,47 @@ integer img_to_array(image * img)
  * makes the code simpler.
  */
 
-#define dumpcharptr(a)				\
-  do {						\
-    integer x;					\
-    if (a!=NULL) {				\
-      x = strlen(a)+1;				\
-      dumpinteger(x);  dump_things(*a, x);	\
-    } else {					\
-      x = 0; dumpinteger(x);			\
-    }						\
+#define dumpcharptr(a)                          \
+  do {                                          \
+    int x;                                      \
+    if (a!=NULL) {                              \
+      x = strlen(a)+1;                          \
+      dumpinteger(x);  dump_things(*a, x);      \
+    } else {                                    \
+      x = 0; dumpinteger(x);                    \
+    }                                           \
   } while (0)
 
-#define undumpcharptr(s)			\
-  do {						\
-    integer x;					\
-    char *a;					\
-    undumpinteger (x);				\
-    if (x>0) {					\
-      a = malloc(x);				\
-      undump_things(*a,x);			\
-      s = a ;					\
-    } else { s = NULL; }			\
+#define undumpcharptr(s)                        \
+  do {                                          \
+    int x;                                      \
+    char *a;                                    \
+    undumpinteger (x);                          \
+    if (x>0) {                                  \
+      a = malloc(x);                            \
+      undump_things(*a,x);                      \
+      s = a ;                                   \
+    } else { s = NULL; }                        \
   } while (0)
 
 void dumpimagemeta(void)
 {
-    int cur_image, i;
-    image *img;
+    int cur_index, i;
     image_dict *idict;
 
-    dumpinteger(img_limit);
-    cur_image = (img_ptr - img_array);
-    dumpinteger(cur_image);
+    i = (int) idict_limit;
+    dumpinteger(i);
+    cur_index = (idict_ptr - idict_array);
+    dumpinteger(cur_index);
 
-    for (i = 0; i < cur_image; i++) {
-        img = img_array[i];
-        assert(img != NULL);
-        idict = img_dict(img);
+    for (i = 1; i < cur_index; i++) {
+        idict = idict_array[i];
+        assert(idict != NULL);
         dumpcharptr(img_filename(idict));
         dumpinteger(img_type(idict));
         dumpinteger(img_color(idict));
-        dumpinteger(img_width(img));
-        dumpinteger(img_height(img));
+        dumpinteger(img_xsize(idict));
+        dumpinteger(img_ysize(idict));
         dumpinteger(img_xres(idict));
         dumpinteger(img_yres(idict));
         dumpinteger(img_totalpages(idict));
@@ -751,29 +750,29 @@ void dumpimagemeta(void)
     }
 }
 
-void undumpimagemeta(integer pdfversion, integer pdfinclusionerrorlevel)
+void undumpimagemeta(PDF pdf, int pdfversion, int pdfinclusionerrorlevel)
 {
-    int cur_image, i;
-    image *img;
+    int cur_index, i;
     image_dict *idict;
 
-    undumpinteger(img_limit);
+    assert(pdf != NULL);
+    undumpinteger(i);
+    idict_limit = i;
 
-    img_array = xtalloc(img_limit, img_entry);
-    undumpinteger(cur_image);
-    img_ptr = img_array + cur_image;
+    idict_array = xtalloc(idict_limit, idict_entry);
+    undumpinteger(cur_index);
+    idict_ptr = idict_array + cur_index;
 
-    for (i = 0; i < cur_image; i++) {
-        img = new_image();
-        assert(img_arrayidx(img) == -1);
-        img_arrayidx(img) = img_to_array(img);
-        idict = img_dict(img) = new_image_dict();
+    for (i = 1; i < cur_index; i++) {
+        idict = new_image_dict();
         assert(idict != NULL);
+        assert(img_index(idict) == -1);
+        idict_to_array(idict);
         undumpcharptr(img_filename(idict));
         undumpinteger(img_type(idict));
         undumpinteger(img_color(idict));
-        undumpinteger(img_width(img));
-        undumpinteger(img_height(img));
+        undumpinteger(img_xsize(idict));
+        undumpinteger(img_ysize(idict));
         undumpinteger(img_xres(idict));
         undumpinteger(img_yres(idict));
         undumpinteger(img_totalpages(idict));
@@ -800,158 +799,32 @@ void undumpimagemeta(integer pdfversion, integer pdfinclusionerrorlevel)
         default:
             pdftex_fail("unknown type of image");
         }
-        read_img(idict, pdfversion, pdfinclusionerrorlevel);
+        read_img(pdf, idict, pdfversion, pdfinclusionerrorlevel);
     }
 }
 
-/**********************************************************************/
-/* stuff to be accessible from TeX */
-
-integer read_image(integer objnum, integer index, str_number filename,
-                   integer page_num, str_number page_name, str_number attr,
-                   integer colorspace, integer page_box,
-                   integer pdf_minor_version, integer pdf_inclusion_errorlevel)
+/* scans rule spec to |alt_rule| */
+scaled_whd scan_alt_rule(void)
 {
-    image_dict *idict;
-    image *a = new_image();
-    assert(img_arrayidx(a) == -1);
-    img_arrayidx(a) = img_to_array(a);
-    idict = img_dict(a) = new_image_dict();
-    assert(idict != NULL);
-    img_objnum(idict) = objnum;
-    img_index(idict) = index;
-    /* img_xsize, img_ysize, img_xres, img_yres set by read_img() */
-    img_colorspace(idict) = colorspace;
-    img_pagenum(idict) = page_num;
-    /* img_totalpages set by read_img() */
-    if (page_name != 0)
-        img_pagename(idict) = xstrdup(makecstring(page_name));
-    cur_file_name = makecfilename(filename);
-    assert(cur_file_name != NULL);
-    img_filename(idict) = xstrdup(cur_file_name);
-    if (attr != 0)
-        img_attr(idict) = xstrdup(makecstring(attr));
-    img_pagebox(idict) = page_box;
-    read_img(idict, pdf_minor_version, pdf_inclusion_errorlevel);
-    img_unset_scaled(a);
-    return img_arrayidx(a);
-}
-
-void set_image_dimensions(integer ref, integer wd, integer ht, integer dp)
-{
-    image *a = img_array[ref];
-    img_width(a) = wd;
-    img_height(a) = ht;
-    img_depth(a) = dp;
-}
-
-void scale_image(integer ref)
-{
-    scale_img(img_array[ref]);
-}
-
-void out_image(integer ref, scaled hpos, scaled vpos)
-{
-    image *a = img_array[ref];
-    scaledpos pos;
-    pos.h = hpos;
-    pos.v = vpos;
-    out_img(a, pstruct, &pos);
-}
-
-void write_image(integer ref)
-{
-    write_img(img_dict(img_array[ref]));
-}
-
-integer image_pages(integer ref)
-{
-    return img_totalpages(img_dict(img_array[ref]));
-}
-
-integer image_colordepth(integer ref)
-{
-    return img_colordepth(img_dict(img_array[ref]));
-}
-
-integer image_group_ref(integer ref)
-{
-    return img_group_ref(img_dict(img_array[ref]));
-}
-
-/* The following five functions are for \pdfximagebbox */
-
-integer epdf_xsize(integer ref)
-{
-    return img_xsize(img_dict(img_array[ref]));
-}
-
-integer epdf_ysize(integer ref)
-{
-    return img_ysize(img_dict(img_array[ref]));
-}
-
-integer epdf_orig_x(integer ref)
-{
-    return img_xorig(img_dict(img_array[ref]));
-}
-
-integer epdf_orig_y(integer ref)
-{
-    return img_yorig(img_dict(img_array[ref]));
-}
-
-boolean is_pdf_image(integer ref)
-{
-    return img_type(img_dict(img_array[ref])) == IMG_TYPE_PDF;
-}
-
-boolean is_png_image(integer ref)
-{
-    return img_type(img_dict(img_array[ref])) == IMG_TYPE_PNG;
-}
-
-integer image_objnum(integer ref)
-{
-    return img_objnum(img_dict(img_array[ref]));
-}
-
-integer image_index(integer ref)
-{
-    return img_index(img_dict(img_array[ref]));
-}
-
-integer image_width(integer ref)
-{
-    return img_width(img_array[ref]);
-}
-
-integer image_height(integer ref)
-{
-    return img_height(img_array[ref]);
-}
-
-integer image_depth(integer ref)
-{
-    return img_depth(img_array[ref]);
-}
-
-void update_image_procset(integer ref)
-{
-    pdf_image_procset |= img_color(img_dict(img_array[ref]));
-}
-
-boolean check_image_b(integer procset)
-{
-    return procset & IMAGE_COLOR_B;
-}
-
-boolean check_image_c(integer procset)
-{
-    return procset & IMAGE_COLOR_C;
-}
-
-boolean check_image_i(integer procset)
-{
-    return procset & IMAGE_COLOR_I;
+    scaled_whd alt_rule;
+    alt_rule.wd = null_flag;
+    alt_rule.ht = null_flag;
+    alt_rule.dp = null_flag;
+  RESWITCH:
+    if (scan_keyword("width")) {
+        scan_normal_dimen();
+        alt_rule.wd = cur_val;
+        goto RESWITCH;
+    }
+    if (scan_keyword("height")) {
+        scan_normal_dimen();
+        alt_rule.ht = cur_val;
+        goto RESWITCH;
+    }
+    if (scan_keyword("depth")) {
+        scan_normal_dimen();
+        alt_rule.dp = cur_val;
+        goto RESWITCH;
+    }
+    return alt_rule;
 }
