@@ -23,8 +23,8 @@
 #include "image.h"
 
 static const char _svn_version[] =
-    "$Id: writepng.c 2317 2009-04-18 00:12:18Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/image/writepng.c $";
+    "$Id: writepng.c 3261 2009-12-18 11:38:21Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/image/writepng.c $";
 
 static int transparent_page_group = -1;
 
@@ -42,7 +42,7 @@ static void close_and_cleanup_png(image_dict * idict)
     img_png_ptr(idict) = NULL;
 }
 
-void read_png_info(image_dict * idict, img_readtype_e readtype)
+void read_png_info(PDF pdf, image_dict * idict, img_readtype_e readtype)
 {
     double gamma;
     png_structp png_p;
@@ -72,20 +72,20 @@ void read_png_info(image_dict * idict, img_readtype_e readtype)
         png_set_tRNS_to_alpha(png_p);
     }
     /* alpha channel support  */
-    if (fixed_pdf_minor_version < 4 && png_p->color_type | PNG_COLOR_MASK_ALPHA)
+    if (pdf->minor_version < 4 && png_p->color_type | PNG_COLOR_MASK_ALPHA)
         png_set_strip_alpha(png_p);
     /* 16bit depth support */
-    if (fixed_pdf_minor_version < 5)
-        fixed_image_hicolor = 0;
-    if (info_p->bit_depth == 16 && !fixed_image_hicolor)
+    if (pdf->minor_version < 5)
+        pdf->image_hicolor = 0;
+    if ((info_p->bit_depth == 16) && (pdf->image_hicolor == 0))
         png_set_strip_16(png_p);
     /* gamma support */
-    if (fixed_image_apply_gamma) {
+    if (pdf->image_apply_gamma) {
         if (png_get_gAMA(png_p, info_p, &gamma))
-            png_set_gamma(png_p, (fixed_gamma / 1000.0), gamma);
+            png_set_gamma(png_p, (pdf->gamma / 1000.0), gamma);
         else
-            png_set_gamma(png_p, (fixed_gamma / 1000.0),
-                          (1000.0 / fixed_image_gamma));
+            png_set_gamma(png_p, (pdf->gamma / 1000.0),
+                          (1000.0 / pdf->image_gamma));
     }
     /* reset structure */
     png_read_update_info(png_p, info_p);
@@ -119,22 +119,22 @@ void read_png_info(image_dict * idict, img_readtype_e readtype)
 }
 
 #define write_gray_pixel_16(r)                           \
-  if (j % 4 == 0||j % 4 == 1) pdf_buf[pdf_ptr++] = *r++; \
+    if (j % 4 == 0||j % 4 == 1) pdf_quick_out(pdf,*r++); \
   else                        smask[smask_ptr++] = *r++
 
 #define write_gray_pixel_8(r)                   \
-    if (j % 2 == 0)  pdf_buf[pdf_ptr++] = *r++; \
+    if (j % 2 == 0)  pdf_quick_out(pdf,*r++);   \
     else             smask[smask_ptr++] = *r++
 
 #define write_rgb_pixel_16(r)                                  \
-    if (!(j % 8 == 6||j % 8 == 7)) pdf_buf[pdf_ptr++]  = *r++; \
+    if (!(j % 8 == 6||j % 8 == 7)) pdf_quick_out(pdf,*r++);    \
     else                           smask[smask_ptr++] = *r++
 
 #define write_rgb_pixel_8(r)                   \
-    if (j % 4 != 3) pdf_buf[pdf_ptr++] = *r++; \
+    if (j % 4 != 3) pdf_quick_out(pdf,*r++);   \
     else            smask[smask_ptr++] = *r++
 
-#define write_simple_pixel(r)    pdf_buf[pdf_ptr++] = *r++
+#define write_simple_pixel(r)    pdf_quick_out(pdf,*r++)
 
 #define write_noninterlaced(outmac)                      \
     for (i = 0; i < (int)info_p->height; i++) {          \
@@ -142,8 +142,8 @@ void read_png_info(image_dict * idict, img_readtype_e readtype)
     r = row;                                             \
     k = info_p->rowbytes;                                \
     while(k > 0) {                                       \
-        l = (k > pdf_buf_size)? pdf_buf_size : k;        \
-                pdfroom(l);                              \
+        l = (k > pdf->buf_size)? pdf->buf_size : k;      \
+                pdf_room(pdf,l);                         \
                 for (j = 0; j < l; j++) {                \
                   outmac;                                \
                 }                                        \
@@ -156,8 +156,8 @@ void read_png_info(image_dict * idict, img_readtype_e readtype)
             row = rows[i];                               \
             k = info_p->rowbytes;                        \
             while(k > 0) {                               \
-                l = (k > pdf_buf_size)? pdf_buf_size : k;\
-                pdfroom(l);                              \
+                l = (k > pdf->buf_size)?pdf->buf_size: k;\
+                pdf_room(pdf,l);                         \
                 for (j = 0; j < l; j++) {                \
                   outmac;                                \
                 }                                        \
@@ -166,22 +166,22 @@ void read_png_info(image_dict * idict, img_readtype_e readtype)
             xfree(rows[i]);                              \
         }
 
-static void write_png_palette(image_dict * idict)
+static void write_png_palette(PDF pdf, image_dict * idict)
 {
     int i, j, k, l;
     png_structp png_p = img_png_png_ptr(idict);
     png_infop info_p = img_png_info_ptr(idict);
     png_bytep row, r, *rows;
-    integer palette_objnum = 0;
+    int palette_objnum = 0;
     if (img_colorspace(idict) != 0) {
-        pdf_printf("%i 0 R\n", (int) img_colorspace(idict));
+        pdf_printf(pdf, "%i 0 R\n", (int) img_colorspace(idict));
     } else {
-        pdf_create_obj(0, 0);
-        palette_objnum = obj_ptr;
-        pdf_printf("[/Indexed /DeviceRGB %i %i 0 R]\n",
+        pdf_create_obj(pdf, obj_type_others, 0);
+        palette_objnum = pdf->obj_ptr;
+        pdf_printf(pdf, "[/Indexed /DeviceRGB %i %i 0 R]\n",
                    (int) (info_p->num_palette - 1), (int) palette_objnum);
     }
-    pdf_begin_stream();
+    pdf_begin_stream(pdf);
     if (info_p->interlace_type == PNG_INTERLACE_NONE) {
         row = xtalloc(info_p->rowbytes, png_byte);
         write_noninterlaced(write_simple_pixel(r));
@@ -197,32 +197,32 @@ static void write_png_palette(image_dict * idict)
         write_interlaced(write_simple_pixel(row));
         xfree(rows);
     }
-    pdf_end_stream();
+    pdf_end_stream(pdf);
     if (palette_objnum > 0) {
-        pdf_begin_dict(palette_objnum, 0);
-        pdf_begin_stream();
+        pdf_begin_dict(pdf, palette_objnum, 0);
+        pdf_begin_stream(pdf);
         for (i = 0; (unsigned) i < info_p->num_palette; i++) {
-            pdfroom(3);
-            pdf_buf[pdf_ptr++] = info_p->palette[i].red;
-            pdf_buf[pdf_ptr++] = info_p->palette[i].green;
-            pdf_buf[pdf_ptr++] = info_p->palette[i].blue;
+            pdf_room(pdf, 3);
+            pdf_quick_out(pdf, info_p->palette[i].red);
+            pdf_quick_out(pdf, info_p->palette[i].green);
+            pdf_quick_out(pdf, info_p->palette[i].blue);
         }
-        pdf_end_stream();
+        pdf_end_stream(pdf);
     }
 }
 
-static void write_png_gray(image_dict * idict)
+static void write_png_gray(PDF pdf, image_dict * idict)
 {
     int i, j, k, l;
     png_structp png_p = img_png_png_ptr(idict);
     png_infop info_p = img_png_info_ptr(idict);
     png_bytep row, r, *rows;
     if (img_colorspace(idict) != 0) {
-        pdf_printf("%i 0 R\n", (int) img_colorspace(idict));
+        pdf_printf(pdf, "%i 0 R\n", (int) img_colorspace(idict));
     } else {
-        pdf_puts("/DeviceGray\n");
+        pdf_puts(pdf, "/DeviceGray\n");
     }
-    pdf_begin_stream();
+    pdf_begin_stream(pdf);
     if (info_p->interlace_type == PNG_INTERLACE_NONE) {
         row = xtalloc(info_p->rowbytes, png_byte);
         write_noninterlaced(write_simple_pixel(r));
@@ -238,34 +238,34 @@ static void write_png_gray(image_dict * idict)
         write_interlaced(write_simple_pixel(row));
         xfree(rows);
     }
-    pdf_end_stream();
+    pdf_end_stream(pdf);
 }
 
-static void write_png_gray_alpha(image_dict * idict)
+static void write_png_gray_alpha(PDF pdf, image_dict * idict)
 {
     int i, j, k, l;
     png_structp png_p = img_png_png_ptr(idict);
     png_infop info_p = img_png_info_ptr(idict);
     png_bytep row, r, *rows;
-    integer smask_objnum = 0;
+    int smask_objnum = 0;
     png_bytep smask;
-    integer smask_ptr = 0;
-    integer smask_size = 0;
+    int smask_ptr = 0;
+    int smask_size = 0;
     int bitdepth;
     if (img_colorspace(idict) != 0) {
-        pdf_printf("%i 0 R\n", (int) img_colorspace(idict));
+        pdf_printf(pdf, "%i 0 R\n", (int) img_colorspace(idict));
     } else {
-        pdf_puts("/DeviceGray\n");
+        pdf_puts(pdf, "/DeviceGray\n");
     }
-    pdf_create_obj(0, 0);
-    smask_objnum = obj_ptr;
-    pdf_printf("/SMask %i 0 R\n", (int) smask_objnum);
+    pdf_create_obj(pdf, obj_type_others, 0);
+    smask_objnum = pdf->obj_ptr;
+    pdf_printf(pdf, "/SMask %i 0 R\n", (int) smask_objnum);
     smask_size = (info_p->rowbytes / 2) * info_p->height;
     smask = xtalloc(smask_size, png_byte);
-    pdf_begin_stream();
+    pdf_begin_stream(pdf);
     if (info_p->interlace_type == PNG_INTERLACE_NONE) {
         row = xtalloc(info_p->rowbytes, png_byte);
-        if ((info_p->bit_depth == 16) && fixed_image_hicolor) {
+        if ((info_p->bit_depth == 16) && (pdf->image_hicolor != 0)) {
             write_noninterlaced(write_gray_pixel_16(r));
         } else {
             write_noninterlaced(write_gray_pixel_8(r));
@@ -279,49 +279,49 @@ static void write_png_gray_alpha(image_dict * idict)
         for (i = 0; (unsigned) i < info_p->height; i++)
             rows[i] = xtalloc(info_p->rowbytes, png_byte);
         png_read_image(png_p, rows);
-        if ((info_p->bit_depth == 16) && fixed_image_hicolor) {
+        if ((info_p->bit_depth == 16) && (pdf->image_hicolor != 0)) {
             write_interlaced(write_gray_pixel_16(row));
         } else {
             write_interlaced(write_gray_pixel_8(row));
         }
         xfree(rows);
     }
-    pdf_end_stream();
-    pdf_flush();
+    pdf_end_stream(pdf);
+    pdf_flush(pdf);
     /* now write the Smask object */
     bitdepth = (int) info_p->bit_depth;
-    pdf_begin_dict(smask_objnum, 0);
-    pdf_puts("/Type /XObject\n/Subtype /Image\n");
+    pdf_begin_dict(pdf, smask_objnum, 0);
+    pdf_puts(pdf, "/Type /XObject\n/Subtype /Image\n");
     if (img_attr(idict) != NULL && strlen(img_attr(idict)) > 0)
-        pdf_printf("%s\n", img_attr(idict));
-    pdf_printf("/Width %i\n/Height %i\n/BitsPerComponent %i\n",
+        pdf_printf(pdf, "%s\n", img_attr(idict));
+    pdf_printf(pdf, "/Width %i\n/Height %i\n/BitsPerComponent %i\n",
                (int) info_p->width,
                (int) info_p->height, (bitdepth == 16 ? 8 : bitdepth));
-    pdf_puts("/ColorSpace /DeviceGray\n");
-    pdf_begin_stream();
+    pdf_puts(pdf, "/ColorSpace /DeviceGray\n");
+    pdf_begin_stream(pdf);
     for (i = 0; i < smask_size; i++) {
         if (i % 8 == 0)
-            pdfroom(8);
-        pdf_buf[pdf_ptr++] = smask[i];
+            pdf_room(pdf, 8);
+        pdf_quick_out(pdf, smask[i]);
         if (bitdepth == 16)
             i++;
     }
-    pdf_end_stream();
+    pdf_end_stream(pdf);
     xfree(smask);
 }
 
-static void write_png_rgb(image_dict * idict)
+static void write_png_rgb(PDF pdf, image_dict * idict)
 {
     int i, j, k, l;
     png_structp png_p = img_png_png_ptr(idict);
     png_infop info_p = img_png_info_ptr(idict);
     png_bytep row, r, *rows;
     if (img_colorspace(idict) != 0) {
-        pdf_printf("%i 0 R\n", (int) img_colorspace(idict));
+        pdf_printf(pdf, "%i 0 R\n", (int) img_colorspace(idict));
     } else {
-        pdf_puts("/DeviceRGB\n");
+        pdf_puts(pdf, "/DeviceRGB\n");
     }
-    pdf_begin_stream();
+    pdf_begin_stream(pdf);
     if (info_p->interlace_type == PNG_INTERLACE_NONE) {
         row = xtalloc(info_p->rowbytes, png_byte);
         write_noninterlaced(write_simple_pixel(r));
@@ -337,34 +337,34 @@ static void write_png_rgb(image_dict * idict)
         write_interlaced(write_simple_pixel(row));
         xfree(rows);
     }
-    pdf_end_stream();
+    pdf_end_stream(pdf);
 }
 
-static void write_png_rgb_alpha(image_dict * idict)
+static void write_png_rgb_alpha(PDF pdf, image_dict * idict)
 {
     int i, j, k, l;
     png_structp png_p = img_png_png_ptr(idict);
     png_infop info_p = img_png_info_ptr(idict);
     png_bytep row, r, *rows;
-    integer smask_objnum = 0;
+    int smask_objnum = 0;
     png_bytep smask;
-    integer smask_ptr = 0;
-    integer smask_size = 0;
+    int smask_ptr = 0;
+    int smask_size = 0;
     int bitdepth;
     if (img_colorspace(idict) != 0) {
-        pdf_printf("%i 0 R\n", (int) img_colorspace(idict));
+        pdf_printf(pdf, "%i 0 R\n", (int) img_colorspace(idict));
     } else {
-        pdf_puts("/DeviceRGB\n");
+        pdf_puts(pdf, "/DeviceRGB\n");
     }
-    pdf_create_obj(0, 0);
-    smask_objnum = obj_ptr;
-    pdf_printf("/SMask %i 0 R\n", (int) smask_objnum);
+    pdf_create_obj(pdf, obj_type_others, 0);
+    smask_objnum = pdf->obj_ptr;
+    pdf_printf(pdf, "/SMask %i 0 R\n", (int) smask_objnum);
     smask_size = (info_p->rowbytes / 2) * info_p->height;
     smask = xtalloc(smask_size, png_byte);
-    pdf_begin_stream();
+    pdf_begin_stream(pdf);
     if (info_p->interlace_type == PNG_INTERLACE_NONE) {
         row = xtalloc(info_p->rowbytes, png_byte);
-        if ((info_p->bit_depth == 16) && fixed_image_hicolor) {
+        if ((info_p->bit_depth == 16) && (pdf->image_hicolor != 0)) {
             write_noninterlaced(write_rgb_pixel_16(r));
         } else {
             write_noninterlaced(write_rgb_pixel_8(r));
@@ -378,36 +378,36 @@ static void write_png_rgb_alpha(image_dict * idict)
         for (i = 0; (unsigned) i < info_p->height; i++)
             rows[i] = xtalloc(info_p->rowbytes, png_byte);
         png_read_image(png_p, rows);
-        if ((info_p->bit_depth == 16) && fixed_image_hicolor) {
+        if ((info_p->bit_depth == 16) && (pdf->image_hicolor != 0)) {
             write_interlaced(write_rgb_pixel_16(row));
         } else {
             write_interlaced(write_rgb_pixel_8(row));
         }
         xfree(rows);
     }
-    pdf_end_stream();
-    pdf_flush();
+    pdf_end_stream(pdf);
+    pdf_flush(pdf);
     /* now write the Smask object */
     if (smask_objnum > 0) {
         bitdepth = (int) info_p->bit_depth;
-        pdf_begin_dict(smask_objnum, 0);
-        pdf_puts("/Type /XObject\n/Subtype /Image\n");
+        pdf_begin_dict(pdf, smask_objnum, 0);
+        pdf_puts(pdf, "/Type /XObject\n/Subtype /Image\n");
         if (img_attr(idict) != NULL && strlen(img_attr(idict)) > 0)
-            pdf_printf("%s\n", img_attr(idict));
-        pdf_printf("/Width %i\n/Height %i\n/BitsPerComponent %i\n",
+            pdf_printf(pdf, "%s\n", img_attr(idict));
+        pdf_printf(pdf, "/Width %i\n/Height %i\n/BitsPerComponent %i\n",
                    (int) info_p->width,
                    (int) info_p->height, (bitdepth == 16 ? 8 : bitdepth));
-        pdf_puts("/ColorSpace /DeviceGray\n");
-        pdf_begin_stream();
+        pdf_puts(pdf, "/ColorSpace /DeviceGray\n");
+        pdf_begin_stream(pdf);
         for (i = 0; i < smask_size; i++) {
             if (i % 8 == 0)
-                pdfroom(8);
-            pdf_buf[pdf_ptr++] = smask[i];
+                pdf_room(pdf, 8);
+            pdf_quick_out(pdf, smask[i]);
             if (bitdepth == 16)
                 i++;
         }
         xfree(smask);
-        pdf_end_stream();
+        pdf_end_stream(pdf);
     }
 }
 
@@ -434,7 +434,7 @@ static int spng_getint(FILE * fp)
 #define SPNG_CHUNK_IDAT 0x49444154
 #define SPNG_CHUNK_IEND 0x49454E44
 
-static void copy_png(image_dict * idict)
+static void copy_png(PDF pdf, image_dict * idict)
 {
     png_structp png_p;
     png_infop info_p;
@@ -463,7 +463,7 @@ static void copy_png(image_dict * idict)
                 pdftex_fail("writepng: fseek in PNG file failed");
         }
     } while (endflag == false);
-    pdf_printf("/Length %d\n"
+    pdf_printf(pdf, "/Length %d\n"
                "/Filter/FlateDecode\n"
                "/DecodeParms<<"
                "/Colors %d"
@@ -485,17 +485,17 @@ static void copy_png(image_dict * idict)
                 pdftex_fail("writepng: IDAT chunk sequence broken");
             idat = 1;
             while (len > 0) {
-                i = (len > pdf_buf_size) ? pdf_buf_size : len;
-                pdfroom(i);
-                fread(&pdf_buf[pdf_ptr], 1, i, fp);
-                pdf_ptr += i;
+                i = (len > pdf->buf_size) ? pdf->buf_size : len;
+                pdf_room(pdf, i);
+                fread(&pdf->buf[pdf->ptr], 1, i, fp);
+                pdf->ptr += i;
                 len -= i;
             }
             if (fseek(fp, 4, SEEK_CUR) != 0)
                 pdftex_fail("writepng: fseek in PNG file failed");
             break;
         case SPNG_CHUNK_IEND:  /* done */
-            pdf_end_stream();
+            pdf_end_stream(pdf);
             endflag = true;
             break;
         default:
@@ -507,14 +507,14 @@ static void copy_png(image_dict * idict)
     } while (endflag == false);
 }
 
-static void reopen_png(image_dict * idict)
+static void reopen_png(PDF pdf, image_dict * idict)
 {
-    integer width, height, xres, yres;
+    int width, height, xres, yres;
     width = img_xsize(idict);   /* do consistency check */
     height = img_ysize(idict);
     xres = img_xres(idict);
     yres = img_yres(idict);
-    read_png_info(idict, IMG_KEEPOPEN);
+    read_png_info(pdf, idict, IMG_KEEPOPEN);
     if (width != img_xsize(idict) || height != img_ysize(idict)
         || xres != img_xres(idict) || yres != img_yres(idict))
         pdftex_fail("writepng: image dimensions have changed");
@@ -522,82 +522,81 @@ static void reopen_png(image_dict * idict)
 
 static boolean last_png_needs_page_group;
 
-void write_png(image_dict * idict)
+void write_png(PDF pdf, image_dict * idict)
 {
     double gamma, checked_gamma;
     int i;
-    integer palette_objnum = 0;
+    int palette_objnum = 0;
     png_structp png_p;
     png_infop info_p;
     assert(idict != NULL);
     last_png_needs_page_group = false;
     if (img_file(idict) == NULL)
-        reopen_png(idict);
+        reopen_png(pdf, idict);
     assert(img_png_ptr(idict) != NULL);
     png_p = img_png_png_ptr(idict);
     info_p = img_png_info_ptr(idict);
-    if (fixed_pdf_minor_version < 5)
-        fixed_image_hicolor = 0;
-    pdf_puts("/Type /XObject\n/Subtype /Image\n");
+    if (pdf->minor_version < 5)
+        pdf->image_hicolor = 0;
+    pdf_puts(pdf, "/Type /XObject\n/Subtype /Image\n");
     if (img_attr(idict) != NULL && strlen(img_attr(idict)) > 0)
-        pdf_printf("%s\n", img_attr(idict));
-    pdf_printf("/Width %i\n/Height %i\n/BitsPerComponent %i\n",
+        pdf_printf(pdf, "%s\n", img_attr(idict));
+    pdf_printf(pdf, "/Width %i\n/Height %i\n/BitsPerComponent %i\n",
                (int) info_p->width,
                (int) info_p->height, (int) info_p->bit_depth);
-    pdf_puts("/ColorSpace ");
+    pdf_puts(pdf, "/ColorSpace ");
     checked_gamma = 1.0;
-    if (fixed_image_apply_gamma) {
+    if (pdf->image_apply_gamma) {
         if (png_get_gAMA(png_p, info_p, &gamma)) {
-            checked_gamma = (fixed_gamma / 1000.0) * gamma;
+            checked_gamma = (pdf->gamma / 1000.0) * gamma;
         } else {
-            checked_gamma =
-                (fixed_gamma / 1000.0) * (1000.0 / fixed_image_gamma);
+            checked_gamma = (pdf->gamma / 1000.0) * (1000.0 / pdf->image_gamma);
         }
     }
     /* the switching between |info_p| and |png_p| queries has been trial and error.
      */
-    if (fixed_pdf_minor_version > 1 && info_p->interlace_type == PNG_INTERLACE_NONE && (png_p->transformations == 0 || png_p->transformations == 0x2000)        /* gamma */
+    if (pdf->minor_version > 1 && info_p->interlace_type == PNG_INTERLACE_NONE && (png_p->transformations == 0 || png_p->transformations == 0x2000)     /* gamma */
         &&!(png_p->color_type == PNG_COLOR_TYPE_GRAY_ALPHA ||
             png_p->color_type == PNG_COLOR_TYPE_RGB_ALPHA)
-        && (fixed_image_hicolor || (png_p->bit_depth <= 8))
+        && ((pdf->image_hicolor != 0) || (png_p->bit_depth <= 8))
         && (checked_gamma <= 1.01 && checked_gamma > 0.99)
         ) {
         if (img_colorspace(idict) != 0) {
-            pdf_printf("%i 0 R\n", (int) img_colorspace(idict));
+            pdf_printf(pdf, "%i 0 R\n", (int) img_colorspace(idict));
         } else {
             switch (info_p->color_type) {
             case PNG_COLOR_TYPE_PALETTE:
-                pdf_create_obj(0, 0);
-                palette_objnum = obj_ptr;
-                pdf_printf("[/Indexed /DeviceRGB %i %i 0 R]\n",
+                pdf_create_obj(pdf, obj_type_others, 0);
+                palette_objnum = pdf->obj_ptr;
+                pdf_printf(pdf, "[/Indexed /DeviceRGB %i %i 0 R]\n",
                            (int) (info_p->num_palette - 1),
                            (int) palette_objnum);
                 break;
             case PNG_COLOR_TYPE_GRAY:
-                pdf_puts("/DeviceGray\n");
+                pdf_puts(pdf, "/DeviceGray\n");
                 break;
             default:           /* RGB */
-                pdf_puts("/DeviceRGB\n");
+                pdf_puts(pdf, "/DeviceRGB\n");
             };
         }
         if (tracefilenames)
             tex_printf(" (PNG copy)");
-        copy_png(idict);
+        copy_png(pdf, idict);
         if (palette_objnum > 0) {
-            pdf_begin_dict(palette_objnum, 0);
-            pdf_begin_stream();
+            pdf_begin_dict(pdf, palette_objnum, 0);
+            pdf_begin_stream(pdf);
             for (i = 0; (unsigned) i < info_p->num_palette; i++) {
-                pdfroom(3);
-                pdf_buf[pdf_ptr++] = info_p->palette[i].red;
-                pdf_buf[pdf_ptr++] = info_p->palette[i].green;
-                pdf_buf[pdf_ptr++] = info_p->palette[i].blue;
+                pdf_room(pdf, 3);
+                pdf_quick_out(pdf, info_p->palette[i].red);
+                pdf_quick_out(pdf, info_p->palette[i].green);
+                pdf_quick_out(pdf, info_p->palette[i].blue);
             }
-            pdf_end_stream();
+            pdf_end_stream(pdf);
         }
     } else {
         if (0) {
             tex_printf(" PNG copy skipped because: ");
-            if (fixed_image_apply_gamma &&
+            if ((pdf->image_apply_gamma != 0) &&
                 (checked_gamma > 1.01 || checked_gamma < 0.99))
                 tex_printf("gamma delta=%lf ", checked_gamma);
             if (png_p->transformations != PNG_TRANSFORM_IDENTITY)
@@ -606,8 +605,8 @@ void write_png(image_dict * idict)
                 && (info_p->color_type != PNG_COLOR_TYPE_RGB)
                 && (info_p->color_type != PNG_COLOR_TYPE_PALETTE))
                 tex_printf("colortype ");
-            if (fixed_pdf_minor_version <= 1)
-                tex_printf("version=%d ", (int) fixed_pdf_minor_version);
+            if (pdf->minor_version <= 1)
+                tex_printf("version=%d ", pdf->minor_version);
             if (info_p->interlace_type != PNG_INTERLACE_NONE)
                 tex_printf("interlaced ");
             if (info_p->bit_depth > 8)
@@ -617,34 +616,34 @@ void write_png(image_dict * idict)
         }
         switch (info_p->color_type) {
         case PNG_COLOR_TYPE_PALETTE:
-            write_png_palette(idict);
+            write_png_palette(pdf, idict);
             break;
         case PNG_COLOR_TYPE_GRAY:
-            write_png_gray(idict);
+            write_png_gray(pdf, idict);
             break;
         case PNG_COLOR_TYPE_GRAY_ALPHA:
-            if (fixed_pdf_minor_version >= 4) {
-                write_png_gray_alpha(idict);
+            if (pdf->minor_version >= 4) {
+                write_png_gray_alpha(pdf, idict);
                 last_png_needs_page_group = true;
             } else
-                write_png_gray(idict);
+                write_png_gray(pdf, idict);
             break;
         case PNG_COLOR_TYPE_RGB:
-            write_png_rgb(idict);
+            write_png_rgb(pdf, idict);
             break;
         case PNG_COLOR_TYPE_RGB_ALPHA:
-            if (fixed_pdf_minor_version >= 4) {
-                write_png_rgb_alpha(idict);
+            if (pdf->minor_version >= 4) {
+                write_png_rgb_alpha(pdf, idict);
                 last_png_needs_page_group = true;
             } else
-                write_png_rgb(idict);
+                write_png_rgb(pdf, idict);
             break;
         default:
             pdftex_fail("unsupported type of color_type <%i>",
                         info_p->color_type);
         }
     }
-    pdf_flush();
+    pdf_flush(pdf);
     close_and_cleanup_png(idict);
 }
 
@@ -653,20 +652,21 @@ static boolean transparent_page_group_was_written = false;
 /* Called after the xobject generated by write_png has been finished; used to
  * write out additional objects */
 
-void write_additional_png_objects(void)
+void write_additional_png_objects(PDF pdf)
 {
     return;                     /* this interferes with current macro-based usage and cannot be configured */
     if (last_png_needs_page_group) {
         if (!transparent_page_group_was_written && transparent_page_group > 1) {
             /* create new group object */
             transparent_page_group_was_written = true;
-            pdf_begin_obj(transparent_page_group, 2);
-            if (get_pdf_compress_level() == 0) {
-                pdf_puts("%PTEX Group needed for transparent pngs\n");
+            pdf_begin_obj(pdf, transparent_page_group, 2);
+            if (pdf->compress_level == 0) {
+                pdf_puts(pdf, "%PTEX Group needed for transparent pngs\n");
             }
             pdf_puts
-                ("<</Type/Group /S/Transparency /CS/DeviceRGB /I true /K true>>\n");
-            pdf_end_obj();
+                (pdf,
+                 "<</Type/Group /S/Transparency /CS/DeviceRGB /I true /K true>>\n");
+            pdf_end_obj(pdf);
         }
     }
 }
