@@ -17,17 +17,16 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
+
 #include <ptexlib.h>
 
 #include <string.h>
 
-#include "nodes.h"
-#include "font/texfont.h"
-#include "hyphen.h"
+#include "lua/luatex-api.h"
+
 
 static const char _svn_version[] =
-    "$Id: texlang.c 2450 2009-06-08 08:30:52Z taco $ $URL: http://scm.foundry.supelec.fr/svn/luatex/trunk/src/texk/web2c/luatexdir/lang/texlang.c $";
+    "$Id: texlang.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://scm.foundry.supelec.fr/svn/luatex/trunk/src/texk/web2c/luatexdir/lang/texlang.c $";
 
 /* functions from the fontforge unicode library */
 
@@ -112,6 +111,7 @@ static int u_strlen(register unichar_t * str)
 #define MAX_TEX_LANGUAGES  32768
 
 static struct tex_language *tex_languages[MAX_TEX_LANGUAGES] = { NULL };
+
 static int next_lang_id = 0;
 
 struct tex_language *new_language(int n)
@@ -157,14 +157,14 @@ struct tex_language *get_language(int n)
     }
 }
 
-void set_pre_hyphen_char(integer n, integer v)
+void set_pre_hyphen_char(int n, int v)
 {
     struct tex_language *l = get_language((int) n);
     if (l != NULL)
         l->pre_hyphen_char = (int) v;
 }
 
-void set_post_hyphen_char(integer n, integer v)
+void set_post_hyphen_char(int n, int v)
 {
     struct tex_language *l = get_language((int) n);
     if (l != NULL)
@@ -172,14 +172,14 @@ void set_post_hyphen_char(integer n, integer v)
 }
 
 
-void set_pre_exhyphen_char(integer n, integer v)
+void set_pre_exhyphen_char(int n, int v)
 {
     struct tex_language *l = get_language((int) n);
     if (l != NULL)
         l->pre_exhyphen_char = (int) v;
 }
 
-void set_post_exhyphen_char(integer n, integer v)
+void set_post_exhyphen_char(int n, int v)
 {
     struct tex_language *l = get_language((int) n);
     if (l != NULL)
@@ -188,37 +188,37 @@ void set_post_exhyphen_char(integer n, integer v)
 
 
 
-integer get_pre_hyphen_char(integer n)
+int get_pre_hyphen_char(int n)
 {
     struct tex_language *l = get_language((int) n);
     if (l == NULL)
         return -1;
-    return (integer) l->pre_hyphen_char;
+    return (int) l->pre_hyphen_char;
 }
 
-integer get_post_hyphen_char(integer n)
+int get_post_hyphen_char(int n)
 {
     struct tex_language *l = get_language((int) n);
     if (l == NULL)
         return -1;
-    return (integer) l->post_hyphen_char;
+    return (int) l->post_hyphen_char;
 }
 
 
-integer get_pre_exhyphen_char(integer n)
+int get_pre_exhyphen_char(int n)
 {
     struct tex_language *l = get_language((int) n);
     if (l == NULL)
         return -1;
-    return (integer) l->pre_exhyphen_char;
+    return (int) l->pre_exhyphen_char;
 }
 
-integer get_post_exhyphen_char(integer n)
+int get_post_exhyphen_char(int n)
 {
     struct tex_language *l = get_language((int) n);
     if (l == NULL)
         return -1;
-    return (integer) l->post_exhyphen_char;
+    return (int) l->post_exhyphen_char;
 }
 
 void load_patterns(struct tex_language *lang, unsigned char *buffer)
@@ -247,12 +247,10 @@ void load_tex_patterns(int curlang, halfword head)
 }
 
 
-#define STORE_CHAR(x) do {						\
-    int xx = get_lc_code(x);						\
-    if (xx==0) xx = 0xFFFE;						\
-    word[w] = xx;							\
-    if (w<MAX_WORD_LEN) w++;						\
-  } while (0)
+#define STORE_CHAR(x) do {                          \
+        word[w] = x;                                \
+        if (w<MAX_WORD_LEN) w++;                    \
+    } while (0)
 
 /* todo change this! */
 
@@ -790,16 +788,17 @@ int valid_wordend(halfword s)
     if (r == null)
         return 1;
     while ((r != null) && ((type(r) == glyph_node && is_simple_character(r)
-                            && clang == char_lang(r)) || (type(r) == kern_node
-                                                          && subtype(r) ==
-                                                          normal))) {
+                            && clang == char_lang(r)) ||
+                           (type(r) == kern_node && (subtype(r) == normal))
+           )) {
         r = vlink(r);
     }
     if (r == null || (type(r) == glyph_node && is_simple_character(r)
                       && clang != char_lang(r)) || type(r) == glue_node
         || type(r) == whatsit_node || type(r) == ins_node
         || type(r) == adjust_node || type(r) == penalty_node
-        || (type(r) == kern_node && subtype(r) == explicit))
+        || (type(r) == kern_node
+            && (subtype(r) == explicit || subtype(r) == acc_kern)))
         return 1;
     return 0;
 }
@@ -951,7 +950,7 @@ void new_hyphenation(halfword head, halfword tail)
 void dump_one_language(int i)
 {
     char *s = NULL;
-    integer x = 0;
+    int x = 0;
     struct tex_language *lang;
     lang = tex_languages[i];
     dump_int(lang->id);
@@ -978,7 +977,7 @@ void dump_one_language(int i)
 
 void dump_language_data(void)
 {
-    integer i;
+    int i;
     dump_int(next_lang_id);
     for (i = 0; i < next_lang_id; i++) {
         if (tex_languages[i]) {
@@ -994,7 +993,7 @@ void dump_language_data(void)
 void undump_one_language(int i)
 {
     char *s = NULL;
-    integer x = 0;
+    int x = 0;
     struct tex_language *lang = get_language(i);
     undump_int(x);
     lang->id = x;
@@ -1026,7 +1025,7 @@ void undump_one_language(int i)
 
 void undump_language_data(void)
 {
-    integer i, x, numlangs;
+    int i, x, numlangs;
     undump_int(numlangs);
     next_lang_id = numlangs;
     for (i = 0; i < numlangs; i++) {
@@ -1036,3 +1035,67 @@ void undump_language_data(void)
         }
     }
 }
+
+/*
+When \TeX\ has scanned `\.{\\hyphenation}', it calls on a procedure named
+|new_hyph_exceptions| to do the right thing.
+*/
+
+void new_hyph_exceptions(void)
+{                               /* enters new exceptions */
+    (void) scan_toks(false, true);
+    load_tex_hyphenation(int_par(language_code), def_ref);
+    flush_list(def_ref);
+}
+
+/*
+Similarly, when \TeX\ has scanned `\.{\\patterns}', it calls on a
+procedure named |new_patterns|.
+*/
+
+void new_patterns(void)
+{                               /* initializes the hyphenation pattern data */
+    (void) scan_toks(false, true);
+    load_tex_patterns(int_par(language_code), def_ref);
+    flush_list(def_ref);
+}
+
+/*
+`\.{\\prehyphenchar}', sets the |pre_break| character, and
+`\.{\\posthyphenchar}' the |post_break| character. Their respective
+defaults are ascii hyphen ("-") and zero (nul).
+*/
+
+void new_pre_hyphen_char(void)
+{
+    scan_optional_equals();
+    scan_int();
+    set_pre_hyphen_char(int_par(language_code), cur_val);
+}
+
+void new_post_hyphen_char(void)
+{
+    scan_optional_equals();
+    scan_int();
+    set_post_hyphen_char(int_par(language_code), cur_val);
+}
+
+/*
+`\.{\\preexhyphenchar}', sets the |pre_break| character, and
+`\.{\\postexhyphenchar}' the |post_break| character. Their
+defaults are both zero (nul).
+*/
+
+void new_pre_exhyphen_char(void)
+{
+    scan_optional_equals();
+    scan_int();
+    set_pre_exhyphen_char(int_par(language_code), cur_val);
+}
+
+void new_post_exhyphen_char(void)
+{
+    scan_optional_equals();
+    scan_int();
+    set_post_exhyphen_char(int_par(language_code), cur_val);
+}
