@@ -1,6 +1,6 @@
 /* lnodelib.c
    
-   Copyright 2006-2008 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -17,15 +17,12 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
-#include <ptexlib.h>
-
-#include "nodes.h"
-
-#include "commands.h"
-
 static const char _svn_version[] =
-    "$Id: lnodelib.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/lua/lnodelib.c $";
+    "$Id: lnodelib.c 3265 2009-12-18 16:21:40Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/lnodelib.c $";
+
+#include "lua/luatex-api.h"
+#include <ptexlib.h>
 
 #define init_luaS_index(a) do {                                         \
     lua_pushliteral(L,#a);                                              \
@@ -45,6 +42,8 @@ static const char _svn_version[] =
 
 make_luaS_index(luatex_node);
 
+static int nodelib_getdir(lua_State * L, int n);        /* forward */
+
 halfword *check_isnode(lua_State * L, int ud)
 {
     register halfword *p = lua_touserdata(L, ud);
@@ -168,7 +167,7 @@ void lua_nodelib_push_fast(lua_State * L, halfword n)
 
 static int lua_nodelib_id(lua_State * L)
 {
-    integer i = get_node_type_id(L, 1);
+    int i = get_node_type_id(L, 1);
     if (i >= 0) {
         lua_pushnumber(L, i);
     } else {
@@ -180,7 +179,7 @@ static int lua_nodelib_id(lua_State * L)
 
 static int lua_nodelib_subtype(lua_State * L)
 {
-    integer i = get_node_subtype_id(L, 1);
+    int i = get_node_subtype_id(L, 1);
     if (i >= 0) {
         lua_pushnumber(L, i);
     } else {
@@ -193,7 +192,7 @@ static int lua_nodelib_subtype(lua_State * L)
 
 static int lua_nodelib_type(lua_State * L)
 {
-    integer i = get_node_type_id(L, 1);
+    int i = get_node_type_id(L, 1);
     if (i >= 0) {
         lua_pushstring(L, node_data[i].name);
     } else {
@@ -451,10 +450,10 @@ static int lua_nodelib_append(lua_State * L)
     for (i = 1; i <= j; i++) {
         n = check_isnode(L, i);
         m = *n;
-        new_tail_append(m);
+        tail_append(m);
         while (vlink(m) != null) {
             m = vlink(m);
-            new_tail_append(m);
+            tail_append(m);
         }
     }
     return 0;
@@ -477,8 +476,9 @@ static int lua_nodelib_hpack(lua_State * L)
 {
     halfword n, p;
     char *s;
-    integer w = 0;
+    int w = 0;
     int m = 1;
+    int d = -1;
     n = *(check_isnode(L, 1));
     if (lua_gettop(L) > 1) {
         w = lua_tointeger(L, 2);
@@ -494,28 +494,81 @@ static int lua_nodelib_hpack(lua_State * L)
                                    "3rd argument should be either additional or exactly");
                     lua_error(L);
                 }
-            }
-
-            else if (lua_type(L, 3) == LUA_TNUMBER) {
+            } else if (lua_type(L, 3) == LUA_TNUMBER) {
                 m = lua_tonumber(L, 3);
             } else {
                 lua_pushstring(L, "incorrect 3rd argument");
             }
+            if (lua_gettop(L) > 3) {
+                if (lua_type(L, 4) == LUA_TSTRING) {
+                    d = nodelib_getdir(L, 4);
+                } else {
+                    lua_pushstring(L, "incorrect 4th argument");
+                }
+            }
         }
     }
-    p = hpack(n, w, m);
+    p = hpack(n, w, m, d);
     lua_nodelib_push_fast(L, p);
     return 1;
 }
 
 
+static int lua_nodelib_dimensions(lua_State * L)
+{
+    int top;
+    top = lua_gettop(L);
+    if (top > 0) {
+        scaled_whd siz;
+        glue_ratio g_mult = 1.0;
+        int g_sign = normal;
+        int g_order = normal;
+        int i = 1;
+        int d = -1;
+        halfword n = null, p = null;
+        if (lua_isnumber(L, 1)) {
+            if (top < 4) {
+                lua_pushnil(L);
+                return 1;
+            }
+            i += 3;
+            g_mult = (glue_ratio) lua_tonumber(L, 1);
+            g_sign = lua_tonumber(L, 2);
+            g_order = lua_tonumber(L, 3);
+        }
+        n = *(check_isnode(L, i));
+        if (lua_gettop(L) > i && !lua_isnil(L, (i + 1))) {
+            if (lua_type(L, (i + 1)) == LUA_TSTRING) {
+                d = nodelib_getdir(L, (i + 1));
+            } else {
+                p = *(check_isnode(L, (i + 1)));
+            }
+        }
+        if (lua_gettop(L) > (i + 1) && lua_type(L, (i + 2)) == LUA_TSTRING) {
+            d = nodelib_getdir(L, (i + 2));
+        }
+        siz = natural_sizes(n, p, g_mult, g_sign, g_order, d);
+        lua_pushnumber(L, siz.wd);
+        lua_pushnumber(L, siz.ht);
+        lua_pushnumber(L, siz.dp);
+        return 3;
+    } else {
+        lua_pushstring(L,
+                       "missing  argument to 'dimensions' (luatex_node expected)");
+        lua_error(L);
+    }
+    return 0;                   /* not reached */
+}
+
+
 /* build a vbox */
 static int lua_nodelib_vpack(lua_State * L)
 {
     halfword n, p;
     char *s;
-    integer w = 0;
+    int w = 0;
     int m = 1;
+    int d = -1;
     n = *(check_isnode(L, 1));
     if (lua_gettop(L) > 1) {
         w = lua_tointeger(L, 2);
@@ -531,6 +584,13 @@ static int lua_nodelib_vpack(lua_State * L)
                                    "3rd argument should be either additional or exactly");
                     lua_error(L);
                 }
+                if (lua_gettop(L) > 3) {
+                    if (lua_type(L, 4) == LUA_TSTRING) {
+                        d = nodelib_getdir(L, 4);
+                    } else {
+                        lua_pushstring(L, "incorrect 4th argument");
+                    }
+                }
             }
 
             else if (lua_type(L, 3) == LUA_TNUMBER) {
@@ -540,7 +600,7 @@ static int lua_nodelib_vpack(lua_State * L)
             }
         }
     }
-    p = vpackage(n, w, m, max_dimen);
+    p = vpackage(n, w, m, max_dimen, d);
     lua_nodelib_push_fast(L, p);
     return 1;
 }
@@ -557,10 +617,7 @@ static int lua_nodelib_mlist_to_hlist(lua_State * L)
     w = luaL_checkoption(L, 2, "text", math_style_names);
     luaL_checkany(L, 3);
     m = lua_toboolean(L, 3);
-    cur_mlist = n;
-    cur_style = w;
-    mlist_penalties = m;
-    mlist_to_hlist();
+    mlist_to_hlist_args(n, w, m);
     lua_nodelib_push_fast(L, vlink(temp_head));
     return 1;
 }
@@ -777,6 +834,24 @@ static int lua_nodelib_tail(lua_State * L)
     return 1;
 }
 
+static int lua_nodelib_tail_only(lua_State * L)
+{
+    halfword *n;
+    halfword t;
+    if (lua_isnil(L, 1))
+        return 1;               /* the nil itself */
+    n = check_isnode(L, 1);
+    t = *n;
+    if (t == null)
+        return 1;               /* the old userdata */
+    while (vlink(t) != null) {
+        t = vlink(t);
+    }
+    lua_nodelib_push_fast(L, t);
+    return 1;
+}
+
+
 /* a few utility functions for attribute stuff */
 
 static int lua_nodelib_has_attribute(lua_State * L)
@@ -971,21 +1046,30 @@ static int lua_nodelib_count(lua_State * L)
 #define nodelib_pushattr(L,n) { lua_pushnumber(L,n); lua_nodelib_push(L); }
 #define nodelib_pushspec(L,n) { lua_pushnumber(L,n); lua_nodelib_push(L); }
 #define nodelib_pushaction(L,n) { lua_pushnumber(L,n); lua_nodelib_push(L); }
-#define nodelib_pushstring(L,n) { lua_pushstring(L,makecstring(n)); }
+#define nodelib_pushstring(L,n) { char *ss=makecstring(n); lua_pushstring(L,ss); free(ss); }
 
 static void nodelib_pushdir(lua_State * L, int n, boolean dirnode)
 {
-    int f = 0;
-    char dirstring[5] = { 0 };
+    char s[2];
     if (dirnode) {
-        dirstring[f++] = (n < 0 ? '-' : '+');
+        s[0] = (n < 0 ? '-' : '+');
+        s[1] = 0;
+    } else {
+        s[0] = 0;
     }
     if (n < 0)
         n += 64;
-    dirstring[f++] = dir_names[(int) dir_primary[n]];
-    dirstring[f++] = dir_names[(int) dir_secondary[n]];
-    dirstring[f++] = dir_names[(int) dir_tertiary[n]];
-    lua_pushstring(L, dirstring);
+    if (n == dir_TLT) {
+        lua_pushfstring(L, "%sTLT", s);
+    } else if (n == dir_TRT) {
+        lua_pushfstring(L, "%sTRT", s);
+    } else if (n == dir_LTL) {
+        lua_pushfstring(L, "%sLTL", s);
+    } else if (n == dir_RTT) {
+        lua_pushfstring(L, "%sRTT", s);
+    } else {
+        lua_pushstring(L, "???");
+    }
 }
 
 static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
@@ -1115,13 +1199,13 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
         case pdf_refxform_node:
             switch (field) {
             case 4:
-                lua_pushnumber(L, pdf_width(n));
+                lua_pushnumber(L, width(n));
                 break;
             case 5:
-                lua_pushnumber(L, pdf_height(n));
+                lua_pushnumber(L, depth(n));
                 break;
             case 6:
-                lua_pushnumber(L, pdf_depth(n));
+                lua_pushnumber(L, height(n));
                 break;
             case 7:
                 lua_pushnumber(L, pdf_xform_objnum(n));
@@ -1133,16 +1217,19 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
         case pdf_refximage_node:
             switch (field) {
             case 4:
-                lua_pushnumber(L, pdf_width(n));
+                lua_pushnumber(L, width(n));
                 break;
             case 5:
-                lua_pushnumber(L, pdf_height(n));
+                lua_pushnumber(L, depth(n));
                 break;
             case 6:
-                lua_pushnumber(L, pdf_depth(n));
+                lua_pushnumber(L, height(n));
                 break;
             case 7:
-                lua_pushnumber(L, pdf_ximage_objnum(n));
+                lua_pushnumber(L, pdf_ximage_transform(n));
+                break;
+            case 8:
+                lua_pushnumber(L, pdf_ximage_index(n));
                 break;
             default:
                 lua_pushnil(L);
@@ -1151,13 +1238,13 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
         case pdf_annot_node:
             switch (field) {
             case 4:
-                lua_pushnumber(L, pdf_width(n));
+                lua_pushnumber(L, width(n));
                 break;
             case 5:
-                lua_pushnumber(L, pdf_height(n));
+                lua_pushnumber(L, depth(n));
                 break;
             case 6:
-                lua_pushnumber(L, pdf_depth(n));
+                lua_pushnumber(L, height(n));
                 break;
             case 7:
                 lua_pushnumber(L, pdf_annot_objnum(n));
@@ -1172,13 +1259,13 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
         case pdf_start_link_node:
             switch (field) {
             case 4:
-                lua_pushnumber(L, pdf_width(n));
+                lua_pushnumber(L, width(n));
                 break;
             case 5:
-                lua_pushnumber(L, pdf_height(n));
+                lua_pushnumber(L, depth(n));
                 break;
             case 6:
-                lua_pushnumber(L, pdf_depth(n));
+                lua_pushnumber(L, height(n));
                 break;
             case 7:
                 lua_pushnumber(L, pdf_link_objnum(n));
@@ -1196,13 +1283,13 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
         case pdf_dest_node:
             switch (field) {
             case 4:
-                lua_pushnumber(L, pdf_width(n));
+                lua_pushnumber(L, width(n));
                 break;
             case 5:
-                lua_pushnumber(L, pdf_height(n));
+                lua_pushnumber(L, depth(n));
                 break;
             case 6:
-                lua_pushnumber(L, pdf_depth(n));
+                lua_pushnumber(L, height(n));
                 break;
             case 7:
                 lua_pushnumber(L, pdf_dest_named_id(n));
@@ -1230,13 +1317,13 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
         case pdf_start_thread_node:
             switch (field) {
             case 4:
-                lua_pushnumber(L, pdf_width(n));
+                lua_pushnumber(L, width(n));
                 break;
             case 5:
-                lua_pushnumber(L, pdf_height(n));
+                lua_pushnumber(L, depth(n));
                 break;
             case 6:
-                lua_pushnumber(L, pdf_depth(n));
+                lua_pushnumber(L, height(n));
                 break;
             case 7:
                 lua_pushnumber(L, pdf_thread_named_id(n));
@@ -1877,10 +1964,10 @@ static int lua_nodelib_getfield(lua_State * L)
         case 2:
             lua_pushnumber(L, subtype(n));
             break;
-        case 3:
+        case 4:
             lua_pushnumber(L, width(n));
             break;
-        case 4:
+        case 5:
             nodelib_pushlist(L, margin_char(n));
             break;
         default:
@@ -1973,7 +2060,6 @@ static int nodelib_getdir(lua_State * L, int n)
 {
     char *s = NULL;
     int d = 32;                 /* invalid number */
-    int a = -1, b = -1, c = -1;
     if (lua_type(L, n) == LUA_TSTRING) {
         s = (char *) lua_tostring(L, n);
         if (strlen(s) == 3) {
@@ -1987,36 +2073,19 @@ static int nodelib_getdir(lua_State * L, int n)
             s++;
         }
         if (strlen(s) == 3) {
-            switch (*s) {
-                /*  *INDENT-OFF* */
-                case 'T': a=0; break;
-                case 'L': a=1; break;
-                case 'B': a=2; break;
-                case 'R': a=3; break;
-                /* *INDENT-ON* */
-            }
-            switch (*(s + 1)) {
-                /*  *INDENT-OFF* */
-                case 'T': b=0; break;
-                case 'L': b=1; break;
-                case 'B': b=2; break;
-                case 'R': b=3; break;
-                /* *INDENT-ON* */
-            }
-            switch (*(s + 2)) {
-                /*  *INDENT-OFF* */
-                case 'T': c=0; break;
-                case 'L': c=1; break;
-                case 'B': c=2; break;
-                case 'R': c=3; break;
-                /* *INDENT-ON* */
+            if (strcmp(s, "TLT") == 0) {
+                d += dir_TLT;
+            } else if (strcmp(s, "TRT") == 0) {
+                d += dir_TRT;
+            } else if (strcmp(s, "LTL") == 0) {
+                d += dir_LTL;
+            } else if (strcmp(s, "RTT") == 0) {
+                d += dir_RTT;
             }
         }
-        if (a != -1 && b != -1 && c != -1 && !dir_parallel(a, b)) {
-            d += (a * 8 + dir_rearrange[b] * 4 + c);
-        }
-    } else if (lua_isnumber(L, n)) {
-        d = lua_tonumber(L, n);
+    } else {
+        lua_pushstring(L, "Direction specifiers have to be strings");
+        lua_error(L);
     }
     if ((d > 31) || (d < -64) || (d < 0 && (d + 64) > 31)) {
         d = 0;
@@ -2187,13 +2256,13 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
     case pdf_refxform_node:
         switch (field) {
         case 4:
-            pdf_width(n) = lua_tointeger(L, 3);
+            width(n) = lua_tointeger(L, 3);
             break;
         case 5:
-            pdf_height(n) = lua_tointeger(L, 3);
+            depth(n) = lua_tointeger(L, 3);
             break;
         case 6:
-            pdf_depth(n) = lua_tointeger(L, 3);
+            height(n) = lua_tointeger(L, 3);
             break;
         case 7:
             pdf_xform_objnum(n) = lua_tointeger(L, 3);
@@ -2205,16 +2274,19 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
     case pdf_refximage_node:
         switch (field) {
         case 4:
-            pdf_width(n) = lua_tointeger(L, 3);
+            width(n) = lua_tointeger(L, 3);
             break;
         case 5:
-            pdf_height(n) = lua_tointeger(L, 3);
+            depth(n) = lua_tointeger(L, 3);
             break;
         case 6:
-            pdf_depth(n) = lua_tointeger(L, 3);
+            height(n) = lua_tointeger(L, 3);
             break;
         case 7:
-            pdf_ximage_objnum(n) = lua_tointeger(L, 3);
+            pdf_ximage_transform(n) = lua_tointeger(L, 3);
+            break;
+        case 8:
+            pdf_ximage_index(n) = lua_tointeger(L, 3);
             break;
         default:
             return nodelib_cantset(L, field, n);
@@ -2223,19 +2295,19 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
     case pdf_annot_node:
         switch (field) {
         case 4:
-            pdf_width(n) = lua_tointeger(L, 3);
+            width(n) = lua_tointeger(L, 3);
             break;
         case 5:
-            pdf_height(n) = lua_tointeger(L, 3);
+            depth(n) = lua_tointeger(L, 3);
             break;
         case 6:
-            pdf_depth(n) = lua_tointeger(L, 3);
+            height(n) = lua_tointeger(L, 3);
             break;
         case 7:
             pdf_annot_objnum(n) = lua_tointeger(L, 3);
             break;
         case 8:
-            pdf_annot_data(n) = nodelib_getstring(L, 3);
+            pdf_annot_data(n) = nodelib_gettoks(L, 3);
             break;
         default:
             return nodelib_cantset(L, field, n);
@@ -2244,19 +2316,19 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
     case pdf_start_link_node:
         switch (field) {
         case 4:
-            pdf_width(n) = lua_tointeger(L, 3);
+            width(n) = lua_tointeger(L, 3);
             break;
         case 5:
-            pdf_height(n) = lua_tointeger(L, 3);
+            depth(n) = lua_tointeger(L, 3);
             break;
         case 6:
-            pdf_depth(n) = lua_tointeger(L, 3);
+            height(n) = lua_tointeger(L, 3);
             break;
         case 7:
             pdf_link_objnum(n) = lua_tointeger(L, 3);
             break;
         case 8:
-            pdf_link_attr(n) = nodelib_getstring(L, 3);
+            pdf_link_attr(n) = nodelib_gettoks(L, 3);
             break;
         case 9:
             pdf_link_action(n) = nodelib_getaction(L, 3);
@@ -2274,13 +2346,13 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
     case pdf_dest_node:
         switch (field) {
         case 4:
-            pdf_width(n) = lua_tointeger(L, 3);
+            width(n) = lua_tointeger(L, 3);
             break;
         case 5:
-            pdf_height(n) = lua_tointeger(L, 3);
+            depth(n) = lua_tointeger(L, 3);
             break;
         case 6:
-            pdf_depth(n) = lua_tointeger(L, 3);
+            height(n) = lua_tointeger(L, 3);
             break;
         case 7:
             pdf_dest_named_id(n) = lua_tointeger(L, 3);
@@ -2308,13 +2380,13 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
     case pdf_start_thread_node:
         switch (field) {
         case 4:
-            pdf_width(n) = lua_tointeger(L, 3);
+            width(n) = lua_tointeger(L, 3);
             break;
         case 5:
-            pdf_height(n) = lua_tointeger(L, 3);
+            depth(n) = lua_tointeger(L, 3);
             break;
         case 6:
-            pdf_depth(n) = lua_tointeger(L, 3);
+            height(n) = lua_tointeger(L, 3);
             break;
         case 7:
             pdf_thread_named_id(n) = lua_tointeger(L, 3);
@@ -2943,10 +3015,10 @@ static int lua_nodelib_setfield(lua_State * L)
             case 2:
                 subtype(n) = lua_tointeger(L, 3);
                 break;
-            case 3:
+            case 4:
                 width(n) = lua_tointeger(L, 3);
                 break;
-            case 4:
+            case 5:
                 margin_char(n) = nodelib_getlist(L, 3);
                 break;
             default:
@@ -3227,6 +3299,7 @@ static const struct luaL_reg nodelib_f[] = {
     {"count", lua_nodelib_count},
     {"traverse", lua_nodelib_traverse},
     {"traverse_id", lua_nodelib_traverse_filtered},
+    {"tail", lua_nodelib_tail_only},
     {"slide", lua_nodelib_tail},
     {"types", lua_nodelib_types},
     {"whatsits", lua_nodelib_whatsits},
@@ -3241,6 +3314,7 @@ static const struct luaL_reg nodelib_f[] = {
     {"last_node", lua_nodelib_last_node},
     {"copy", lua_nodelib_copy},
     {"copy_list", lua_nodelib_copy_list},
+    {"dimensions", lua_nodelib_dimensions},
     {"hpack", lua_nodelib_hpack},
     {"vpack", lua_nodelib_vpack},
     {"mlist_to_hlist", lua_nodelib_mlist_to_hlist},
