@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
+#include "lua/luatex-api.h"
 #include <ptexlib.h>
 
 #include <sys/stat.h>
@@ -25,7 +25,7 @@
 #include <time.h>
 
 static const char _svn_version[] =
-    "$Id: loslibext.c 2376 2009-05-08 08:40:13Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/lua/loslibext.c $";
+    "$Id: loslibext.c 3288 2009-12-24 10:05:46Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/loslibext.c $";
 
 #if defined(_WIN32) || defined(WIN32) || defined(__NT__)
 #  define MKDIR(a,b) mkdir(a)
@@ -81,6 +81,9 @@ static const char _svn_version[] =
 #  elif defined(__MACH__) && defined(__APPLE__)
 #    undef OS_PLATNAME
 #    define OS_PLATNAME "macosx"
+#  elif defined(__GNU__)
+#    undef OS_PLATNAME
+#    define OS_PLATNAME "gnu"
 #  endif
 #endif
 
@@ -117,7 +120,7 @@ static const char _svn_version[] =
 
 static int exec_command(const char *file, char *const *argv, char *const *envp)
 {
-    char path[PATH_MAX];
+    char *path;
     const char *searchpath, *esp;
     size_t prefixlen, filelen, totallen;
 
@@ -125,6 +128,7 @@ static int exec_command(const char *file, char *const *argv, char *const *envp)
         return execve(file, argv, envp);
 
     filelen = strlen(file);
+    path = NULL;
 
     searchpath = getenv("PATH");
     if (!searchpath)
@@ -141,14 +145,20 @@ static int exec_command(const char *file, char *const *argv, char *const *envp)
 
         if (prefixlen == 0 || searchpath[prefixlen - 1] == '/') {
             totallen = prefixlen + filelen;
+#ifdef PATH_MAX
             if (totallen >= PATH_MAX)
                 continue;
+#endif
+            path = malloc(totallen + 1);
             memcpy(path, searchpath, prefixlen);
             memcpy(path + prefixlen, file, filelen);
         } else {
             totallen = prefixlen + filelen + 1;
+#ifdef PATH_MAX
             if (totallen >= PATH_MAX)
                 continue;
+#endif
+            path = malloc(totallen + 1);
             memcpy(path, searchpath, prefixlen);
             path[prefixlen] = '/';
             memcpy(path + prefixlen + 1, file, filelen);
@@ -156,6 +166,8 @@ static int exec_command(const char *file, char *const *argv, char *const *envp)
         path[totallen] = '\0';
 
         execve(path, argv, envp);
+        free(path);
+        path = NULL;
         if (errno == E2BIG || errno == ENOEXEC ||
             errno == ENOMEM || errno == ETXTBSY)
             break;              /* Report this as an error, no more search */
@@ -244,11 +256,44 @@ static int spawn_command(const char *file, char *const *argv, char *const *envp)
 
 extern char **environ;
 
-static char **do_split_command(char *maincmd)
+#ifdef WIN32
+static char *get_command_name(char *maincmd)
 {
+    /* retrieve argv[0] part from the command string, 
+    it will be truncated to MAX_PATH if it's too long */
+    char *cmdname = (char *) malloc(sizeof(char) * MAX_PATH);
+    int i, k, quoted;
+    quoted = k = 0;
+    for (i = 0; (i < MAX_PATH) && maincmd[i] && 
+                (maincmd[i] != ' ' && maincmd[i] != '\t' || quoted); i++) {
+        if (maincmd[i] == '"') {
+            quoted = !quoted;
+        } else {
+            cmdname[k] = maincmd[i];
+            k++;
+        }
+    }
+    cmdname[k] = '\0';
+    return cmdname;
+}
+#endif
+
+static char **do_split_command(char *maincmd, char **runcmd)
+{
+    char **cmdline = NULL;
+#ifdef WIN32
+    /* On WIN32 don't split anything, because 
+    _spawnvpe can't put it back together properly 
+    if there are quoted arguments with spaces. 
+    Instead, dump everything into one argument 
+    and it will be passed through as is */
+    cmdline = malloc(sizeof(char *) * 2);
+    cmdline[0] = xstrdup(maincmd);
+    cmdline[1] = NULL;
+    *runcmd = get_command_name(cmdline[0]);
+#else
     char *piece, *start_piece;
     char *cmd;
-    char **cmdline = NULL;
     unsigned int i, j;
     int ret = 0;
     int in_string = 0;
@@ -298,6 +343,8 @@ static char **do_split_command(char *maincmd)
         quoted = 0;
     }
     xfree(start_piece);
+    *runcmd = cmdline[0];
+#endif
     return cmdline;
 }
 
@@ -339,7 +386,11 @@ static char **do_flatten_command(lua_State * L, char **runcmd)
 
     lua_rawgeti(L, -1, 0);
     if (lua_isnil(L, -1) || (s = (char *) lua_tostring(L, -1)) == NULL) {
+#ifdef WIN32
+        *runcmd = get_command_name(cmdline[0]);
+#else
         *runcmd = cmdline[0];
+#endif
     } else {
         *runcmd = xstrdup(s);
     }
@@ -368,8 +419,7 @@ static int os_exec(lua_State * L)
     }
     if (lua_type(L, 1) == LUA_TSTRING) {
         maincmd = (char *) lua_tostring(L, 1);
-        cmdline = do_split_command(maincmd);
-        runcmd = cmdline[0];
+        cmdline = do_split_command(maincmd, &runcmd);
     } else if (lua_type(L, 1) == LUA_TTABLE) {
         cmdline = do_flatten_command(L, &runcmd);
     }
@@ -447,8 +497,7 @@ static int os_spawn(lua_State * L)
     }
     if (lua_type(L, 1) == LUA_TSTRING) {
         maincmd = (char *) lua_tostring(L, 1);
-        cmdline = do_split_command(maincmd);
-        runcmd = cmdline[0];
+        cmdline = do_split_command(maincmd, &runcmd);
     } else if (lua_type(L, 1) == LUA_TTABLE) {
         cmdline = do_flatten_command(L, &runcmd);
     }
