@@ -17,14 +17,22 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
+#include "lua/luatex-api.h"
 #include <ptexlib.h>
-#include "nodes.h"
-#include "commands.h"
-#include "tokens.h"
+
+
 
 static const char _svn_version[] =
-    "$Id: ltexlib.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/lua/ltexlib.c $";
+    "$Id: ltexlib.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/ltexlib.c $";
+
+#define attribute(A) eqtb[attribute_base+(A)].hh.rh
+#define dimen(A) eqtb[scaled_base+(A)].hh.rh
+#undef skip
+#define skip(A) eqtb[skip_base+(A)].hh.rh
+#define mu_skip(A) eqtb[mu_skip_base+(A)].hh.rh
+#define count(A) eqtb[count_base+(A)].hh.rh
+#define box(A) equiv(box_base+(A))
+
 
 extern halfword *check_isnode(lua_State * L, int ud);
 extern void lua_nodelib_push_fast(lua_State * L, halfword n);
@@ -34,7 +42,7 @@ typedef struct {
     unsigned int tsize;
     void *next;
     boolean partial;
-    integer cattable;
+    int cattable;
 } rope;
 
 typedef struct {
@@ -53,7 +61,7 @@ static int spindle_size = 0;
 static spindle *spindles = NULL;
 static int spindle_index = 0;
 
-static void luac_store(lua_State * L, int i, int partial, integer cattable)
+static void luac_store(lua_State * L, int i, int partial, int cattable)
 {
     char *st, *sttemp;
     size_t tsize;
@@ -84,7 +92,7 @@ static void luac_store(lua_State * L, int i, int partial, integer cattable)
 static int do_luacprint(lua_State * L, int partial, int deftable)
 {
     int i, n;
-    integer cattable = (integer) deftable;
+    int cattable = deftable;
     int startstrings = 1;
     n = lua_gettop(L);
     if (cattable != NO_CAT_TABLE) {
@@ -130,9 +138,9 @@ int luacsprint(lua_State * L)
     return do_luacprint(L, PARTIAL_LINE, DEFAULT_CAT_TABLE);
 }
 
-integer luacstring_cattable(void)
+int luacstring_cattable(void)
 {
-    return (integer) read_spindle.tail->cattable;
+    return (int) read_spindle.tail->cattable;
 }
 
 int luacstring_partial(void)
@@ -166,8 +174,7 @@ int luacstring_input(void)
         last = first;
         ret = last;
         check_buffer_overflow(last + t->tsize);
-        /* make sure it fits in the pool as well (for show_token_list c.s) */
-        check_pool_overflow(pool_ptr + t->tsize);
+
         while (t->tsize-- > 0)
             buffer[last++] = *st++;
         if (!t->partial) {
@@ -223,10 +230,6 @@ void luacstring_close(int n)
 
 /* local (static) versions */
 
-#define width_offset 1
-#define depth_offset 2
-#define height_offset 3
-
 #define check_index_range(j,s)						\
   if (j<0 || j > 65535) {							\
     lua_pushfstring(L, "incorrect index value %d for tex.%s()", (int)j, s); \
@@ -270,23 +273,22 @@ int dimen_to_number(lua_State * L, char *s)
 }
 
 
-integer get_item_index(lua_State * L, int i, integer base)
+int get_item_index(lua_State * L, int i, int base)
 {
     size_t kk;
-    integer k;
+    int k;
     int cur_cs;
     char *s;
     if (lua_type(L, i) == LUA_TSTRING) {
         s = (char *) lua_tolstring(L, i, &kk);
         cur_cs = string_lookup(s, kk);
-        if (cur_cs == static_undefined_control_sequence ||
-            is_undefined_cs(cur_cs)) {
+        if (cur_cs == undefined_control_sequence || cur_cs == undefined_cs_cmd) {
             k = -1;             /* guarandeed invalid */
         } else {
-            k = (zget_equiv(cur_cs) - base);
+            k = (equiv(cur_cs) - base);
         }
     } else {
-        k = (integer) luaL_checkinteger(L, i);
+        k = (int) luaL_checkinteger(L, i);
     }
     return k;
 }
@@ -295,10 +297,10 @@ integer get_item_index(lua_State * L, int i, integer base)
 static int vsetdimen(lua_State * L, int is_global)
 {
     int i, j, err;
-    integer k;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int k;
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     i = lua_gettop(L);
     j = 0;
     /* find the value */
@@ -310,10 +312,10 @@ static int vsetdimen(lua_State * L, int is_global)
             lua_error(L);
     } else
         j = (int) lua_tonumber(L, i);
-    k = get_item_index(L, (i - 1), get_scaled_base());
+    k = get_item_index(L, (i - 1), scaled_base);
     check_index_range(k, "setdimen");
     err = set_tex_dimen_register(k, j);
-    int_par(param_global_defs_code) = save_global_defs;
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
         lua_pushstring(L, "incorrect value");
         lua_error(L);
@@ -336,8 +338,8 @@ static int setdimen(lua_State * L)
 static int getdimen(lua_State * L)
 {
     int j;
-    integer k;
-    k = get_item_index(L, lua_gettop(L), get_scaled_base());
+    int k;
+    k = get_item_index(L, lua_gettop(L), scaled_base);
     check_index_range(k, "getdimen");
     j = get_tex_dimen_register(k);
     lua_pushnumber(L, j);
@@ -348,16 +350,16 @@ static int vsetskip(lua_State * L, int is_global)
 {
     int i, err;
     halfword *j;
-    integer k;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int k;
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     i = lua_gettop(L);
     j = check_isnode(L, i);     /* the value */
-    k = get_item_index(L, (i - 1), get_skip_base());
+    k = get_item_index(L, (i - 1), skip_base);
     check_index_range(k, "setskip");    /* the index */
     err = set_tex_skip_register(k, *j);
-    int_par(param_global_defs_code) = save_global_defs;
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
         lua_pushstring(L, "incorrect value");
         lua_error(L);
@@ -380,8 +382,8 @@ static int setskip(lua_State * L)
 int getskip(lua_State * L)
 {
     halfword j;
-    integer k;
-    k = get_item_index(L, lua_gettop(L), get_skip_base());
+    int k;
+    k = get_item_index(L, lua_gettop(L), skip_base);
     check_index_range(k, "getskip");
     j = get_tex_skip_register(k);
     lua_nodelib_push_fast(L, j);
@@ -393,16 +395,16 @@ int getskip(lua_State * L)
 static int vsetcount(lua_State * L, int is_global)
 {
     int i, j, err;
-    integer k;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int k;
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     i = lua_gettop(L);
     j = (int) luaL_checkinteger(L, i);
-    k = get_item_index(L, (i - 1), get_count_base());
+    k = get_item_index(L, (i - 1), count_base);
     check_index_range(k, "setcount");
     err = set_tex_count_register(k, j);
-    int_par(param_global_defs_code) = save_global_defs;
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
         lua_pushstring(L, "incorrect value");
         lua_error(L);
@@ -425,8 +427,8 @@ static int setcount(lua_State * L)
 static int getcount(lua_State * L)
 {
     int j;
-    integer k;
-    k = get_item_index(L, lua_gettop(L), get_count_base());
+    int k;
+    k = get_item_index(L, lua_gettop(L), count_base);
     check_index_range(k, "getcount");
     j = get_tex_count_register(k);
     lua_pushnumber(L, j);
@@ -437,16 +439,16 @@ static int getcount(lua_State * L)
 static int vsetattribute(lua_State * L, int is_global)
 {
     int i, j, err;
-    integer k;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int k;
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     i = lua_gettop(L);
     j = (int) luaL_checkinteger(L, i);
-    k = get_item_index(L, (i - 1), get_attribute_base());
+    k = get_item_index(L, (i - 1), attribute_base);
     check_index_range(k, "setattribute");
     err = set_tex_attribute_register(k, j);
-    int_par(param_global_defs_code) = save_global_defs;
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
         lua_pushstring(L, "incorrect value");
         lua_error(L);
@@ -469,8 +471,8 @@ static int setattribute(lua_State * L)
 static int getattribute(lua_State * L)
 {
     int j;
-    integer k;
-    k = get_item_index(L, lua_gettop(L), get_attribute_base());
+    int k;
+    k = get_item_index(L, lua_gettop(L), attribute_base);
     check_index_range(k, "getattribute");
     j = get_tex_attribute_register(k);
     lua_pushnumber(L, j);
@@ -479,26 +481,23 @@ static int getattribute(lua_State * L)
 
 int vsettoks(lua_State * L, int is_global)
 {
-    int i, j, err;
-    size_t len;
-    integer k;
-    char *st;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int i, err;
+    int k;
+    lstring str;
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     i = lua_gettop(L);
     if (!lua_isstring(L, i)) {
         lua_pushstring(L, "unsupported value type");
         lua_error(L);
     }
-    st = (char *) lua_tolstring(L, i, &len);
-    k = get_item_index(L, (i - 1), get_toks_base());
+    str.s = (unsigned char *) lua_tolstring(L, i, &str.l);
+    k = get_item_index(L, (i - 1), toks_base);
     check_index_range(k, "settoks");
-    j = maketexlstring(st, len);
-    err = zset_tex_toks_register(k, j);
-    int_par(param_global_defs_code) = save_global_defs;
+    err = set_tex_toks_register(k, str);
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
-        flush_str(j);
         lua_pushstring(L, "incorrect value");
         lua_error(L);
     }
@@ -519,12 +518,15 @@ static int settoks(lua_State * L)
 
 static int gettoks(lua_State * L)
 {
-    integer k;
+    int k;
     str_number t;
-    k = get_item_index(L, lua_gettop(L), get_toks_base());
+    char *ss;
+    k = get_item_index(L, lua_gettop(L), toks_base);
     check_index_range(k, "gettoks");
     t = get_tex_toks_register(k);
-    lua_pushstring(L, makecstring(t));
+    ss = makecstring(t);
+    lua_pushstring(L, ss);
+    free(ss);
     flush_str(t);
     return 1;
 }
@@ -532,16 +534,16 @@ static int gettoks(lua_State * L)
 static int get_box_id(lua_State * L, int i)
 {
     char *s;
-    integer cur_cs, cur_cmd;
+    int cur_cs, cur_cmd;
     size_t k = 0;
     int j = -1;
     if (lua_type(L, i) == LUA_TSTRING) {
         s = (char *) lua_tolstring(L, i, &k);
         cur_cs = string_lookup(s, k);
-        cur_cmd = zget_eq_type(cur_cs);
+        cur_cmd = eq_type(cur_cs);
         if (cur_cmd == char_given_cmd ||
             cur_cmd == math_given_cmd || cur_cmd == omath_given_cmd) {
-            j = zget_equiv(cur_cs);
+            j = equiv(cur_cs);
         }
     } else {
         j = (int) lua_tonumber(L, (i));
@@ -562,9 +564,9 @@ static int getbox(lua_State * L)
 static int vsetbox(lua_State * L, int is_global)
 {
     int i, j, k, err;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     k = get_box_id(L, -2);
     check_index_range(k, "setbox");
     i = get_tex_box_register(k);
@@ -576,9 +578,16 @@ static int vsetbox(lua_State * L, int is_global)
             return 0;
     } else {
         j = nodelist_from_lua(L);
+        if (j != null && type(j) != hlist_node && type(j) != vlist_node) {
+            lua_pushfstring(L, "setbox: incompatible node type (%s)\n",
+                            get_node_name(type(j), subtype(j)));
+            lua_error(L);
+            return 0;
+        }
+
     }
     err = set_tex_box_register(k, j);
-    int_par(param_global_defs_code) = save_global_defs;
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
         lua_pushstring(L, "incorrect value");
         lua_error(L);
@@ -639,9 +648,9 @@ int getboxdp(lua_State * L)
 static int vsetboxdim(lua_State * L, int whichdim, int is_global)
 {
     int i, j, k, err;
-    integer save_global_defs = int_par(param_global_defs_code);
+    int save_global_defs = int_par(global_defs_code);
     if (is_global)
-        int_par(param_global_defs_code) = 1;
+        int_par(global_defs_code) = 1;
     i = lua_gettop(L);
     if (!lua_isnumber(L, i)) {
         j = dimen_to_number(L, (char *) lua_tostring(L, i));
@@ -665,7 +674,7 @@ static int vsetboxdim(lua_State * L, int whichdim, int is_global)
     case depth_offset:
         err = set_tex_box_depth(k, j);
     }
-    int_par(param_global_defs_code) = save_global_defs;
+    int_par(global_defs_code) = save_global_defs;
     if (err) {
         lua_pushstring(L, "not a box");
         lua_error(L);
@@ -729,12 +738,11 @@ int settex(lua_State * L)
             }
             cur_cs = string_lookup(st, k);
             flush_str(texstr);
-            cur_cmd = zget_eq_type(cur_cs);
+            cur_cmd = eq_type(cur_cs);
             if (is_int_assign(cur_cmd)) {
                 if (lua_isnumber(L, i)) {
                     assign_internal_value((isglobal ? 4 : 0),
-                                          zget_equiv(cur_cs), lua_tonumber(L,
-                                                                           i));
+                                          equiv(cur_cs), lua_tonumber(L, i));
                 } else {
                     lua_pushstring(L, "unsupported value type");
                     lua_error(L);
@@ -750,67 +758,62 @@ int settex(lua_State * L)
                 } else {
                     j = (int) lua_tonumber(L, i);
                 }
-                assign_internal_value((isglobal ? 4 : 0), zget_equiv(cur_cs),
-                                      j);
+                assign_internal_value((isglobal ? 4 : 0), equiv(cur_cs), j);
+            } else if (is_toks_assign(cur_cmd)) {
+                if (lua_isstring(L, i)) {
+                    j = tokenlist_from_lua(L);  /* uses stack -1 */
+                    assign_internal_value((isglobal ? 4 : 0), equiv(cur_cs), j);
+
+                } else {
+                    lua_pushstring(L, "unsupported value type");
+                    lua_error(L);
+                }
+
             } else {
                 lua_pushstring(L, "unsupported tex internal assignment");
                 lua_error(L);
             }
         } else {
-            lua_rawset(L, (i - 2));
+            if (lua_istable(L, (i - 2)))
+                lua_rawset(L, (i - 2));
         }
     } else {
-        lua_rawset(L, (i - 2));
+        if (lua_istable(L, (i - 2)))
+            lua_rawset(L, (i - 2));
     }
     return 0;
 }
 
-char *get_something_internal(int cur_cmd, int cur_code)
-{
-    int texstr;
-    char *str;
-    int save_cur_val, save_cur_val_level;
-    save_cur_val = cur_val;
-    save_cur_val_level = cur_val_level;
-    zscan_something_simple(cur_cmd, cur_code);
-    texstr = the_scanned_result();
-    cur_val = save_cur_val;
-    cur_val_level = save_cur_val_level;
-    str = makecstring(texstr);
-    flush_str(texstr);
-    return str;
-}
-
 int do_convert(lua_State * L, int cur_code)
 {
     int texstr;
-    integer i = -1;
+    int i = -1;
     char *str = NULL;
     switch (cur_code) {
-    case convert_pdf_creation_date_code:       /* ? */
-    case convert_pdf_insert_ht_code:   /* arg <register int> */
-    case convert_pdf_ximage_bbox_code: /* arg 2 ints */
-    case convert_lua_code:     /* arg complex */
-    case convert_lua_escape_string_code:       /* arg token list */
-    case convert_pdf_colorstack_init_code:     /* arg complex */
-    case convert_left_margin_kern_code:        /* arg box */
-    case convert_right_margin_kern_code:       /* arg box */
+    case pdf_creation_date_code:       /* ? */
+    case pdf_insert_ht_code:   /* arg <register int> */
+    case pdf_ximage_bbox_code: /* arg 2 ints */
+    case lua_code:             /* arg complex */
+    case lua_escape_string_code:       /* arg token list */
+    case pdf_colorstack_init_code:     /* arg complex */
+    case left_margin_kern_code:        /* arg box */
+    case right_margin_kern_code:       /* arg box */
         break;
-    case convert_string_code:  /* arg token */
-    case convert_meaning_code: /* arg token */
+    case string_code:          /* arg token */
+    case meaning_code:         /* arg token */
         break;
 
         /* the next fall through, and come from 'official' indices! */
-    case convert_font_name_code:       /* arg fontid */
-    case convert_font_identifier_code: /* arg fontid */
-    case convert_pdf_font_name_code:   /* arg fontid */
-    case convert_pdf_font_objnum_code: /* arg fontid */
-    case convert_pdf_font_size_code:   /* arg fontid */
-    case convert_uniform_deviate_code: /* arg int */
-    case convert_number_code:  /* arg int */
-    case convert_roman_numeral_code:   /* arg int */
-    case convert_pdf_page_ref_code:    /* arg int */
-    case convert_pdf_xform_name_code:  /* arg int */
+    case font_name_code:       /* arg fontid */
+    case font_identifier_code: /* arg fontid */
+    case pdf_font_name_code:   /* arg fontid */
+    case pdf_font_objnum_code: /* arg fontid */
+    case pdf_font_size_code:   /* arg fontid */
+    case uniform_deviate_code: /* arg int */
+    case number_code:          /* arg int */
+    case roman_numeral_code:   /* arg int */
+    case pdf_page_ref_code:    /* arg int */
+    case pdf_xform_name_code:  /* arg int */
         if (lua_gettop(L) < 1) {
             /* error */
         }
@@ -822,10 +825,12 @@ int do_convert(lua_State * L, int cur_code)
             flush_str(texstr);
         }
     }
-    if (str)
+    if (str) {
         lua_pushstring(L, str);
-    else
+        free(str);
+    } else {
         lua_pushnil(L);
+    }
     return 1;
 }
 
@@ -837,7 +842,7 @@ int do_scan_internal(lua_State * L, int cur_cmd, int cur_code)
     int save_cur_val, save_cur_val_level;
     save_cur_val = cur_val;
     save_cur_val_level = cur_val_level;
-    zscan_something_simple(cur_cmd, cur_code);
+    scan_something_simple(cur_cmd, cur_code);
 
     if (cur_val_level == int_val_level ||
         cur_val_level == dimen_val_level || cur_val_level == attr_val_level) {
@@ -847,10 +852,12 @@ int do_scan_internal(lua_State * L, int cur_cmd, int cur_code)
     } else {                    /* dir_val_level, mu_val_level, tok_val_level */
         texstr = the_scanned_result();
         str = makecstring(texstr);
-        if (str)
+        if (str) {
             lua_pushstring(L, str);
-        else
+            free(str);
+        } else {
             lua_pushnil(L);
+        }
         flush_str(texstr);
     }
     cur_val = save_cur_val;
@@ -863,69 +870,69 @@ int do_lastitem(lua_State * L, int cur_code)
     int retval = 1;
     switch (cur_code) {
         /* the next two do not actually exist */
-    case last_item_lastattr_code:
-    case last_item_attrexpr_code:
+    case lastattr_code:
+    case attrexpr_code:
         lua_pushnil(L);
         break;
         /* the expressions do something complicated with arguments, yuck */
-    case last_item_numexpr_code:
-    case last_item_dimexpr_code:
-    case last_item_glueexpr_code:
-    case last_item_muexpr_code:
+    case numexpr_code:
+    case dimexpr_code:
+    case glueexpr_code:
+    case muexpr_code:
         lua_pushnil(L);
         break;
         /* these read a glue or muglue, todo */
-    case last_item_mu_to_glue_code:
-    case last_item_glue_to_mu_code:
-    case last_item_glue_stretch_order_code:
-    case last_item_glue_shrink_order_code:
-    case last_item_glue_stretch_code:
-    case last_item_glue_shrink_code:
+    case mu_to_glue_code:
+    case glue_to_mu_code:
+    case glue_stretch_order_code:
+    case glue_shrink_order_code:
+    case glue_stretch_code:
+    case glue_shrink_code:
         lua_pushnil(L);
         break;
         /* these read a fontid and a char, todo */
-    case last_item_font_char_wd_code:
-    case last_item_font_char_ht_code:
-    case last_item_font_char_dp_code:
-    case last_item_font_char_ic_code:
+    case font_char_wd_code:
+    case font_char_ht_code:
+    case font_char_dp_code:
+    case font_char_ic_code:
         lua_pushnil(L);
         break;
         /* these read an integer, todo */
-    case last_item_par_shape_length_code:
-    case last_item_par_shape_indent_code:
-    case last_item_par_shape_dimen_code:
+    case par_shape_length_code:
+    case par_shape_indent_code:
+    case par_shape_dimen_code:
         lua_pushnil(L);
         break;
-    case last_item_lastpenalty_code:
-    case last_item_lastkern_code:
-    case last_item_lastskip_code:
-    case last_item_last_node_type_code:
-    case last_item_input_line_no_code:
-    case last_item_badness_code:
-    case last_item_pdftex_version_code:
-    case last_item_pdf_last_obj_code:
-    case last_item_pdf_last_xform_code:
-    case last_item_pdf_last_ximage_code:
-    case last_item_pdf_last_ximage_pages_code:
-    case last_item_pdf_last_annot_code:
-    case last_item_pdf_last_x_pos_code:
-    case last_item_pdf_last_y_pos_code:
-    case last_item_pdf_retval_code:
-    case last_item_pdf_last_ximage_colordepth_code:
-    case last_item_random_seed_code:
-    case last_item_pdf_last_link_code:
-    case last_item_luatex_version_code:
-    case last_item_Aleph_version_code:
-    case last_item_Omega_version_code:
-    case last_item_Aleph_minor_version_code:
-    case last_item_Omega_minor_version_code:
-    case last_item_eTeX_minor_version_code:
-    case last_item_eTeX_version_code:
-    case last_item_current_group_level_code:
-    case last_item_current_group_type_code:
-    case last_item_current_if_level_code:
-    case last_item_current_if_type_code:
-    case last_item_current_if_branch_code:
+    case lastpenalty_code:
+    case lastkern_code:
+    case lastskip_code:
+    case last_node_type_code:
+    case input_line_no_code:
+    case badness_code:
+    case pdftex_version_code:
+    case pdf_last_obj_code:
+    case pdf_last_xform_code:
+    case pdf_last_ximage_code:
+    case pdf_last_ximage_pages_code:
+    case pdf_last_annot_code:
+    case pdf_last_x_pos_code:
+    case pdf_last_y_pos_code:
+    case pdf_retval_code:
+    case pdf_last_ximage_colordepth_code:
+    case random_seed_code:
+    case pdf_last_link_code:
+    case luatex_version_code:
+    case Aleph_version_code:
+    case Omega_version_code:
+    case Aleph_minor_version_code:
+    case Omega_minor_version_code:
+    case eTeX_minor_version_code:
+    case eTeX_version_code:
+    case current_group_level_code:
+    case current_group_type_code:
+    case current_if_level_code:
+    case current_if_type_code:
+    case current_if_branch_code:
         retval = do_scan_internal(L, last_item_cmd, cur_code);
         break;
     default:
@@ -972,59 +979,59 @@ static int tex_getmathparm(lua_State * L)
 
 static int getfontname(lua_State * L)
 {
-    return do_convert(L, convert_font_name_code);
+    return do_convert(L, font_name_code);
 }
 
 static int getfontidentifier(lua_State * L)
 {
-    return do_convert(L, convert_font_identifier_code);
+    return do_convert(L, font_identifier_code);
 }
 
 static int getpdffontname(lua_State * L)
 {
-    return do_convert(L, convert_pdf_font_name_code);
+    return do_convert(L, pdf_font_name_code);
 }
 
 static int getpdffontobjnum(lua_State * L)
 {
-    return do_convert(L, convert_pdf_font_objnum_code);
+    return do_convert(L, pdf_font_objnum_code);
 }
 
 static int getpdffontsize(lua_State * L)
 {
-    return do_convert(L, convert_pdf_font_size_code);
+    return do_convert(L, pdf_font_size_code);
 }
 
 static int getuniformdeviate(lua_State * L)
 {
-    return do_convert(L, convert_uniform_deviate_code);
+    return do_convert(L, uniform_deviate_code);
 }
 
 static int getnumber(lua_State * L)
 {
-    return do_convert(L, convert_number_code);
+    return do_convert(L, number_code);
 }
 
 static int getromannumeral(lua_State * L)
 {
-    return do_convert(L, convert_roman_numeral_code);
+    return do_convert(L, roman_numeral_code);
 }
 
 static int getpdfpageref(lua_State * L)
 {
-    return do_convert(L, convert_pdf_page_ref_code);
+    return do_convert(L, pdf_page_ref_code);
 }
 
 static int getpdfxformname(lua_State * L)
 {
-    return do_convert(L, convert_pdf_xform_name_code);
+    return do_convert(L, pdf_xform_name_code);
 }
 
 
 int get_parshape(lua_State * L)
 {
     int n;
-    halfword par_shape_ptr = get_par_shape_ptr();
+    halfword par_shape_ptr = equiv(par_shape_loc);
     if (par_shape_ptr != 0) {
         int m = 1;
         n = vinfo(par_shape_ptr + 1);
@@ -1203,8 +1210,6 @@ int setlist(lua_State * L)
     return 0;
 }
 
-#define infinity 2147483647
-
 static int do_integer_error(double m)
 {
     char *help[] = { "I can only go up to 2147483647='17777777777=" "7FFFFFFF,",
@@ -1288,8 +1293,8 @@ static int tex_definefont(lua_State * L)
         geq_define(u, set_font_cmd, f);
     else
         eq_define(u, set_font_cmd, f);
-    zeqtb[get_font_id_base() + f] = zeqtb[u];
-    hash_text(get_font_id_base() + f) = t;
+    eqtb[font_id_base + f] = eqtb[u];
+    hash_text(font_id_base + f) = t;
     return 0;
 }
 
@@ -1298,14 +1303,15 @@ static int tex_hashpairs(lua_State * L)
     int cmd, chr;
     str_number s = 0;
     int cs = 1;
-    int eqtb_size = get_eqtb_size();
     lua_newtable(L);
     while (cs < eqtb_size) {
         s = hash_text(cs);
         if (s > 0) {
-            lua_pushstring(L, makecstring(s));
-            cmd = zget_eq_type(cs);
-            chr = zget_equiv(cs);
+            char *ss = makecstring(s);
+            lua_pushstring(L, ss);
+            free(ss);
+            cmd = eq_type(cs);
+            chr = equiv(cs);
             make_token_table(L, cmd, chr, cs);
             lua_rawset(L, -3);
         }
@@ -1323,7 +1329,9 @@ static int tex_primitives(lua_State * L)
     while (cs < prim_size) {
         s = get_prim_text(cs);
         if (s > 0) {
-            lua_pushstring(L, makecstring(s));
+            char *ss = makecstring(s);
+            lua_pushstring(L, ss);
+            free(ss);
             cmd = get_prim_eq_type(cs);
             chr = get_prim_equiv(cs);
             make_token_table(L, cmd, chr, 0);
@@ -1372,7 +1380,9 @@ static int tex_extraprimitives(lua_State * L)
         s = get_prim_text(cs);
         if (s > 0) {
             if (get_prim_origin(cs) & mask) {
-                lua_pushstring(L, makecstring(s));
+                char *ss = makecstring(s);
+                lua_pushstring(L, ss);
+                free(ss);
                 lua_rawseti(L, -2, i++);
             }
         }
@@ -1403,7 +1413,7 @@ static int tex_enableprimitives(lua_State * L)
                     halfword prim_val = prim_lookup(s);
                     if (prim_val != undefined_primitive) {
                         char *newprim;
-                        integer val;
+                        int val;
                         size_t newl;
                         halfword cur_cmd = get_prim_eq_type(prim_val);
                         halfword cur_chr = get_prim_equiv(prim_val);
@@ -1418,8 +1428,8 @@ static int tex_enableprimitives(lua_State * L)
                             strcpy(newprim, prim);
                         }
                         val = string_lookup(newprim, newl);
-                        if (val == static_undefined_control_sequence ||
-                            zget_eq_type(val) == undefined_cs_cmd) {
+                        if (val == undefined_control_sequence ||
+                            eq_type(val) == undefined_cs_cmd) {
                             primitive_def(newprim, newl, cur_cmd, cur_chr);
                         }
                         free(newprim);
@@ -1442,8 +1452,96 @@ static int tex_enableprimitives(lua_State * L)
     return 0;
 }
 
+extern int tex_table_id;
+extern int pdf_table_id;
+extern int token_table_id;
+extern int node_table_id;
+extern int main_initialize(void);
+
+static int tex_run_boot(lua_State * L)
+{
+    int n = lua_gettop(L);
+    char *format = NULL;
+    if (n >= 1) {
+        ini_version = 0;
+        format = (char *) luaL_checkstring(L, 1);
+    } else {
+        ini_version = 1;
+    }
+    if (main_initialize()) {    /* > 0 = failure */
+        lua_pushboolean(L, 0);  /* false */
+        return 1;
+    }
+    if (format) {
+        if (!zopen_w_input(&fmt_file, format, DUMP_FORMAT, FOPEN_RBIN_MODE)) {
+            lua_pushboolean(L, 0);      /* false */
+            return 1;
+        }
+        if (!load_fmt_file(format)) {
+            zwclose(fmt_file);
+            lua_pushboolean(L, 0);      /* false */
+            return 1;
+        }
+        zwclose(fmt_file);
+    }
+    fix_date_and_time();
+    if (format == NULL)
+        make_pdftex_banner();
+    random_seed = (microseconds * 1000) + (epochseconds % 1000000);
+    init_randoms(random_seed);
+    initialize_math();
+    fixup_selector(log_opened);
+    check_texconfig_init();
+    text_dir_ptr = new_dir(0);
+    history = spotless;         /* ready to go! */
+    /* Initialize synctex primitive */
+    synctex_init_command();
+    /* tex is ready to go, now */
+    unhide_lua_table(Luas, "tex", tex_table_id);
+    unhide_lua_table(Luas, "pdf", pdf_table_id);
+    unhide_lua_table(Luas, "token", token_table_id);
+    unhide_lua_table(Luas, "node", node_table_id);
+
+    lua_pushboolean(L, 1);      /* true */
+    return 1;
+
+}
+
+static int tex_run_main(lua_State * L)
+{
+    (void) L;
+    main_control();
+    return 0;
+}
+
+static int tex_run_end(lua_State * L)
+{
+    (void) L;
+    final_cleanup();            /* prepare for death */
+    close_files_and_terminate();
+    do_final_end();
+    return 0;
+}
+
+void init_tex_table(lua_State * L)
+{
+    lua_createtable(L, 0, 3);
+    lua_pushcfunction(L, tex_run_boot);
+    lua_setfield(L, -2, "initialize");
+    lua_pushcfunction(L, tex_run_main);
+    lua_setfield(L, -2, "run");
+    lua_pushcfunction(L, tex_run_end);
+    lua_setfield(L, -2, "finish");
+    lua_setglobal(L, "tex");
+}
+
+
+
 
 static const struct luaL_reg texlib[] = {
+    {"run", tex_run_main},      /* may be needed  */
+    {"finish", tex_run_end},    /* may be needed  */
+    {"write", luacwrite},
     {"write", luacwrite},
     {"print", luacprint},
     {"sprint", luacsprint},
@@ -1524,6 +1622,5 @@ int luaopen_tex(lua_State * L)
     spindle_size = 1;
     /* a somewhat odd place for this assert, maybe */
     assert(command_names[data_cmd].command_offset == data_cmd);
-    assert(param_int_pars == (get_count_base() - get_int_base()));
     return 1;
 }
