@@ -19,13 +19,11 @@
 
 #include <sys/stat.h>
 
-#include "luatex-api.h"
+#include "lua/luatex-api.h"
 #include <ptexlib.h>
 
-#include <luatexdir/luatexextra.h>
-
 static const char _svn_version[] =
-    "$Id: luainit.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/lua/luainit.c $";
+    "$Id: luainit.c 3188 2009-11-23 14:46:20Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/lua/luainit.c $";
 
 /* TH: TODO
  *
@@ -460,10 +458,9 @@ void fix_dumpname(void)
         /* adjust array for Pascal and provide extension, if needed */
         dist = strlen(dump_name) - strlen(DUMP_EXT);
         if (strstr(dump_name, DUMP_EXT) == dump_name + dist)
-            DUMP_VAR = concat(" ", dump_name);
+            TEX_format_default = dump_name;
         else
-            DUMP_VAR = concat3(" ", dump_name, DUMP_EXT);
-        DUMP_LENGTH_VAR = strlen(DUMP_VAR + 1);
+            TEX_format_default = concat(dump_name, DUMP_EXT);
     } else {
         /* For dump_name to be NULL is a bug.  */
         if (!ini_version)
@@ -471,20 +468,179 @@ void fix_dumpname(void)
     }
 }
 
+/* lua require patch */
+
+/* The lua search function.
+ * When kpathsea is not initialized, then it runs the
+ * normal lua function that is saved in the registry, otherwise
+ * it uses kpathsea.
+ */
+
+/* two registry ref variables are needed: one for the actual lua 
+ *  function, the other for its environment .
+ */
+
+static int lua_loader_function = 0;
+static int lua_loader_env = 0;
+
+static int luatex_kpse_lua_find(lua_State * L)
+{
+    const char *filename;
+    const char *name;
+    name = luaL_checkstring(L, 1);
+    if (program_name_set == 0) {
+        lua_CFunction orig_func;
+        lua_rawgeti(L, LUA_REGISTRYINDEX, lua_loader_function);
+        lua_rawgeti(L, LUA_REGISTRYINDEX, lua_loader_env);
+        lua_replace(L, LUA_ENVIRONINDEX);
+        orig_func = lua_tocfunction(L, -1);
+        lua_pop(L, 1);
+        return (orig_func) (L);
+    }
+    filename = kpse_find_file(name, kpse_lua_format, false);
+    if (filename == NULL)
+        return 1;               /* library not found in this path */
+    if (luaL_loadfile(L, filename) != 0) {
+        luaL_error(L, "error loading module %s from file %s:\n\t%s",
+                   lua_tostring(L, 1), filename, lua_tostring(L, -1));
+    }
+    return 1;                   /* library loaded successfully */
+}
+
+/* The lua lib search function.
+ * When kpathsea is not initialized, then it runs the
+ * normal lua function that is saved in the registry, otherwise
+ * it uses kpathsea.
+ */
+
+/* two registry ref variables are needed: one for the actual lua 
+ *  function, the other for its environment .
+ */
+
+static int clua_loader_function = 0;
+static int clua_loader_env = 0;
+
+extern int loader_C_luatex(lua_State * L, const char *name,
+                           const char *filename);
+
+static int luatex_kpse_clua_find(lua_State * L)
+{
+    const char *filename;
+    const char *name;
+    if (safer_option)
+        return 1;               /* library not found in this path */
+    name = luaL_checkstring(L, 1);
+    if (program_name_set == 0) {
+        lua_CFunction orig_func;
+        lua_rawgeti(L, LUA_REGISTRYINDEX, clua_loader_function);
+        lua_rawgeti(L, LUA_REGISTRYINDEX, clua_loader_env);
+        lua_replace(L, LUA_ENVIRONINDEX);
+        orig_func = lua_tocfunction(L, -1);
+        lua_pop(L, 1);
+        return (orig_func) (L);
+    }
+    filename = kpse_find_file(name, kpse_clua_format, false);
+    if (filename == NULL)
+        return 1;               /* library not found in this path */
+    return loader_C_luatex(L, name, filename);
+}
+
+/* Setting up the new search functions. 
+ * This replaces package.loaders[2] with the function defined above.
+ */
+
+static void setup_lua_path(lua_State * L)
+{
+    lua_getglobal(L, "package");
+    lua_getfield(L, -1, "loaders");
+    lua_rawgeti(L, -1, 2);      /* package.loaders[2] */
+    lua_getfenv(L, -1);
+    lua_loader_env = luaL_ref(L, LUA_REGISTRYINDEX);
+    lua_loader_function = luaL_ref(L, LUA_REGISTRYINDEX);
+    lua_pushcfunction(L, luatex_kpse_lua_find);
+    lua_rawseti(L, -2, 2);      /* replace the normal lua loader */
+
+    lua_rawgeti(L, -1, 3);      /* package.loaders[3] */
+    lua_getfenv(L, -1);
+    clua_loader_env = luaL_ref(L, LUA_REGISTRYINDEX);
+    clua_loader_function = luaL_ref(L, LUA_REGISTRYINDEX);
+    lua_pushcfunction(L, luatex_kpse_clua_find);
+    lua_rawseti(L, -2, 3);      /* replace the normal lua lib loader */
+    lua_pop(L, 2);              /* pop the array and table */
+}
+
+/* helper variables for the safe keeping of table ids */
+int tex_table_id;
+int pdf_table_id;
+int token_table_id;
+int node_table_id;
+
+extern void init_tex_table(lua_State * L);
+
+#if defined(WIN32) || defined(__MINGW32__) || defined(__CYGWIN__)
+char **suffixlist;
+
+#  define EXE_SUFFIXES ".com;.exe;.bat;.cmd;.vbs;.vbe;.js;.jse;.wsf;.wsh;.ws;.tcl;.py;.pyw"
+
+static void mk_suffixlist(void)
+{
+    char **p;
+    char *q, *r, *v;
+    int n;
+
+#  if defined(__CYGWIN__)
+    v = xstrdup(EXE_SUFFIXES);
+#  else
+    v = (char *) getenv("PATHEXT");
+    if (v)                      /* strlwr() exists also in MingW */
+        v = (char *) strlwr(xstrdup(v));
+    else
+        v = xstrdup(EXE_SUFFIXES);
+#  endif
+
+    q = v;
+    n = 0;
+
+    while ((r = strchr(q, ';')) != NULL) {
+        n++;
+        r++;
+        q = r;
+    }
+    if (*q)
+        n++;
+    suffixlist = (char **) xmalloc((n + 2) * sizeof(char *));
+    p = suffixlist;
+    *p = xstrdup(".dll");
+    p++;
+    q = v;
+    while ((r = strchr(q, ';')) != NULL) {
+        *r = '\0';
+        *p = xstrdup(q);
+        p++;
+        r++;
+        q = r;
+    }
+    if (*q) {
+        *p = xstrdup(q);
+        p++;
+    }
+    *p = NULL;
+    free(v);
+}
+#endif                          /* WIN32 || __MIBGW32__ || __CYGWIN__ */
+
 void lua_initialize(int ac, char **av)
 {
 
     char *given_file = NULL;
     int kpse_init;
-    int tex_table_id;
-    int pdf_table_id;
-    int token_table_id;
-    int node_table_id;
     /* Save to pass along to topenin.  */
     argc = ac;
     argv = av;
 
-    ptexbanner = BANNER;
+    ptexbanner = malloc(256);
+    snprintf(ptexbanner, 256, "This is LuaTeX, Version %s-%d",
+             luatex_version_string, luatex_date_info);
 
     program_invocation_name = cleaned_invocation_name(argv[0]);
 
@@ -494,6 +650,9 @@ void lua_initialize(int ac, char **av)
          STREQ(argv[1], "--luaconly") || STREQ(argv[1], "--luac"))) {
         exit(luac_main(ac, av));
     }
+#if defined(WIN32) || defined(__MINGW32__) || defined(__CYGWIN__)
+    mk_suffixlist();
+#endif                          /* WIN32 || __MIBGW32__ || __CYGWIN__ */
 
     /* Must be initialized before options are parsed.  */
     interactionoption = 4;
@@ -523,6 +682,7 @@ void lua_initialize(int ac, char **av)
     luainterpreter();
 
     prepare_cmdline(Luas, argv, argc, lua_offset);      /* collect arguments */
+    setup_lua_path(Luas);
 
     if (startup_filename != NULL) {
         given_file = xstrdup(startup_filename);
@@ -541,6 +701,8 @@ void lua_initialize(int ac, char **av)
             fprintf(stdout, "%s\n", lua_tostring(Luas, -1));
             exit(1);
         }
+        /* */
+        init_tex_table(Luas);
         if (lua_pcall(Luas, 0, 0, 0)) {
             fprintf(stdout, "%s\n", lua_tostring(Luas, -1));
             exit(1);
