@@ -17,12 +17,13 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
+
 #include <ptexlib.h>
-#include "nodes.h"
+
+
 
 static const char _svn_version[] =
-    "$Id: postlinebreak.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/tex/postlinebreak.c $";
+    "$Id: postlinebreak.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/tex/postlinebreak.c $";
 
 /* So far we have gotten a little way into the |line_break| routine, having
 covered its important |try_break| subroutine. Now let's consider the
@@ -42,22 +43,6 @@ point to a glue node, penalty node, explicit kern node, or math node.
 
 */
 
-static halfword do_push_dir_node(halfword p, halfword a)
-{
-    halfword n;
-    n = copy_node(a);
-    vlink(n) = p;
-    return n;
-}
-
-static halfword do_pop_dir_node(halfword p)
-{
-    halfword n = vlink(p);
-    flush_node(p);
-    return n;
-}
-
-
 /* The total number of lines that will be set by |post_line_break|
 is |best_line-prev_graf-1|. The last breakpoint is specified by
 |break_node(best_bet)|, and this passive node points to the other breakpoints
@@ -75,14 +60,9 @@ and begin direction instructions at the beginnings of lines.
 
 #define next_break prev_break   /*new name for |prev_break| after links are reversed */
 
-#define append_list(a,b)                                                \
-  { vlink(cur_list.tail_field)=vlink((a)); cur_list.tail_field = b; }
-
-#define left_skip_code 7        /*glue at left of justified lines */
-#define right_skip_code 8       /*glue at right of justified lines */
-
 /* the ints are actually halfwords */
 void ext_post_line_break(boolean d,
+                         int paragraph_dir,
                          int right_skip,
                          int left_skip,
                          int pdf_protrude_chars,
@@ -120,7 +100,7 @@ void ext_post_line_break(boolean d,
     boolean post_disc_break;    /*and did it have a nonempty post-break part? */
     scaled cur_width;           /*width of line number |cur_line| */
     scaled cur_indent;          /*left margin of line number |cur_line| */
-    integer pen;                /*use when calculating penalties between lines */
+    int pen;                    /*use when calculating penalties between lines */
     halfword cur_p;             /* cur_p, but localized */
     halfword cur_line;          /*the current line number being justified */
 
@@ -171,6 +151,9 @@ void ext_post_line_break(boolean d,
         for (q = dir_ptr; q != null; q = vlink(q)) {
             halfword tmp = new_dir(dir_dir(q));
             halfword nxt = vlink(temp_head);
+            delete_attribute_ref(node_attr(tmp));
+            node_attr(tmp) = node_attr(temp_head);
+            add_node_attr_ref(node_attr(tmp));
             couple_nodes(temp_head, tmp);
             try_couple_nodes(tmp, nxt); /* \break\par */
         }
@@ -300,6 +283,9 @@ void ext_post_line_break(boolean d,
             e = vlink(r);
             for (p = dir_ptr; p != null; p = vlink(p)) {
                 halfword s = new_dir(dir_dir(p) - 64);
+                delete_attribute_ref(node_attr(s));
+                node_attr(s) = node_attr(r);
+                add_node_attr_ref(node_attr(s));
                 couple_nodes(r, s);
                 try_couple_nodes(s, e);
                 r = s;
@@ -334,11 +320,14 @@ void ext_post_line_break(boolean d,
                 p = alink(q);   /* get |vlink(p) = q| */
                 assert(vlink(p) == q);
                 ptmp = p;
-                p = find_protchar_right(vlink(temp_head), p);
             }
+            p = find_protchar_right(vlink(temp_head), p);
             w = char_pw(p, right_side);
             if (w != 0) {       /* we have found a marginal kern, append it after |ptmp| */
                 k = new_margin_kern(-w, last_rightmost_char, right_side);
+                delete_attribute_ref(node_attr(k));
+                node_attr(k) = node_attr(p);
+                add_node_attr_ref(node_attr(k));
                 vlink(k) = vlink(ptmp);
                 vlink(ptmp) = k;
                 if (ptmp == q)
@@ -351,6 +340,9 @@ void ext_post_line_break(boolean d,
             /* @<Put the \(r)\.{\\rightskip} glue after node |q|@>; */
             halfword r = new_param_glue(right_skip_code);
             vlink(r) = vlink(q);
+            delete_attribute_ref(node_attr(r));
+            node_attr(r) = node_attr(q);
+            add_node_attr_ref(node_attr(r));
             vlink(q) = r;
             q = r;
         }
@@ -390,12 +382,18 @@ void ext_post_line_break(boolean d,
             w = char_pw(p, left_side);
             if (w != 0) {
                 k = new_margin_kern(-w, last_leftmost_char, left_side);
+                delete_attribute_ref(node_attr(k));
+                node_attr(k) = node_attr(q);
+                add_node_attr_ref(node_attr(k));
                 vlink(k) = q;
                 q = k;
             }
         };
         if (left_skip != zero_glue) {
             r = new_param_glue(left_skip_code);
+            delete_attribute_ref(node_attr(r));
+            node_attr(r) = node_attr(q);
+            add_node_attr_ref(node_attr(r));
             vlink(r) = q;
             q = r;
         }
@@ -417,14 +415,14 @@ void ext_post_line_break(boolean d,
             cur_width = varmem[(par_shape_ptr + 2 * cur_line + 1)].cint;
         }
         adjust_tail = adjust_head;
-        pack_direction = paragraph_dir;
         pre_adjust_tail = pre_adjust_head;
         if (pdf_adjust_spacing > 0) {
-            just_box = hpack(q, cur_width, cal_expand_ratio);
+            just_box = hpack(q, cur_width, cal_expand_ratio, paragraph_dir);
         } else {
-            just_box = hpack(q, cur_width, exactly);
+            just_box = hpack(q, cur_width, exactly, paragraph_dir);
         }
         shift_amount(just_box) = cur_indent;
+        subtype(just_box) = HLIST_SUBTYPE_LINE;
         /* /Call the packaging subroutine, setting |just_box| to the justified box; */
 
         /* Append the new box to the current vertical list, followed by the list of
@@ -546,7 +544,7 @@ void ext_post_line_break(boolean d,
         }
     } while (cur_p != null);
     if ((cur_line != best_line) || (vlink(temp_head) != null))
-        tconfusion("line breaking");
+        confusion("line breaking");
     cur_list.pg_field = best_line - 1;  /* prevgraf */
     cur_list.dirs_field = dir_ptr;      /* dir_save */
     dir_ptr = null;
