@@ -17,45 +17,50 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "luatex-api.h"
+
 #include <ptexlib.h>
 
-#include "commands.h"
+
 #include "primitive.h"
-#include "tokens.h"
 
 static const char _svn_version[] =
-    "$Id: primitive.c 2448 2009-06-08 07:43:50Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.40.5/source/texk/web2c/luatexdir/tex/primitive.c $";
+    "$Id: primitive.c 3261 2009-12-18 11:38:21Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.50.0/source/texk/web2c/luatexdir/tex/primitive.c $";
 
-/* as usual, the file starts with a bunch of #defines that mimic pascal @ds */
+/*
+Control sequences are stored and retrieved by means of a fairly standard hash
+table algorithm called the method of ``coalescing lists'' (cf.\ Algorithm 6.4C
+in {\sl The Art of Computer Programming\/}). Once a control sequence enters the
+table, it is never removed, because there are complicated situations
+involving \.{\\gdef} where the removal of a control sequence at the end of
+a group would be a mistake preventable only by the introduction of a
+complicated reference-count mechanism.
 
-#define level_one 1
-#define flush_string() do { decr(str_ptr); pool_ptr=str_start_macro(str_ptr); } while (0)
-#define cur_length (pool_ptr - str_start_macro(str_ptr))
-#define append_char(a) str_pool[pool_ptr++]=(a)
+The actual sequence of letters forming a control sequence identifier is
+stored in the |str_pool| array together with all the other strings. An
+auxiliary array |hash| consists of items with two halfword fields per
+word. The first of these, called |next(p)|, points to the next identifier
+belonging to the same coalesced list as the identifier corresponding to~|p|;
+and the other, called |text(p)|, points to the |str_start| entry for
+|p|'s identifier. If position~|p| of the hash table is empty, we have
+|text(p)=0|; if position |p| is either empty or the end of a coalesced
+hash list, we have |next(p)=0|. An auxiliary pointer variable called
+|hash_used| is maintained in such a way that all locations |p>=hash_used|
+are nonempty. The global variable |cs_count| tells how many multiletter
+control sequences have been defined, if statistics are being kept.
+
+A global boolean variable called |no_new_control_sequence| is set to
+|true| during the time that new hash table entries are forbidden.
+*/
+
+two_halves *hash;               /* the hash table */
+halfword hash_used;             /* allocation pointer for |hash| */
+int hash_extra;                 /* |hash_extra=hash| above |eqtb_size| */
+halfword hash_top;              /* maximum of the hash array */
+halfword hash_high;             /* pointer to next high hash location */
+boolean no_new_control_sequence;        /* are new identifiers legal? */
+int cs_count;                   /* total number of known identifiers */
 
-#define next(a) hash[(a)].lhfield       /* link for coalesced lists */
-#define text(a) hash[(a)].rh    /* string number for control sequence name */
 #define hash_is_full (hash_used==hash_base)     /* test if all positions are occupied */
-#define hash_size 65536
-
-#define span_code 1114113
-#define unless_code 32          /* amount added for `\.{\\unless}' prefix */
-#define protected_token 0x1C00001       /* $2^{21}\cdot|end_match|+1$ */
-#define offset_ocp_name 1
-#define ocp_name(A) ocp_tables[(A)][offset_ocp_name]
-
-#define skip_base      get_skip_base()
-#define mu_skip_base   get_mu_skip_base()
-#define glue_base      static_glue_base
-#define toks_base      get_toks_base()
-#define count_base     get_count_base()
-#define int_base       static_int_base
-#define attribute_base get_attribute_base()
-#define scaled_base    get_scaled_base()
-#define dimen_base     get_dimen_base()
-
-
 
 /* \primitive support needs a few extra variables and definitions */
 
@@ -135,25 +140,23 @@ static halfword compute_hash(char *j, pool_pointer l, halfword prime_number)
 
 pointer prim_lookup(str_number s)
 {
-    integer h;                  /* hash code */
+    int h;                      /* hash code */
     pointer p;                  /* index in |hash| array */
-    pool_pointer j, l;
-    if (s < string_offset) {
+    unsigned char *j;
+    unsigned l;
+    if (s < STRING_OFFSET) {
         p = s;
         if ((p < 0) || (get_prim_eq_type(p) == undefined_cs_cmd)) {
             p = undefined_primitive;
         }
     } else {
-        j = str_start_macro(s);
-        if (s == str_ptr)
-            l = cur_length;
-        else
-            l = length(s);
-        h = compute_hash((char *) (str_pool + j), l, prim_prime);
+        j = str_string(s);
+        l = str_length(s);
+        h = compute_hash((char *) j, l, prim_prime);
         p = h + prim_base;      /* we start searching here; note that |0<=h<hash_prime| */
         while (1) {
             if (prim_text(p) > 0)
-                if (length(prim_text(p)) == l)
+                if (str_length(prim_text(p)) == l)
                     if (str_eq_str(prim_text(p), s))
                         goto FOUND;
             if (prim_next(p) == 0) {
@@ -164,7 +167,7 @@ pointer prim_lookup(str_number s)
                     if (prim_text(p) > 0) {
                         do {    /* search for an empty location in |prim| */
                             if (prim_is_full)
-                                overflow_string("primitive size", prim_size);
+                                overflow("primitive size", prim_size);
                             decr(prim_used);
                         } while (prim_text(prim_used) != 0);
                         prim_next(p) = prim_used;
@@ -185,9 +188,12 @@ pointer prim_lookup(str_number s)
 
 boolean is_primitive(str_number csname)
 {
-    integer n, m;
+    int n, m;
+    char *ss;
     m = prim_lookup(csname);
-    n = string_lookup(makecstring(csname), length(csname));
+    ss = makecstring(csname);
+    n = string_lookup(ss, str_length(csname));
+    free(ss);
     return ((n != undefined_cs_cmd) &&
             (m != undefined_primitive) &&
             (eq_type(n) == prim_eq_type(m)) && (equiv(n) == prim_equiv(m)));
@@ -196,22 +202,22 @@ boolean is_primitive(str_number csname)
 
 /* a few simple accessors */
 
-quarterword get_prim_eq_type(integer p)
+quarterword get_prim_eq_type(int p)
 {
     return prim_eq_type(p);
 }
 
-quarterword get_prim_origin(integer p)
+quarterword get_prim_origin(int p)
 {
     return prim_origin(p);
 }
 
-halfword get_prim_equiv(integer p)
+halfword get_prim_equiv(int p)
 {
     return prim_equiv(p);
 }
 
-str_number get_prim_text(integer p)
+str_number get_prim_text(int p)
 {
     return prim_text(p);
 }
@@ -326,30 +332,20 @@ store_primitive_name(str_number s, quarterword c, halfword o, halfword offset)
 */
 
 void
-primitive(str_number ss, quarterword c, halfword o, halfword off,
-          int cmd_origin)
+primitive(char *thes, quarterword c, halfword o, halfword off, int cmd_origin)
 {
-    str_number s;               /* actual |str_number| used */
-    integer prim_val;           /* needed to fill |prim_eqtb| */
-    char *thes;
+    int prim_val;               /* needed to fill |prim_eqtb| */
+    str_number ss;
     assert(o >= off);
-    if (ss < string_offset) {
-        if (ss > 127)
-            tconfusion("prim"); /* should be ASCII */
-        append_char(ss);
-        s = make_string();
-    } else {
-        s = ss;
-    }
-    thes = makecstring(s);
+    ss = maketexstring(thes);
     if (cmd_origin == tex_command || cmd_origin == core_command) {
         primitive_def(thes, strlen(thes), c, o);
     }
-    prim_val = prim_lookup(s);
+    prim_val = prim_lookup(ss);
     prim_origin(prim_val) = cmd_origin;
     prim_eq_type(prim_val) = c;
     prim_equiv(prim_val) = o;
-    store_primitive_name(s, c, o, off);
+    store_primitive_name(ss, c, o, off);
 }
 
 
@@ -359,39 +355,40 @@ primitive(str_number ss, quarterword c, halfword o, halfword off,
 
 static halfword insert_id(halfword p, unsigned char *j, pool_pointer l)
 {
-    integer d;
+    unsigned saved_cur_length;
+    unsigned saved_cur_string_size;
+    unsigned char *saved_cur_string;
     unsigned char *k;
     /* This code far from ideal: the existance of |hash_extra| changes
        all the potential (short) coalesced lists into a single (long)
        one. This will create a slowdown. */
-    if (text(p) > 0) {
+    if (cs_text(p) > 0) {
         if (hash_high < hash_extra) {
             incr(hash_high);
             /* can't use eqtb_top here (perhaps because that is not finalized 
                yet when called from |primitive|?) */
-            next(p) = hash_high + get_eqtb_size();
-            p = next(p);
+            cs_next(p) = hash_high + eqtb_size;
+            p = cs_next(p);
         } else {
             do {
                 if (hash_is_full)
-                    overflow_string("hash size", hash_size + hash_extra);
+                    overflow("hash size", hash_size + hash_extra);
                 decr(hash_used);
-            } while (text(hash_used) != 0);     /* search for an empty location in |hash| */
-            next(p) = hash_used;
+            } while (cs_text(hash_used) != 0);  /* search for an empty location in |hash| */
+            cs_next(p) = hash_used;
             p = hash_used;
         }
     }
-    check_pool_overflow((pool_ptr + l));
-    d = cur_length;
-    while (pool_ptr > str_start_macro(str_ptr)) {
-        /* move current string up to make room for another */
-        decr(pool_ptr);
-        str_pool[pool_ptr + l] = str_pool[pool_ptr];
-    }
+    saved_cur_length = cur_length;
+    saved_cur_string = cur_string;
+    saved_cur_string_size = cur_string_size;
+    reset_cur_string();
     for (k = j; k <= j + l - 1; k++)
         append_char(*k);
-    text(p) = make_string();
-    pool_ptr = pool_ptr + d;
+    cs_text(p) = make_string();
+    cur_length = saved_cur_length;
+    cur_string = saved_cur_string;
+    cur_string_size = saved_cur_string_size;
     incr(cs_count);
     return p;
 }
@@ -408,27 +405,37 @@ static halfword insert_id(halfword p, unsigned char *j, pool_pointer l)
 */
 
 
-pointer id_lookup(integer j, integer l)
+pointer id_lookup(int j, int l)
 {                               /* search the hash table */
-    integer h;                  /* hash code */
+    int h;                      /* hash code */
     pointer p;                  /* index in |hash| array */
 
     h = compute_hash((char *) (buffer + j), l, hash_prime);
+#ifdef VERBOSE
+    {
+        unsigned char *todo = xmalloc(l + 2);
+        strncpy(todo, (buffer + j), l);
+        todo[l] = '\0';
+        todo[l + 1] = '\0';
+        fprintf(stdout, "id_lookup(%s)\n", todo);
+        free(todo);
+    }
+#endif
     p = h + hash_base;          /* we start searching here; note that |0<=h<hash_prime| */
     while (1) {
-        if (text(p) > 0)
-            if (length(text(p)) == l)
-                if (str_eq_buf(text(p), j))
+        if (cs_text(p) > 0)
+            if (str_length(cs_text(p)) == (unsigned) l)
+                if (str_eq_buf(cs_text(p), j))
                     goto FOUND;
-        if (next(p) == 0) {
+        if (cs_next(p) == 0) {
             if (no_new_control_sequence) {
-                p = static_undefined_control_sequence;
+                p = undefined_control_sequence;
             } else {
                 p = insert_id(p, (buffer + j), l);
             }
             goto FOUND;
         }
-        p = next(p);
+        p = cs_next(p);
     }
   FOUND:
     return p;
@@ -442,23 +449,23 @@ pointer id_lookup(integer j, integer l)
 
 pointer string_lookup(char *s, size_t l)
 {                               /* search the hash table */
-    integer h;                  /* hash code */
+    int h;                      /* hash code */
     pointer p;                  /* index in |hash| array */
     h = compute_hash(s, l, hash_prime);
     p = h + hash_base;          /* we start searching here; note that |0<=h<hash_prime| */
     while (1) {
-        if (text(p) > 0)
-            if (str_eq_cstr(text(p), s, l))
+        if (cs_text(p) > 0)
+            if (str_eq_cstr(cs_text(p), s, l))
                 goto FOUND;
-        if (next(p) == 0) {
+        if (cs_next(p) == 0) {
             if (no_new_control_sequence) {
-                p = static_undefined_control_sequence;
+                p = undefined_control_sequence;
             } else {
                 p = insert_id(p, (unsigned char *) s, l);
             }
             goto FOUND;
         }
-        p = next(p);
+        p = cs_next(p);
     }
   FOUND:
     return p;
@@ -500,7 +507,7 @@ void prim_cmd_chr(quarterword cmd, halfword chr_code)
 
 void print_cmd_chr(quarterword cmd, halfword chr_code)
 {
-    integer n;                  /* temp variable */
+    int n;                      /* temp variable */
     switch (cmd) {
     case left_brace_cmd:
         chr_cmd("begin-group character ");
@@ -512,7 +519,10 @@ void print_cmd_chr(quarterword cmd, halfword chr_code)
         chr_cmd("math shift character ");
         break;
     case mac_param_cmd:
-        chr_cmd("macro parameter character ");
+        if (chr_code == tab_mark_cmd_code)
+            tprint_esc("alignmark");
+        else
+            chr_cmd("macro parameter character ");
         break;
     case sup_mark_cmd:
         chr_cmd("superscript character ");
@@ -535,6 +545,8 @@ void print_cmd_chr(quarterword cmd, halfword chr_code)
     case tab_mark_cmd:
         if (chr_code == span_code)
             tprint_esc("span");
+        else if (chr_code == tab_mark_cmd_code)
+            tprint_esc("aligntab");
         else
             chr_cmd("alignment tab character ");
         break;
@@ -576,7 +588,7 @@ void print_cmd_chr(quarterword cmd, halfword chr_code)
     case outer_call_cmd:
     case long_outer_call_cmd:
         n = cmd - call_cmd;
-        if (info(link(chr_code)) == protected_token)
+        if (token_info(token_link(chr_code)) == protected_token)
             n = n + 4;
         if (odd(n / 4))
             tprint_esc("protected");
@@ -609,7 +621,7 @@ void print_cmd_chr(quarterword cmd, halfword chr_code)
     case assign_glue_cmd:
     case assign_mu_glue_cmd:
         if (chr_code < skip_base) {
-            print_skip_param(chr_code - glue_base);
+            prim_cmd_chr(cmd, chr_code);
         } else if (chr_code < mu_skip_base) {
             tprint_esc("skip");
             print_int(chr_code - skip_base);
@@ -628,7 +640,7 @@ void print_cmd_chr(quarterword cmd, halfword chr_code)
         break;
     case assign_int_cmd:
         if (chr_code < count_base) {
-            print_param(chr_code - int_base);
+            prim_cmd_chr(cmd, chr_code);
         } else {
             tprint_esc("count");
             print_int(chr_code - count_base);
@@ -640,7 +652,7 @@ void print_cmd_chr(quarterword cmd, halfword chr_code)
         break;
     case assign_dimen_cmd:
         if (chr_code < scaled_base) {
-            print_length_param(chr_code - dimen_base);
+            prim_cmd_chr(cmd, chr_code);
         } else {
             tprint_esc("dimen");
             print_int(chr_code - scaled_base);
